@@ -15,11 +15,13 @@
 namespace vm {
 
 // Arrays are vectors with push and pop functions.
-class array : public mem::deque<item>, public gc {
+class array : public mem::vector<item> {
 bool cycle;  
 public:
+  array() {}
+  
   array(size_t n)
-    : mem::deque<item>(n), cycle(false)
+    : mem::vector<item>(n), cycle(false)
   {}
 
   void push(item i)
@@ -35,7 +37,7 @@ public:
   }
 
   template <typename T>
-  T read(size_t i)
+  T read(size_t i) const
   {
     return get<T>((*this)[i]);
   }
@@ -44,7 +46,7 @@ public:
     cycle=b;
   }
   
-  bool cyclic() {
+  bool cyclic() const {
     return cycle;
   }
 };
@@ -56,7 +58,7 @@ inline T read(array *a, size_t i)
 }
 
 template <typename T>
-inline T read(array &a, size_t i)
+inline T read(const array &a, size_t i)
 {
   return a.array::read<T>(i);
 }
