@@ -63,7 +63,7 @@ void opArray(vm::stack *s)
 }
 
 template<class T>
-void arrayNegate(vm::stack *s)
+inline void arrayNegate(vm::stack *s)
 {
   array *a=pop<array*>(s);
   size_t size=checkArray(a);
@@ -73,6 +73,17 @@ void arrayNegate(vm::stack *s)
   s->push(c);
 }
 
+template<>
+inline void arrayNegate<Int>(vm::stack *s)
+{
+  array *a=pop<array*>(s);
+  size_t size=checkArray(a);
+  array *c=new array(size);
+  for(size_t i=0; i < size; i++)
+    (*c)[i]=Negate(read<Int>(a,i),i);
+  s->push(c);
+}
+
 template<class T>
 void sumArray(vm::stack *s)
 {
@@ -201,15 +212,15 @@ void searchArray(vm::stack *s)
 {
   T key=pop<T>(s);
   array *a=pop<array*>(s);
-  int size=(int) a->size();
+  Int size=(Int) a->size();
   if(size == 0) {s->push(0); return;}
   if(key < read<T>(a,0)) {s->push(-1); return;}
-  int u=size-1;
+  Int u=size-1;
   if(key >= read<T>(a,u)) {s->push(u); return;}
-  int l=0;
+  Int l=0;
 	
   while (l < u) {
-    int i=(l+u)/2;
+    Int i=(l+u)/2;
     if(read<T>(a,i) <= key && key < read<T>(a,i+1)) {s->push(i); return;}
     if(key < read<T>(a,i)) u=i;
     else l=i+1;
@@ -265,7 +276,7 @@ void writeArray(vm::stack *s)
   
   size_t asize=checkArray(a);
   size_t Asize=checkArray(A);
-  if(f->Standard()) camp::Stdout.resetlines();
+  if(f->Standard()) interact::lines=0;
   else if(!f->isOpen()) return;
   try {
     if(S != "") {f->write(S); f->writeline();}
@@ -276,8 +287,9 @@ void writeArray(vm::stack *s)
       cont=false;
       bool first=true;
       if(i < asize) {
+	T v=read<T>(a,i);
 	if(defaultfile) cout << i << ":\t";
-	f->write(read<T>(a,i)); cont=true;
+	f->write(v); cont=true;
 	first=false;
       }
       unsigned count=0;
@@ -311,7 +323,7 @@ void writeArray2(vm::stack *s)
   camp::file *f=pop<camp::file*>(s,&camp::Stdout);
   
   size_t size=checkArray(a);
-  if(f->Standard()) camp::Stdout.resetlines();
+  if(f->Standard()) interact::lines=0;
   else if(!f->isOpen()) return;
   
   try {
@@ -336,7 +348,7 @@ void writeArray3(vm::stack *s)
   camp::file *f=pop<camp::file*>(s,&camp::Stdout);
   
   size_t size=checkArray(a);
-  if(f->Standard()) camp::Stdout.resetlines();
+  if(f->Standard()) interact::lines=0;
   else if(!f->isOpen()) return;
   
   try {
@@ -360,16 +372,14 @@ void writeArray3(vm::stack *s)
   f->flush();
 }
 
-template <double (*func)(double)>
-void realArrayFunc(vm::stack *s) 
+template <class T, class S, T (*func)(S)>
+void arrayFunc(vm::stack *s) 
 {
   array *a=pop<array*>(s);
   size_t size=checkArray(a);
   array *c=new array(size);
-  for(size_t i=0; i < size; i++) {
-    double x=read<double>(a,i);
-    (*c)[i]=func(x);
-  }
+  for(size_t i=0; i < size; i++)
+    (*c)[i]=func(read<S>(a,i));
   s->push(c);
 }
 
