@@ -7,10 +7,6 @@
 
 #include <cmath>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "builtin.h"
 #include "entry.h"
 #include "runtime.h"
@@ -33,6 +29,7 @@
   
 using namespace types;
 using namespace camp;
+using namespace vm;  
 
 namespace trans {
 using camp::transform;
@@ -65,10 +62,9 @@ void base_tenv(tenv &te)
 const formal noformal(0);  
 
 void addFunc(venv &ve, access *a, ty *result, symbol *id,
-	     formal f1=noformal, formal f2=noformal,
-	     formal f3=noformal, formal f4=noformal,
-	     formal f5=noformal, formal f6=noformal,
-	     formal f7=noformal, formal f8=noformal)
+	     formal f1=noformal, formal f2=noformal, formal f3=noformal,
+	     formal f4=noformal, formal f5=noformal, formal f6=noformal,
+	     formal f7=noformal, formal f8=noformal, formal f9=noformal)
 {
   function *fun = new function(result);
 
@@ -80,6 +76,7 @@ void addFunc(venv &ve, access *a, ty *result, symbol *id,
   if (f6.t) fun->add(f6);
   if (f7.t) fun->add(f7);
   if (f8.t) fun->add(f8);
+  if (f9.t) fun->add(f9);
 
   // NOTE: If the function is a field, we should encode the defining record in
   // the entry
@@ -90,11 +87,12 @@ void addFunc(venv &ve, access *a, ty *result, symbol *id,
 
 // Add a function with one or more default arguments.
 void addFunc(venv &ve, bltin f, ty *result, const char *name, 
-	     formal f1, formal f2, formal f3, formal f4,
-	     formal f5, formal f6, formal f7, formal f8)
+	     formal f1, formal f2, formal f3,
+	     formal f4, formal f5, formal f6,
+	     formal f7, formal f8, formal f9)
 {
   access *a = new bltinAccess(f);
-  addFunc(ve,a,result,symbol::trans(name),f1,f2,f3,f4,f5,f6,f7,f8);
+  addFunc(ve,a,result,symbol::trans(name),f1,f2,f3,f4,f5,f6,f7,f8,f9);
 }
   
 void addFunc(venv &ve, access *a, ty *result, const char *name, formal f1)
@@ -104,10 +102,9 @@ void addFunc(venv &ve, access *a, ty *result, const char *name, formal f1)
 
 // Add a rest function with zero or more default/explicit arguments.
 void addRestFunc(venv &ve, bltin f, ty *result, const char *name, formal frest,
-		 formal f1=noformal, formal f2=noformal,
-		 formal f3=noformal, formal f4=noformal,
-		 formal f5=noformal, formal f6=noformal,
-		 formal f7=noformal, formal f8=noformal)
+		 formal f1=noformal, formal f2=noformal, formal f3=noformal,
+		 formal f4=noformal, formal f5=noformal, formal f6=noformal,
+		 formal f7=noformal, formal f8=noformal, formal f9=noformal)
 {
   access *a = new bltinAccess(f);
   function *fun = new function(result);
@@ -120,6 +117,7 @@ void addRestFunc(venv &ve, bltin f, ty *result, const char *name, formal frest,
   if (f6.t) fun->add(f6);
   if (f7.t) fun->add(f7);
   if (f8.t) fun->add(f8);
+  if (f9.t) fun->add(f9);
 
   if (frest.t) fun->addRest(frest);
 
@@ -137,7 +135,8 @@ template<double (*fcn)(double)>
 void addRealFunc(venv &ve, const char* name)
 {
   addFunc(ve, realReal<fcn>, primReal(), name, formal(primReal(),"x"));
-  addFunc(ve, realArrayFunc<fcn>, realArray(), name, formal(realArray(),"a"));
+  addFunc(ve, arrayFunc<double,double,fcn>, realArray(), name,
+	  formal(realArray(),"a"));
 }
 
 #define addRealFunc(fcn) addRealFunc<fcn>(ve, #fcn);
@@ -169,7 +168,7 @@ inline void checkGSLerror()
 template <double (*func)(double)>
 void realRealGSL(vm::stack *s) 
 {
-  double x=vm::pop<double>(s);
+  double x=pop<double>(s);
   s->push(func(x));
   checkGSLerror();
 }
@@ -177,7 +176,7 @@ void realRealGSL(vm::stack *s)
 template <double (*func)(double, gsl_mode_t)>
 void realRealDOUBLE(vm::stack *s) 
 {
-  double x=vm::pop<double>(s);
+  double x=pop<double>(s);
   s->push(func(x,GSL_PREC_DOUBLE));
   checkGSLerror();
 }
@@ -185,46 +184,43 @@ void realRealDOUBLE(vm::stack *s)
 template <double (*func)(double, double, gsl_mode_t)>
 void realRealRealDOUBLE(vm::stack *s) 
 {
-  double y=vm::pop<double>(s);
-  double x=vm::pop<double>(s);
+  double y=pop<double>(s);
+  double x=pop<double>(s);
   s->push(func(x,y,GSL_PREC_DOUBLE));
   checkGSLerror();
 }
 
-template <double (*func)(unsigned int)>
+template <double (*func)(unsigned)>
 void realIntGSL(vm::stack *s) 
 {
-  int n=vm::pop<int>(s);
-  if(n < 0) n=0;
-  s->push(func(n));
+  s->push(func(unsignedcast(pop<Int>(s))));
   checkGSLerror();
 }
 
 template <double (*func)(int, double)>
 void realIntRealGSL(vm::stack *s) 
 {
-  double x=vm::pop<double>(s);
-  int n=vm::pop<int>(s);
-  s->push(func(n,x));
+  double x=pop<double>(s);
+  
+  s->push(func(intcast(pop<Int>(s)),x));
   checkGSLerror();
 }
 
 template <double (*func)(double, double)>
 void realRealRealGSL(vm::stack *s) 
 {
-  double x=vm::pop<double>(s);
-  double n=vm::pop<double>(s);
+  double x=pop<double>(s);
+  double n=pop<double>(s);
   s->push(func(n,x));
   checkGSLerror();
 }
 
-template <double (*func)(double, unsigned int)>
+template <double (*func)(double, unsigned)>
 void realRealIntGSL(vm::stack *s) 
 {
-  int n=vm::pop<int>(s);
-  double x=vm::pop<double>(s);
-  if(n < 0) n=0;
-  s->push(func(x,n));
+  Int n=pop<Int>(s);
+  double x=pop<double>(s);
+  s->push(func(x,unsignedcast(n)));
   checkGSLerror();
 }
 
@@ -251,7 +247,7 @@ void addGSLDOUBLE2Func(const char* name)
 	  formal(primReal(),"phi"), formal(primReal(),"k"));
 }
 
-template<double (*fcn)(unsigned int)>
+template<double (*fcn)(unsigned)>
 void addGSLIntFunc(const char* name)
 {
   addFunc(GSLModule->e.ve, realIntGSL<fcn>, primReal(), name,
@@ -272,7 +268,7 @@ void addGSLRealRealFunc(const char* name)
 	  formal(primReal(),"nu"), formal(primReal(),"x"));
 }
 
-template<double (*fcn)(double, unsigned int)>
+template<double (*fcn)(double, unsigned)>
 void addGSLRealIntFunc(const char* name)
 {
   addFunc(GSLModule->e.ve, realRealIntGSL<fcn>, primReal(), name, 
@@ -335,7 +331,7 @@ void addConstant(venv &ve, T value, ty *t, const char *name,
 // operation, and no functions are called.
 identAccess id;
 
-function *intRealFunction()
+function *IntRealFunction()
 {
   return new function(primInt(),primReal());
 }
@@ -353,7 +349,7 @@ function *voidFileFunction()
 void addInitializers(venv &ve)
 {
   addInitializer(ve, primBoolean(), boolFalse);
-  addInitializer(ve, primInt(), intZero);
+  addInitializer(ve, primInt(), IntZero);
   addInitializer(ve, primReal(), realZero);
 
   addInitializer(ve, primString(), emptyString);
@@ -364,24 +360,24 @@ void addInitializers(venv &ve)
   addInitializer(ve, primPath(), nullPath);
   addInitializer(ve, primPen(), newPen);
   addInitializer(ve, primPicture(), newPicture);
-  addInitializer(ve, primFile(), standardOut);
+  addInitializer(ve, primFile(), nullFile);
 }
 
 void addCasts(venv &ve)
 {
-  addExplicitCast(ve, primString(), primInt(), stringCast<int>);
+  addExplicitCast(ve, primString(), primInt(), stringCast<Int>);
   addExplicitCast(ve, primString(), primReal(), stringCast<double>);
   addExplicitCast(ve, primString(), primPair(), stringCast<pair>);
   addExplicitCast(ve, primString(), primTriple(), stringCast<triple>);
-  addExplicitCast(ve, primInt(), primString(), castString<int>);
+  addExplicitCast(ve, primInt(), primString(), castString<Int>);
   addExplicitCast(ve, primReal(), primString(), castString<double>);
   addExplicitCast(ve, primPair(), primString(), castString<pair>);
   addExplicitCast(ve, primTriple(), primString(), castString<triple>);
 
   addExplicitCast(ve, primInt(), primReal(), castDoubleInt);
 
-  addCast(ve, primReal(), primInt(), cast<int,double>);
-  addCast(ve, primPair(), primInt(), cast<int,pair>);
+  addCast(ve, primReal(), primInt(), cast<Int,double>);
+  addCast(ve, primPair(), primInt(), cast<Int,pair>);
   addCast(ve, primPair(), primReal(), cast<double,pair>);
   
   addCast(ve, primPath(), primPair(), cast<pair,path>);
@@ -390,12 +386,12 @@ void addCasts(venv &ve)
   addCast(ve, primPath(), primGuide(), guideToPath);
 
   addCast(ve, primFile(), primNull(), nullFile);
-
-  // Vectorized casts.
-  addExplicitCast(ve, intArray(), realArray(), arrayToArray<double,int>);
   
-  addCast(ve, realArray(), intArray(), arrayToArray<int,double>);
-  addCast(ve, pairArray(), intArray(), arrayToArray<int,pair>);
+  // Vectorized casts.
+  addExplicitCast(ve, IntArray(), realArray(), arrayToArray<double,Int>);
+  
+  addCast(ve, realArray(), IntArray(), arrayToArray<Int,double>);
+  addCast(ve, pairArray(), IntArray(), arrayToArray<Int,pair>);
   addCast(ve, pairArray(), realArray(), arrayToArray<double,pair>);
 }
 
@@ -496,6 +492,14 @@ struct maxbound {
   }
 };
 
+inline double abs(pair z) {
+    return z.length();
+}
+
+inline double abs(triple v) {
+    return v.length();
+}
+
 template<class T, template <class S> class op>
 void addBinOps(venv &ve, ty *t1, ty *t2, ty *t3, ty *t4, const char *name)
 {
@@ -576,7 +580,7 @@ void addArrayOps(venv &ve, types::array *t)
   case 3:
     addFunc(ve, run::array3Copy, t, "copy", formal(t, "a"));
     addFunc(ve, run::array3Transpose, t, "transpose", formal(t, "a"),
-	    formal(intArray(),"perm"));
+	    formal(IntArray(),"perm"));
     break;
   default:
     break;
@@ -611,7 +615,7 @@ void addOperators(venv &ve)
   
   addUnorderedOps<bool>(ve,primBoolean(),boolArray(),boolArray2(),
 			boolArray3());
-  addOps<int>(ve,primInt(),intArray(),intArray2(),intArray3(),false);
+  addOps<Int>(ve,primInt(),IntArray(),IntArray2(),IntArray3(),false);
   addOps<double>(ve,primReal(),realArray(),realArray2(),realArray3());
   addOps<pair>(ve,primPair(),pairArray(),pairArray2(),pairArray3());
   addBasicOps<triple>(ve,primTriple(),tripleArray(),tripleArray2(),
@@ -632,21 +636,26 @@ void addOperators(venv &ve)
   addBinOps<triple,maxbound>(ve,primTriple(),tripleArray(),tripleArray2(),
 			     tripleArray3(),"maxbound");
   
-  addFunc(ve,binaryOp<int,divide>,primReal(),"/",
-	  formal(primInt(),"a"),formal(primInt(),"b"));
-  addFunc(ve,arrayOp<int,divide>,realArray(),"/",
-	  formal(intArray(),"a"),formal(primInt(),"b"));
-  addFunc(ve,opArray<int,divide>,realArray(),"/",
-	  formal(primInt(),"a"),formal(intArray(),"b"));
-  addFunc(ve,arrayArrayOp<int,divide>,realArray(),"/",
-	  formal(intArray(),"a"),formal(intArray(),"b"));
+  addFunc(ve,arrayFunc<double,pair,abs>,realArray(),"abs",
+	  formal(pairArray(),"a"));
+  addFunc(ve,arrayFunc<double,triple,abs>,realArray(),"abs",
+	  formal(tripleArray(),"a"));
   
-  addOrderedOps<int>(ve,primInt(),intArray(),intArray2(),intArray3());
+  addFunc(ve,binaryOp<Int,divide>,primReal(),"/",
+	  formal(primInt(),"a"),formal(primInt(),"b"));
+  addFunc(ve,arrayOp<Int,divide>,realArray(),"/",
+	  formal(IntArray(),"a"),formal(primInt(),"b"));
+  addFunc(ve,opArray<Int,divide>,realArray(),"/",
+	  formal(primInt(),"a"),formal(IntArray(),"b"));
+  addFunc(ve,arrayArrayOp<Int,divide>,realArray(),"/",
+	  formal(IntArray(),"a"),formal(IntArray(),"b"));
+  
+  addOrderedOps<Int>(ve,primInt(),IntArray(),IntArray2(),IntArray3());
   addOrderedOps<double>(ve,primReal(),realArray(),realArray2(),realArray3());
   addOrderedOps<string>(ve,primString(),stringArray(),stringArray2(),
 			stringArray3());
   
-  addOps<int,mod>(ve,primInt(),"%",intArray());
+  addOps<Int,mod>(ve,primInt(),"%",IntArray());
   addOps<double,mod>(ve,primReal(),"%",realArray());
 }
 
@@ -659,7 +668,7 @@ dummyRecord *createDummyRecord(venv &ve, const char *name)
 }
 
 double identity(double x) {return x;}
-double pow10(double x) {return pow(10.0,x);}
+double pow10(double x) {return run::pow(10.0,x);}
 
 // NOTE: We should move all of these into a "builtin" module.
 void base_venv(venv &ve)
@@ -742,8 +751,8 @@ void base_venv(venv &ve)
   addFunc(ve,arrayFunction,realArray(),"map",
 	  formal(realPairFunction(),"f"),
 	  formal(pairArray(),"a"));
-  addFunc(ve,arrayFunction,intArray(),"map",
-	  formal(intRealFunction(),"f"),
+  addFunc(ve,arrayFunction,IntArray(),"map",
+	  formal(IntRealFunction(),"f"),
 	  formal(realArray(),"a"));
   
 #ifdef HAVE_LIBFFTW3
@@ -751,13 +760,14 @@ void base_venv(venv &ve)
 	  formal(primInt(),"sign",true));
 #endif
 
-  addConstant<int>(ve, INT_MAX, primInt(), "intMax");
+  addConstant<Int>(ve, Int_MAX, primInt(), "intMax");
+  addConstant<Int>(ve, Int_MIN, primInt(), "intMin");
   addConstant<double>(ve, HUGE_VAL, primReal(), "inf");
   addConstant<double>(ve, DBL_MAX, primReal(), "realMax");
   addConstant<double>(ve, DBL_MIN, primReal(), "realMin");
   addConstant<double>(ve, DBL_EPSILON, primReal(), "realEpsilon");
-  addConstant<int>(ve, DBL_DIG, primInt(), "realDigits");
-  addConstant<int>(ve, RAND_MAX, primInt(), "randMax");
+  addConstant<Int>(ve, DBL_DIG, primInt(), "realDigits");
+  addConstant<Int>(ve, RAND_MAX, primInt(), "randMax");
   addConstant<double>(ve, PI, primReal(), "pi");
 
   gen_base_venv(ve);
@@ -768,3 +778,46 @@ void base_menv(menv&)
 }
 
 } //namespace trans
+
+namespace run {
+
+void arrayDeleteHelper(vm::stack *Stack)
+{
+  array *a=pop<array *>(Stack);
+  item itj=pop(Stack);
+  item iti=pop(Stack);
+  if(isdefault(iti)) {
+    (*a).clear();
+    return;
+  }
+  Int i=get<Int>(iti);
+  Int j=isdefault(itj) ? i : get<Int>(itj);
+
+  size_t asize=checkArray(a);
+  if(a->cyclic() && asize > 0) {
+    if(j-i+1 >= (Int) asize) {
+      (*a).clear();
+      return;
+    }
+    i=imod(i,asize);
+    j=imod(j,asize);
+    if(j >= i) 
+      (*a).erase((*a).begin()+i,(*a).begin()+j+1);
+    else {
+      (*a).erase((*a).begin()+i,(*a).end());
+      (*a).erase((*a).begin(),(*a).begin()+j+1);
+    }
+    return;
+  }
+  
+  if(i < 0 || i >= (Int) asize || i > j || j >= (Int) asize) {
+    ostringstream buf;
+    buf << "delete called on array of length " << (Int) asize 
+	<< " with out-of-bounds index range [" << i << "," << j << "]";
+    error(buf);
+  }
+
+  (*a).erase((*a).begin()+i,(*a).begin()+j+1);
+}
+  
+}
