@@ -28,7 +28,8 @@ void addFunc(venv &ve, vm::bltin f, types::ty *result, const char *name,
 	     types::formal f1=noformal, types::formal f2=noformal,
 	     types::formal f3=noformal, types::formal f4=noformal,
 	     types::formal f5=noformal, types::formal f6=noformal,
-	     types::formal f7=noformal, types::formal f8=noformal);
+	     types::formal f7=noformal, types::formal f8=noformal,
+	     types::formal f9=noformal);
   
 // Adds standard functions for a newly added types.
 void addArrayOps(venv &ve, types::array *t);
@@ -39,4 +40,8 @@ types::record *getGSLModule();
   
 } //namespace trans
 
+namespace run {
+  void arrayDeleteHelper(vm::stack *Stack);
+}
+
 #endif //BUILTIN_H
