@@ -31,7 +31,7 @@ trans::tyEntry *ty::transAsTyEntry(coenv &e, record *where)
 }
 
 
-void nameTy::prettyprint(ostream &out, int indent)
+void nameTy::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "nameTy",indent);
 
@@ -49,7 +49,7 @@ trans::tyEntry *nameTy::transAsTyEntry(coenv &e, record *)
 }
 
 
-void dimensions::prettyprint(ostream &out, int indent)
+void dimensions::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out, indent);
   out << "dimensions (" << depth << ")\n";
@@ -58,8 +58,8 @@ void dimensions::prettyprint(ostream &out, int indent)
 types::array *dimensions::truetype(types::ty *base)
 {
   if (base->kind == ty_void) {
-    em->compiler(getPos());
-    *em << "cannot declare array of type void";
+    em.error(getPos());
+    em << "cannot declare array of type void";
   }
 
   assert(depth >= 1);
@@ -72,7 +72,7 @@ types::array *dimensions::truetype(types::ty *base)
 }
 
 
-void arrayTy::prettyprint(ostream &out, int indent)
+void arrayTy::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "arrayTy",indent);
 
@@ -80,6 +80,15 @@ void arrayTy::prettyprint(ostream &out, int indent)
   dims->prettyprint(out, indent+1);
 }
 
+// NOTE: Can this be merged with trans somehow?
+void arrayTy::addOps(coenv &e, record *r)
+{
+  types::array *t=dynamic_cast<types::array *>(trans(e, true));
+  e.e.addArrayOps(t);
+  if (r)
+    r->e.addArrayOps(t);
+}
+  
 types::ty *arrayTy::trans(coenv &e, bool tacit)
 {
   types::ty *ct = cell->trans(e, tacit);
@@ -88,11 +97,24 @@ types::ty *arrayTy::trans(coenv &e, bool tacit)
   types::array *t = dims->truetype(ct);
   assert(t);
   
-  e.e.addArrayOps(t);
-  
   return t;
 }
 
+tyEntryTy::tyEntryTy(position pos, types::ty *t)
+  : ty(pos), ent(new trans::tyEntry(t, 0, 0, position()))
+{
+}
+
+void tyEntryTy::prettyprint(ostream &out, Int indent)
+{
+  prettyindent(out,indent);
+  out << "tyEntryTy: " << *(ent->t) << "\n";
+}
+
+types::ty *tyEntryTy::trans(coenv &, bool) {
+  return ent->t;
+}
+
 
 vm::lambda *runnable::transAsCodelet(coenv &e)
 {
@@ -103,13 +125,13 @@ vm::lambda *runnable::transAsCodelet(coenv &e)
 }
 
 
-void block::prettystms(ostream &out, int indent)
+void block::prettystms(ostream &out, Int indent)
 {
   for (list<runnable *>::iterator p = stms.begin(); p != stms.end(); ++p)
     (*p)->prettyprint(out, indent);
 }
 
-void block::prettyprint(ostream &out, int indent)
+void block::prettyprint(ostream &out, Int indent)
 {
   prettyname(out,"block",indent);
   prettystms(out, indent+1);
@@ -155,7 +177,7 @@ record *block::transAsFile(genv& ge, symbol *id)
     autoplainRunnable()->transAsField(ce, r);
   }
   transAsRecordBody(ce, r);
-  em->sync();
+  em.sync();
   
   return r;
 }
@@ -171,13 +193,13 @@ bool block::returns() {
 }
   
 
-void dec::prettyprint(ostream &out, int indent)
+void dec::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "dec", indent);
 }
 
 
-void modifierList::prettyprint(ostream &out, int indent)
+void modifierList::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out,indent);
   out << "modifierList (";
@@ -225,8 +247,8 @@ bool modifierList::staticSet()
 modifier modifierList::getModifier()
 {
   if (mods.size() > 1) {
-    em->error(getPos());
-    *em << "too many modifiers";
+    em.error(getPos());
+    em << "too many modifiers";
   }
 
   assert(staticSet());
@@ -236,15 +258,15 @@ modifier modifierList::getModifier()
 permission modifierList::getPermission()
 {
   if (perms.size() > 1) {
-    em->error(getPos());
-    *em << "too many modifiers";
+    em.error(getPos());
+    em << "too many modifiers";
   }
 
   return perms.empty() ? DEFAULT_PERM : perms.front();
 }
 
 
-void modifiedRunnable::prettyprint(ostream &out, int indent)
+void modifiedRunnable::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "modifierRunnable",indent);
 
@@ -256,8 +278,8 @@ void modifiedRunnable::transAsField(coenv &e, record *r)
 {
   if (mods->staticSet()) {
     if (e.c.isTopLevel()) {
-      em->warning(getPos());
-      *em << "static modifier is meaningless at top level";
+      em.warning(getPos());
+      em << "static modifier is meaningless at top level";
     }
     e.c.pushModifier(mods->getModifier());
   }
@@ -265,8 +287,8 @@ void modifiedRunnable::transAsField(coenv &e, record *r)
   permission p = mods->getPermission();
 #if 0 // This is innocuous 
   if (p != DEFAULT_PERM && (!r || !body->allowPermissions())) {
-    em->warning(pos);
-    *em << "permission modifier is meaningless";
+    em.warning(pos);
+    em << "permission modifier is meaningless";
   }
 #endif  
   e.c.setPermission(p);
@@ -279,7 +301,7 @@ void modifiedRunnable::transAsField(coenv &e, record *r)
 }
 
 
-void decidstart::prettyprint(ostream &out, int indent)
+void decidstart::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out, indent);
   out << "decidstart '" << *id << "'\n";
@@ -311,7 +333,7 @@ void decidstart::addOps(types::ty *base, coenv &e, record *r)
 }
 
 
-void fundecidstart::prettyprint(ostream &out, int indent)
+void fundecidstart::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out, indent);
   out << "fundecidstart '" << *id << "'\n";
@@ -345,6 +367,8 @@ void fundecidstart::addOps(types::ty *base, coenv &e, record *r)
 {
   decidstart::addOps(base, e, r);
 
+  params->addOps(e, r);
+
   types::function *ft=dynamic_cast<types::function *>(getType(base, e, true));
   assert(ft);
 
@@ -354,7 +378,7 @@ void fundecidstart::addOps(types::ty *base, coenv &e, record *r)
 }
 
 
-void decid::prettyprint(ostream &out, int indent)
+void decid::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "decid",indent);
 
@@ -378,8 +402,18 @@ varEntry *makeVarEntry(position pos, coenv &e, record *r, types::ty *t) {
   return makeVarEntryWhere(e, r, t, r, pos);
 }
 
+
+// Defined in constructor.cc.
+bool definesImplicitConstructor(coenv &e, record *r, varEntry *v, symbol *id);
+void addConstructorFromInitializer(position pos, coenv &e, record *r,
+                                   varEntry *init);
+
 void addVar(coenv &e, record *r, varEntry *v, symbol *id)
 {
+  // Test for 'operator init' definitions that implicitly define constructors:
+  if (definesImplicitConstructor(e, r, v, id))
+    addConstructorFromInitializer(position(), e, r, v);
+
   // Add to the record so it can be accessed when qualified; add to the
   // environment so it can be accessed unqualified in the scope of the
   // record definition.
@@ -388,9 +422,10 @@ void addVar(coenv &e, record *r, varEntry *v, symbol *id)
   e.e.addVar(id, v);
 }
 
-void initializeVar(position pos, coenv &e, record *,
-                   varEntry *v, types::ty *t, varinit *init)
+void initializeVar(position pos, coenv &e, varEntry *v, varinit *init)
 {
+  types::ty *t=v->getType();
+
   if (init)
     init->transToType(e, t);
   else {
@@ -407,14 +442,14 @@ void createVar(position pos, coenv &e, record *r,
 {
   varEntry *v=makeVarEntry(pos, e, r, t);
   addVar(e, r, v, id);
-  initializeVar(pos, e, r, v, t, init);
+  initializeVar(pos, e, v, init);
 }
 
 void createVarOutOfOrder(position pos, coenv &e, record *r,
                          symbol *id, types::ty *t, varinit *init)
 {
   varEntry *v=makeVarEntry(pos, e, r, t);
-  initializeVar(pos, e, r, v, t, init);
+  initializeVar(pos, e, v, init);
   addVar(e, r, v, id);
 }
 
@@ -436,8 +471,8 @@ void decid::transAsField(coenv &e, record *r, types::ty *base)
   types::ty *t = start->getType(base, e);
   assert(t);
   if (t->kind == ty_void) {
-    em->compiler(getPos());
-    *em << "cannot declare variable of type void";
+    em.error(getPos());
+    em << "cannot declare variable of type void";
   }
 
   start->addOps(base, e, r);
@@ -451,8 +486,8 @@ void decid::transAsTypedefField(coenv &e, trans::tyEntry *base, record *r)
   assert(ent && ent->t);
 
   if (init) {
-    em->error(getPos());
-    *em << "type definition cannot have initializer";
+    em.error(getPos());
+    em << "type definition cannot have initializer";
   }
    
   start->addOps(base->t, e, r);
@@ -461,7 +496,7 @@ void decid::transAsTypedefField(coenv &e, trans::tyEntry *base, record *r)
 }
 
 
-void decidlist::prettyprint(ostream &out, int indent)
+void decidlist::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "decidlist",indent);
 
@@ -482,7 +517,7 @@ void decidlist::transAsTypedefField(coenv &e, trans::tyEntry *base, record *r)
 }
 
 
-void vardec::prettyprint(ostream &out, int indent)
+void vardec::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "vardec",indent);
 
@@ -492,6 +527,7 @@ void vardec::prettyprint(ostream &out, int indent)
 
 void vardec::transAsTypedefField(coenv &e, record *r)
 {
+  base->addOps(e, r);
   decs->transAsTypedefField(e, base->transAsTyEntry(e, r), r);
 }
 
@@ -509,8 +545,8 @@ public:
     : exp(pos), imp(imp), ft(new function(imp,primString())) {}
 
   types::ty *trans(coenv &) {
-    em->compiler(getPos());
-    *em << "trans called for loadModuleExp";
+    em.compiler(getPos());
+    em << "trans called for loadModuleExp";
     return primError();
   }
 
@@ -535,9 +571,9 @@ varEntry *accessModule(position pos, coenv &e, record *r, symbol *id)
 {
   record *imp=e.e.getModule(id, (string)*id);
   if (!imp) {
-    em->error(pos);
-    *em << "could not load module '" << *id << "'";
-    em->sync();
+    em.error(pos);
+    em << "could not load module '" << *id << "'";
+    em.sync();
     return 0;
   }
   else {
@@ -549,13 +585,13 @@ varEntry *accessModule(position pos, coenv &e, record *r, symbol *id)
     // The varEntry should have whereDefined()==0 as it is not defined inside
     // the record r.
     varEntry *v=makeVarEntryWhere(e, r, imp, 0, pos);
-    initializeVar(pos, e, r, v, imp, &init);
+    initializeVar(pos, e, v, &init);
     return v;
   }
 }
 
 
-void idpair::prettyprint(ostream &out, int indent)
+void idpair::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out, indent);
   out << "idpair (" << "'" << *src << "' as " << *dest << ")\n";
@@ -578,13 +614,13 @@ void idpair::transAsUnravel(coenv &e, record *r,
   if (r)
     r->e.add(src, dest, source, qualifier, e.c);
   if (!e.e.add(src, dest, source, qualifier, e.c)) {
-    em->error(getPos());
-    *em << "no matching types or fields of name '" << *src << "'";
+    em.error(getPos());
+    em << "no matching types or fields of name '" << *src << "'";
   }
 }
 
 
-void idpairlist::prettyprint(ostream &out, int indent)
+void idpairlist::prettyprint(ostream &out, Int indent)
 {
   for (list<idpair *>::iterator p=base.begin();
        p != base.end();
@@ -611,14 +647,14 @@ void idpairlist::transAsUnravel(coenv &e, record *r,
 
 idpairlist * const WILDCARD = 0;
 
-void accessdec::prettyprint(ostream &out, int indent)
+void accessdec::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "accessdec", indent);
   base->prettyprint(out, indent+1);
 }
 
 
-void fromdec::prettyprint(ostream &out, int indent)
+void fromdec::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "fromdec", indent);
   fields->prettyprint(out, indent+1);
@@ -626,27 +662,20 @@ void fromdec::prettyprint(ostream &out, int indent)
 
 void fromdec::transAsField(coenv &e, record *r)
 {
-  varEntry *v=getQualifier(e,r);
-  if (v) {
-    record *qt=dynamic_cast<record *>(v->getType());
-    if (!qt) {
-      em->error(getPos());
-      *em << "qualifier is not a record";
-    }
-    else {
-      if (fields==WILDCARD) {
-        if (r)
-          r->e.add(qt->e, v, e.c);
-        e.e.add(qt->e, v, e.c);
-      }
-      else
-        fields->transAsUnravel(e, r, qt->e, v);
+  qualifier q=getQualifier(e,r);
+  if (q.t) {
+    if (fields==WILDCARD) {
+      if (r)
+        r->e.add(q.t->e, q.v, e.c);
+      e.e.add(q.t->e, q.v, e.c);
     }
+    else
+      fields->transAsUnravel(e, r, q.t->e, q.v);
   }
 }
 
 
-void unraveldec::prettyprint(ostream &out, int indent)
+void unraveldec::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "unraveldec", indent);
   id->prettyprint(out, indent+1);
@@ -654,15 +683,19 @@ void unraveldec::prettyprint(ostream &out, int indent)
   if(f) f->prettyprint(out, indent+1);
 }
 
-varEntry *unraveldec::getQualifier(coenv &e, record *)
+fromdec::qualifier unraveldec::getQualifier(coenv &e, record *)
 {
-  // To report errors.
-  id->getType(e, false);
+  // getType is where errors in the qualifier are reported.
+  record *qt=dynamic_cast<record *>(id->getType(e, false));
+  if (!qt) {
+    em.error(getPos());
+    em << "qualifier is not a record";
+  }
 
-  return id->getVarEntry(e);
+  return qualifier(qt,id->getVarEntry(e));
 }
 
-void fromaccessdec::prettyprint(ostream &out, int indent)
+void fromaccessdec::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out, indent);
   out << "fromaccessdec '" << *id << "'\n";
@@ -670,18 +703,28 @@ void fromaccessdec::prettyprint(ostream &out, int indent)
   if(f) f->prettyprint(out, indent+1);
 }
 
-varEntry *fromaccessdec::getQualifier(coenv &e, record *r)
+fromdec::qualifier fromaccessdec::getQualifier(coenv &e, record *r)
 {
-  return accessModule(getPos(), e, r, id);
+  varEntry *v=accessModule(getPos(), e, r, id);
+  if (v) {
+    record *qt=dynamic_cast<record *>(v->getType());
+    if (!qt) {
+      em.compiler(getPos());
+      em << "qualifier is not a record";
+    }
+    return qualifier(qt, v);
+  }
+  else
+    return qualifier(0,0);
 }
 
-void importdec::prettyprint(ostream &out, int indent)
+void importdec::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "importdec", indent);
   base.prettyprint(out, indent+1);
 }
 
-void includedec::prettyprint(ostream &out, int indent)
+void includedec::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out, indent);
   out << "includedec ('" << filename << "')\n";
@@ -689,22 +732,23 @@ void includedec::prettyprint(ostream &out, int indent)
 
 void includedec::loadFailed(coenv &)
 {
-  em->warning(getPos());
-  *em << "could not parse file of name '" << filename << "'";
-  em->sync();
+  em.warning(getPos());
+  em << "could not parse file of name '" << filename << "'";
+  em.sync();
 }
 
 void includedec::transAsField(coenv &e, record *r)
 {
   file *ast = parser::parseFile(filename,"Including");
-  em->sync();
+  em.sync();
 
-  // The runnables will be run, one at a time, without any additional scoping.
+  // The runnables will be translated, one at a time, without any additional
+  // scoping.
   ast->transAsField(e, r);
 }
 
 
-void typedec::prettyprint(ostream &out, int indent)
+void typedec::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "typedec",indent);
 
@@ -712,7 +756,7 @@ void typedec::prettyprint(ostream &out, int indent)
 }
 
 
-void recorddec::prettyprint(ostream &out, int indent)
+void recorddec::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out, indent);
   out << "structdec '" << *id << "'\n";
@@ -737,6 +781,12 @@ void recorddec::transRecordInitializer(coenv &e, record *parent)
   init.transAsField(e, parent);
 }
 
+void recorddec::addPostRecordEnvironment(coenv &e, record *r, record *parent) {
+  if (parent)
+    parent->e.add(r->postdefenv, 0, e.c);
+  e.e.add(r->postdefenv, 0, e.c);
+}
+
 void recorddec::transAsField(coenv &e, record *parent)
 {
   record *r = parent ? parent->newRecord(id, e.c.isStatic()) :
@@ -757,6 +807,11 @@ void recorddec::transAsField(coenv &e, record *parent)
   // variable of the type of the record is initialized to a new instance by
   // default.
   transRecordInitializer(e, parent);
+
+  // Add types and variables defined during the record that should be added to
+  // the enclosing environment.  These are the implicit constructors defined by
+  // "operator init".
+  addPostRecordEnvironment(e, r, parent);
 }  
 
 runnable *autoplainRunnable() {
