@@ -48,7 +48,11 @@ public:
   ty(position pos)
     : absyn(pos) {}
 
-  virtual void prettyprint(ostream &out, int indent) = 0;
+  virtual void prettyprint(ostream &out, Int indent) = 0;
+
+  // If we introduced a new type, automatically add corresponding functions for
+  // that type.  
+  virtual void addOps(coenv &, record *) {}
 
   // Returns the internal representation of the type.  This method can
   // be called by exp::getType which does not report errors, so tacit is
@@ -68,7 +72,7 @@ public:
   nameTy(name *id)
     : ty(id->getPos()), id(id) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e, bool tacit = false);
   trans::tyEntry *transAsTyEntry(coenv &e, record *where);
@@ -80,7 +84,7 @@ public:
   dimensions(position pos)
     : absyn(pos), depth(1) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   void increase()
     { depth++; }
@@ -103,11 +107,31 @@ public:
   arrayTy(name *id, dimensions *dims)
     : ty(dims->getPos()), cell(new nameTy(id)), dims(dims) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
+
+  void addOps(coenv &e, record *r);
 
   types::ty *trans(coenv &e, bool tacit = false);
 };
 
+// Similar to varEntryExp, this helper class always translates to the same fixed
+// type.
+class tyEntryTy : public ty {
+  trans::tyEntry *ent;
+public:
+  tyEntryTy(position pos, trans::tyEntry *ent)
+    : ty(pos), ent(ent) {}
+
+  tyEntryTy(position pos, types::ty *t);
+
+  void prettyprint(ostream &out, Int indent);
+
+  types::ty *trans(coenv &e, bool tacit = false);
+  trans::tyEntry *transAsTyEntry(coenv &, record *) {
+    return ent;
+  }
+};
+
 // Runnable is anything that can be executed by the program, including
 // any declaration or statement.
 class runnable : public absyn {
@@ -115,7 +139,7 @@ public:
   runnable(position pos)
     : absyn(pos) {}
 
-  virtual void prettyprint(ostream &out, int indent) = 0;
+  virtual void prettyprint(ostream &out, Int indent) = 0;
   
   void markTrans(coenv &e)
   {
@@ -169,7 +193,7 @@ public:
   bool scope;
 
 protected:
-  void prettystms(ostream &out, int indent);
+  void prettystms(ostream &out, Int indent);
 
 public:
   block(position pos, bool scope=true)
@@ -182,7 +206,7 @@ public:
     stms.push_back(r);
   }
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   void trans(coenv &e);
 
@@ -217,7 +241,7 @@ public:
   virtual ~modifierList()
     {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   void add(trans::permission p)
   {
@@ -259,7 +283,7 @@ public:
     mods->add(perm);
   }
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   void transAsField(coenv &e, record *r);
 
@@ -277,7 +301,7 @@ public:
   decidstart(position pos, symbol *id, dimensions *dims = 0)
     : absyn(pos), id(id), dims(dims) {}
 
-  virtual void prettyprint(ostream &out, int indent);
+  virtual void prettyprint(ostream &out, Int indent);
 
   virtual types::ty *getType(types::ty *base, coenv &, bool = false);
   virtual trans::tyEntry *getTyEntry(trans::tyEntry *base, coenv &e,
@@ -304,7 +328,7 @@ public:
 		formals *params = 0)
     : decidstart(pos, id, dims), params(params) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   types::ty *getType(types::ty *base, coenv &e, bool tacit = false);
   trans::tyEntry *getTyEntry(trans::tyEntry *base, coenv &e, record *where);
@@ -322,7 +346,7 @@ public:
   decid(position pos, decidstart *start, varinit *init = 0)
     : absyn(pos), start(start), init(init) {}
 
-  virtual void prettyprint(ostream &out, int indent);
+  virtual void prettyprint(ostream &out, Int indent);
 
   virtual void transAsField(coenv &e, record *r, types::ty *base);
 
@@ -343,7 +367,7 @@ public:
     decs.push_back(p);
   }
 
-  virtual void prettyprint(ostream &out, int indent);
+  virtual void prettyprint(ostream &out, Int indent);
 
   virtual void transAsField(coenv &e, record *r, types::ty *base);
 
@@ -356,7 +380,7 @@ public:
   dec(position pos)
     : runnable(pos) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   // Declarations can be public or private.
   bool allowPermissions()
@@ -374,10 +398,16 @@ public:
   vardec(position pos, ty *base, decidlist *decs)
     : dec(pos), base(base), decs(decs) {}
 
-  void prettyprint(ostream &out, int indent);
+  vardec(position pos, ty *base, decid *di)
+    : dec(pos), base(base), decs(new decidlist(pos))
+  {
+    decs->add(di);
+  }
+  void prettyprint(ostream &out, Int indent);
 
   void transAsField(coenv &e, record *r)
   {
+    base->addOps(e, r);
     decs->transAsField(e, r, base->trans(e));
   }
 
@@ -398,14 +428,14 @@ struct idpair : public absyn {
 
   void checkValidity() {
     if (!valid) {
-      em->error(getPos());
-      *em << "expected 'as'";
+      em.error(getPos());
+      em << "expected 'as'";
     }
   }
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
-  // Translates as: import src as dest;
+  // Translates as: access src as dest;
   void transAsAccess(coenv &e, record *r);
   
   // Translates as: from _ unravel src as dest;
@@ -421,7 +451,7 @@ struct idpairlist : public gc {
     base.push_back(x);
   }
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   void transAsAccess(coenv &e, record *r);
 
@@ -439,7 +469,7 @@ public:
   accessdec(position pos, idpairlist *base)
     : dec(pos), base(base) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   void transAsField(coenv &e, record *r) {
     base->transAsAccess(e,r);
@@ -452,16 +482,38 @@ public:
 //   from _ unravel _;  (unraveldec)
 class fromdec : public dec {
 protected:
-  // Return the record from which the fields are taken.  If this returns 0, it
-  // is assumed that an error occurred and was reported.
-  virtual varEntry *getQualifier(coenv &e, record *r) = 0;
+  struct qualifier {
+    // The varEntry holds the location and the type of the highest framed
+    // structure that can be put on the stack.  The record holds the actual type
+    // of the qualifier.
+    // For example:
+    //   struct A {
+    //     struct B {
+    //       static int x;
+    //     }
+    //   }
+    //   A a=new A;
+    //   from a.B unravel x;
+    //
+    // Here, v->getType() will yield A and v->getLocation() will yield the
+    // location of the the variable a, but the record type t will be B.
+    record *t;
+    varEntry *v;
+
+    qualifier(record *t, varEntry *v)
+      : t(t), v(v) {}
+  };
+
+  // Return the qualifier from which the fields are taken.  If t==0, it is
+  // assumed that an error occurred and was reported.
+  virtual qualifier getQualifier(coenv &e, record *r) = 0;
   idpairlist *fields;
 
 public:
   fromdec(position pos, idpairlist *fields)
     : dec(pos), fields(fields) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   void transAsField(coenv &e, record *r);
 };
@@ -471,12 +523,12 @@ public:
 class unraveldec : public fromdec {
   name *id;
 
-  varEntry *getQualifier(coenv &e, record *);
+  qualifier getQualifier(coenv &e, record *);
 public:
   unraveldec(position pos, name *id, idpairlist *fields)
     : fromdec(pos, fields), id(id) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 };
 
 // A fromaccess declaration dumps fields and types of a module into the local
@@ -484,12 +536,12 @@ public:
 class fromaccessdec : public fromdec {
   symbol *id;
 
-  varEntry *getQualifier(coenv &e, record *r);
+  qualifier getQualifier(coenv &e, record *r);
 public:
   fromaccessdec(position pos, symbol *id, idpairlist *fields)
     : fromdec(pos, fields), id(id) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 };
 
 // An import declaration dumps fields and types of a module into the local
@@ -514,7 +566,7 @@ public:
     base.transAsField(e, r);
   }
   
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 };
 
 // Parses the file given, and translates the resulting runnables as if they
@@ -528,7 +580,7 @@ public:
   includedec(position pos, symbol *id)
     : dec(pos), filename(*id) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
   void loadFailed(coenv &e);
 
   void transAsField(coenv &e, record *r);
@@ -542,7 +594,7 @@ public:
   typedec(position pos, vardec *body)
     : dec(pos), body(body) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   void transAsField(coenv &e, record *r) {
     body->transAsTypedefField(e,r);
@@ -556,6 +608,7 @@ class recorddec : public dec {
   block *body;
 
   void transRecordInitializer(coenv &e, record *parent);
+  void addPostRecordEnvironment(coenv &e, record *r, record *parent);
 
 public:
   recorddec(position pos, symbol *id, block *body)
@@ -564,7 +617,7 @@ public:
   virtual ~recorddec()
     {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   void transAsField(coenv &e, record *parent);
 };
