@@ -101,7 +101,7 @@ public:
            record *where, position pos)
     : entry(perm, r, where, pos), t(t), location(location) {}
 
-  // (Non-destructively) merges two varEntries, created a qualified varEntry.
+  // (Non-destructively) merges two varEntries, creating a qualified varEntry.
   varEntry(varEntry &qv, varEntry &v);
 
   ty *getType()
@@ -137,9 +137,31 @@ public:
 
   tyEntry(tyEntry *base, permission perm, record *r)
     : entry(*base, perm, r), t(base->t), v(base->v) {}
+
+  // Records need a varEntry that refers back to the qualifier qv; i.e. in
+  // the last new of the code
+  //   struct A {
+  //     struct B {}
+  //   }
+  //   A a=new A;
+  //   unravel a;
+  //   new B;
+  // we need to put a's frame on the stack before allocating an instance of B.
+  // NOTE: A possible optimization could be to only qualify the varEntry if
+  // the type is a record, as other types don't use the varEntry.
+private:
+  tyEntry(tyEntry *base, varEntry *qv)
+    : entry(*base, *qv), t(base->t), v(qualifyVarEntry(qv, base->v)) {}
+
+public:
+  // Since the constructor can only be used when qv is non-null it is private
+  // for safety reasons, and we provide this method instead.
+  friend tyEntry *qualifyTyEntry(varEntry *qv, tyEntry *ent);
 };
 
-tyEntry *qualifyTyEntry(varEntry *qv, tyEntry *ent);
+inline tyEntry *qualifyTyEntry(varEntry *qv, tyEntry *ent) {
+  return qv ? new tyEntry(ent, qv) : ent;
+}
 
 // The type environment.
 class tenv : public sym::table<tyEntry *> {
@@ -160,7 +182,12 @@ public:
 #ifdef NOHASH //{{{
 class venv : public sym::table<varEntry*> {
 public:
-  venv();
+  venv() {}
+
+  // This is an optimization in the hashtable version that is duplicated here
+  // for compatibility.  It is identical to venv().
+  struct file_env_tag {};
+  venv(file_env_tag) {}
 
 #if 0
   // Look for a function that exactly matches the signature given.
@@ -189,6 +216,12 @@ public:
   // overloaded type.
   ty *getType(symbol *name);
 
+  // This is an optimization that is only implemented for the hashtable
+  // version.
+  void *getMarker(symbol *) {
+    return 0;
+  }
+
   friend std::ostream& operator<< (std::ostream& out, const venv& ve);
   
   // Prints a list of the variables to the standard output.
@@ -200,7 +233,6 @@ public:
 
 // venv implemented with a hash table.
 class venv {
-public:
   struct key : public gc {
     symbol *name;
     ty *t;
@@ -211,6 +243,8 @@ public:
     key(symbol *name, varEntry *v)
       : name(name), t(v->getType()) {}
   };
+  friend ostream& operator<< (ostream& out, const venv::key &k);
+
   struct value : public gc {
     varEntry *v;
     bool shadowed;
@@ -277,16 +311,29 @@ public:
   // Helper function for endScope.
   void remove(key k);
 
+  // These are roughly the size the hashtables will be after loading the
+  // builtin functions and plain module.
+  static const size_t fileAllSize=2000;
+  static const size_t namesAllSize=1000;
 public:
   venv() {
     beginScope();
   }
 
+  // Most file level modules automatically import plain, so allocate hashtables
+  // big enough to hold it in advance.
+  struct file_env_tag {};
+  venv(file_env_tag)
+    : all(fileAllSize), names(namesAllSize)
+  {
+    beginScope();
+  }
+
   void enter(symbol *name, varEntry *v);
 
   // Add the entries in one environment to another, if qualifier is
-  // non-null, it is a record and the source environment are its fields.
-  // The coder is necessary to check which variables are accessible and
+  // non-null, it is a record and entries of the source environment are its
+  // fields.  The coder is necessary to check which variables are accessible and
   // should be added.
   void add(venv& source, varEntry *qualifier, coder &c);
 
@@ -316,6 +363,19 @@ public:
 
   ty *getType(symbol *name);
 
+  // The abstract syntax caches results for resolving overloaded variables.  It
+  // is not enough to index these cached results by name, as new variables of
+  // the same name could be added, invalidating the cached result.  Instead,
+  // they are indexed by a marker, that is unique to the name and to its current
+  // overloading.  The values list does this, and the values list is uniquely
+  // determined by its first entry (as values are not re-used, and the list is
+  // only manipulated from its front).  Therefore, we return an opaque pointer
+  // to the first entry on the list.
+  void *getMarker(symbol *name) {
+    values &list=names[name];
+    return list.empty() ? 0 : (void *)list.front();
+  }
+
   void beginScope() {
     scopes.push(keymultimap());
   }
