@@ -13,6 +13,9 @@
 
 using namespace types;
 
+namespace absyntax {
+  void clearCachedCalls();
+}
 
 namespace trans {
 
@@ -135,7 +138,7 @@ void protoenv::addFunctionOps(function *f)
 }
 
 env::env(genv &ge)
-  : ge(ge)
+  : protoenv(venv::file_env_tag()), ge(ge)
 {
   // NOTE: May want to make this initial environment into a "builtin" module,
   // and then import the builtin module.
@@ -143,6 +146,14 @@ env::env(genv &ge)
   base_venv(ve);
 }
 
+env::~env()
+{
+  // Once the file-level venv is destroyed, any cached calls with markers
+  // derived from that venv are meaningless; clear them so they don't keep data
+  // live.
+  absyntax::clearCachedCalls();
+}
+
 record *env::getModule(symbol *id, string filename)
 {
   return ge.getModule(id, filename);
