@@ -43,6 +43,9 @@ public:
   // Start an environment for a file-level module.
   protoenv() {}
 
+  protoenv(venv::file_env_tag tag)
+    : ve(tag) {}
+
   protoenv(const protoenv&);
   
   void beginScope()
@@ -101,6 +104,11 @@ public:
     return ve.getType(name);
   }
 
+  void *varGetMarker(symbol *name)
+  {
+    return ve.getMarker(name);
+  }
+
   void addType(symbol *name, tyEntry *desc)
   {
     te.enter(name, desc);
@@ -162,6 +170,8 @@ public:
   // Start an environment for a file-level module.
   env(genv &ge);
 
+  ~env();
+
   record *getModule(symbol *id, string filename);
 };
 
