@@ -21,9 +21,10 @@ namespace absyntax {
 using namespace types;
 using namespace trans;
 using vm::inst;
+using mem::vector;
 
 
-void exp::prettyprint(ostream &out, int indent)
+void exp::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "exp",indent);
 }
@@ -38,17 +39,17 @@ void exp::transToType(coenv &e, types::ty *target)
   types::ty *source=e.e.castSource(target, cgetType(e), symbol::castsym);
   if (source==0) {
     types::ty *sources=cgetType(e);
-    em->error(getPos());
-    *em << "cannot cast ";
+    em.error(getPos());
+    em << "cannot cast ";
     if (sources->kind==ty_overloaded)
-      *em << "expression";
+      em << "expression";
     else
-      *em << "'" << *sources << "'";
-    *em << " to '" << *target << "'";
+      em << "'" << *sources << "'";
+    em << " to '" << *target << "'";
   }
   else if (source->kind==ty_overloaded) {
-    em->error(getPos());
-    *em << "expression is ambiguous in cast to '" << *target << "'";
+    em.error(getPos());
+    em << "expression is ambiguous in cast to '" << *target << "'";
   }
   else {
     transAsType(e, source);
@@ -110,7 +111,7 @@ void varEntryExp::transCall(coenv &e, types::ty *target) {
 }
 
 
-void nameExp::prettyprint(ostream &out, int indent)
+void nameExp::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "nameExp",indent);
 
@@ -118,7 +119,7 @@ void nameExp::prettyprint(ostream &out, int indent)
 }
 
 
-void fieldExp::pseudoName::prettyprint(ostream &out, int indent)
+void fieldExp::pseudoName::prettyprint(ostream &out, Int indent)
 {
   // This should never be called.
   prettyindent(out, indent);
@@ -127,7 +128,7 @@ void fieldExp::pseudoName::prettyprint(ostream &out, int indent)
   object->prettyprint(out, indent+1);
 }
 
-void fieldExp::prettyprint(ostream &out, int indent)
+void fieldExp::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out, indent);
   out << "fieldExp '" << *field << "'\n";
@@ -146,7 +147,7 @@ types::ty *fieldExp::getObject(coenv& e)
 }
 
 
-void subscriptExp::prettyprint(ostream &out, int indent)
+void subscriptExp::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out, indent);
   out << "subscriptExp\n";
@@ -180,8 +181,8 @@ array *subscriptExp::transArray(coenv &e)
   if (a->kind == ty_overloaded) {
     a = ((overloaded *)a)->signatureless();
     if (!a) {
-      em->error(set->getPos());
-      *em << "expression is not an array";
+      em.error(set->getPos());
+      em << "expression is not an array";
       return 0;
     }
   }
@@ -194,8 +195,8 @@ array *subscriptExp::transArray(coenv &e)
     case ty_error:
       return 0;
     default:
-      em->error(set->getPos());
-      *em << "expression is not an array";
+      em.error(set->getPos());
+      em << "expression is not an array";
       return 0;
   }
 }
@@ -216,7 +217,7 @@ types::ty *subscriptExp::trans(coenv &e)
     return primError();
 
   if (isAnArray(e, index)) {
-    index->transToType(e, types::intArray());
+    index->transToType(e, types::IntArray());
     e.c.encode(inst::builtin, run::arrayIntArray);
     return getArrayType(e);
   }
@@ -248,7 +249,7 @@ void subscriptExp::transWrite(coenv &e, types::ty *t)
 }
 
 
-void thisExp::prettyprint(ostream &out, int indent)
+void thisExp::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "thisExp", indent);
 }
@@ -256,8 +257,8 @@ void thisExp::prettyprint(ostream &out, int indent)
 types::ty *thisExp::trans(coenv &e)
 {
   if (!e.c.encodeThis()) {
-    em->error(getPos());
-    *em << "static use of 'this' expression";
+    em.error(getPos());
+    em << "static use of 'this' expression";
   }
   return cgetType(e);
 }
@@ -267,7 +268,7 @@ types::ty *thisExp::getType(coenv &e)
   return e.c.thisType();
 }
 
-void scaleExp::prettyprint(ostream &out, int indent)
+void scaleExp::prettyprint(ostream &out, Int indent)
 {
   exp *left=getLeft(); exp *right=getRight();
 
@@ -281,18 +282,18 @@ types::ty *scaleExp::trans(coenv &e)
   exp *left=getLeft(); exp *right=getRight();
 
   types::ty *lt = left->cgetType(e);
-  if (lt->kind != types::ty_int && lt->kind != types::ty_real) {
+  if (lt->kind != types::ty_Int && lt->kind != types::ty_real) {
     if (lt->kind != types::ty_error) {
-      em->error(left->getPos());
-      *em << "only numeric constants can do implicit scaling";
+      em.error(left->getPos());
+      em << "only numeric constants can do implicit scaling";
     }
     right->trans(e);
     return types::primError();
   }
 
   if (!right->scalable()) {
-    em->warning(right->getPos());
-    *em << "implicit scaling may be unintentional";
+    em.warning(right->getPos());
+    em << "implicit scaling may be unintentional";
   }
 
   // Defer to the binaryExp for multiplication.
@@ -300,7 +301,7 @@ types::ty *scaleExp::trans(coenv &e)
 }
 
 
-void intExp::prettyprint(ostream &out, int indent)
+void intExp::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out,indent);
   out << "intExp: " << value << "\n";
@@ -314,7 +315,7 @@ types::ty *intExp::trans(coenv &e)
 }
 
 
-void realExp::prettyprint(ostream &out, int indent)
+void realExp::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out, indent);
   out << "realExp: " << value << "\n";
@@ -327,7 +328,7 @@ types::ty *realExp::trans(coenv &e)
   return types::primReal();  
 }
 
-void stringExp::prettyprint(ostream &out, int indent)
+void stringExp::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out, indent);
   out << "stringExp '" << str << "'\n";
@@ -341,7 +342,7 @@ types::ty *stringExp::trans(coenv &e)
 }
 
 
-void booleanExp::prettyprint(ostream &out, int indent)
+void booleanExp::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out, indent);
   out << "booleanExp: " << value << "\n";
@@ -354,7 +355,7 @@ types::ty *booleanExp::trans(coenv &e)
   return types::primBoolean();  
 }
 
-void newPictureExp::prettyprint(ostream &out, int indent)
+void newPictureExp::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "newPictureExp",indent);
 }
@@ -366,7 +367,7 @@ types::ty *newPictureExp::trans(coenv &e)
   return types::primPicture();  
 }
 
-void cycleExp::prettyprint(ostream &out, int indent)
+void cycleExp::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "cycleExp",indent);
 }
@@ -378,7 +379,7 @@ types::ty *cycleExp::trans(coenv &e)
   return types::primCycleToken();  
 }
 
-void nullPathExp::prettyprint(ostream &out, int indent)
+void nullPathExp::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "nullPathExp",indent);
 }
@@ -390,7 +391,7 @@ types::ty *nullPathExp::trans(coenv &e)
   return types::primPath();  
 }
 
-void nullExp::prettyprint(ostream &out, int indent)
+void nullExp::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "nullExp",indent);
 }
@@ -403,7 +404,7 @@ types::ty *nullExp::trans(coenv &)
 }
 
 
-void quoteExp::prettyprint(ostream &out, int indent)
+void quoteExp::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "quoteExp", indent);
   value->prettyprint(out, indent+1);
@@ -416,7 +417,7 @@ types::ty *quoteExp::trans(coenv &e)
   return types::primCode();  
 }
 
-void explist::prettyprint(ostream &out, int indent)
+void explist::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "explist",indent);
   for (expvector::iterator p = exps.begin();
@@ -425,7 +426,7 @@ void explist::prettyprint(ostream &out, int indent)
 }
 
 
-void argument::prettyprint(ostream &out, int indent)
+void argument::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out, indent);
   out << "explist";
@@ -438,12 +439,12 @@ void argument::prettyprint(ostream &out, int indent)
 
 void argument::assignAmbiguity(coenv &e) {
   if (name && e.e.varGetType(name) && settings::getSetting<bool>("debug")) {
-    em->warning(val->getPos());
-    *em << "named argument may be mistaken for assignment";
+    em.warning(val->getPos());
+    em << "named argument may be mistaken for assignment";
   }
 }
 
-void arglist::prettyprint(ostream &out, int indent)
+void arglist::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "arglist",indent);
   for (argvector::iterator p = args.begin();
@@ -451,8 +452,74 @@ void arglist::prettyprint(ostream &out, int indent)
     p->prettyprint(out, indent+1);
 }
 
+/*****
+  Global level caching for callExp.
+  Consider the code:
 
-void callExp::prettyprint(ostream &out, int indent)
+    write(1+2);
+    write(3+4);
+
+  In the first line, callExp has to resolve the overloaded operator+ when given
+  two integer arguments, and then resolve the overloaded write function when
+  given an integer argument.  On the second line, it has to do this all over
+  again, with different arguments, but arguments that yield the same types.
+ 
+  The matchCache attempts to mitigate this work by storing which function type
+  was picked when an overloaded function call was resolved.  The matches are
+  indexed by the marker of the callee expression (which uniquely determines the
+  possibly overloaded type of the callee), and the arguments with which it was
+  called.
+
+  For arguments to match, they must not only have the same types, but the same
+  names, if names are specified.  Calls with overloaded arguments are not added
+  to or checked against the cache.
+*****/
+
+struct cachedCall {
+  void *marker;
+  types::signature *args;
+  types::function *match;
+
+  cachedCall()
+    : marker(0), args(0), match(0) {}
+};
+
+// If the cacheSize is too large, too much data is kept live, which slows down
+// the garbage collector.  This seems to be a happy medium.
+const size_t matchCacheSize=53;
+vector<cachedCall> matchCache(matchCacheSize);
+
+size_t hash(void *marker, types::signature *source) {
+  return 5 * (size_t)marker + source->hash();
+}
+
+void clearCachedCalls() {
+  matchCache.clear();
+  matchCache.resize(matchCacheSize);
+}
+
+function *lookupMatch(void *marker, types::signature *source) {
+  assert(marker!=0);
+  size_t h=hash(marker, source);
+  cachedCall &cell=matchCache[h % matchCacheSize];
+  if (cell.marker==marker &&
+      cell.args &&
+      argumentEquivalent(cell.args, source))
+    return cell.match;
+  else
+    return 0;
+}
+
+void addMatch(void *marker, types::signature *source, function *ft) {
+  assert(marker!=0);
+  size_t h=hash(marker, source);
+  cachedCall &cell=matchCache[h % matchCacheSize];
+  cell.marker=marker;
+  cell.args=source;
+  cell.match=ft;
+}
+
+void callExp::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "callExp",indent);
 
@@ -491,32 +558,37 @@ signature *callExp::argTypes(coenv &e)
   return source;
 }
 
-application *callExp::resolve(coenv &e, overloaded *o, signature *source) {
+application *callExp::resolve(coenv &e, overloaded *o, signature *source,
+                              bool tacit) {
   app_list l=multimatch(e.e, o, source, *args);
 
   if (l.empty()) {
     //cerr << "l is empty\n";
-    em->error(getPos());
+    if (!tacit) {
+      em.error(getPos());
 
-    symbol *s = callee->getName();
-    if (s)
-      *em << "no matching function \'" << *s;
-    else
-      *em << "no matching function for signature \'";
-    *em << *source << "\'";
+      symbol *s = callee->getName();
+      if (s)
+        em << "no matching function \'" << *s;
+      else
+        em << "no matching function for signature \'";
+      em << *source << "\'";
+    }
 
     return 0;
   }
   else if (l.size() > 1) { // This may take O(n) time.
     //cerr << "l is full\n";
-    em->error(getPos());
+    if (!tacit) {
+      em.error(getPos());
 
-    symbol *s = callee->getName();
-    if(s)
-      *em << "call of function \'" << *s;
-    else
-      *em << "call with signature \'";
-    *em << *source << "\' is ambiguous";
+      symbol *s = callee->getName();
+      if(s)
+        em << "call of function \'" << *s;
+      else
+        em << "call with signature \'";
+      em << *source << "\' is ambiguous";
+    }
 
     return 0;
   }
@@ -526,26 +598,70 @@ application *callExp::resolve(coenv &e, overloaded *o, signature *source) {
   }
 }
 
+bool noOverloadedArgs(signature *source) {
+  types::formal_vector &formals=source->formals;
+  for (types::formal_vector::iterator f=formals.begin(); f!=formals.end(); ++f)
+    if (f->t->kind == types::ty_overloaded)
+      return false;
+  formal &rest=source->getRest();
+  if (rest.t && rest.t->kind == types::ty_overloaded)
+    return false;
+  return true;
+}
+  
+application *callExp::resolveWithCache(coenv &e,
+                                       overloaded *o, signature *source,
+                                       bool tacit) {
+  void *marker=callee->getMarker(e);
+  if (marker!=0 && noOverloadedArgs(source)) {
+    //cerr << "looking up cache" << endl;
+    function *ft=lookupMatch(marker, source);
+    if (ft) {
+      application *a=application::match(e.e, ft, source, *args);
+      if (!a) {
+        em.compiler(getPos());
+        em << "match caching is corrupted";
+      }
+      //cerr << "successful cache hit" << endl;
+      return a;
+    }
+    else {
+      // Match it and add it to the cache.
+      application *a=resolve(e, o, source, tacit);
+
+      // Add to cache.
+      if (a) {
+        //cerr << "adding to cache" << endl;
+        addMatch(marker, source, a->getType());
+      }
+
+      return a;
+    }
+  } else {
+    // Can't cache, just do normal resolve.
+    return resolve(e, o, source, tacit);
+  }
+}
 
 void callExp::reportMismatch(symbol *s, function *ft, signature *source)
 {
   const char *separator=ft->getSignature()->getNumFormals() > 1 ? "\n" : " ";
 
-  em->error(getPos());
-  *em << "cannot call" << separator << "'" << *ft->getResult() << " ";
+  em.error(getPos());
+  em << "cannot call" << separator << "'" << *ft->getResult() << " ";
   if(s)
-    *em << *s;
-  *em << *ft->getSignature() << "'" << separator;
+    em << *s;
+  em << *ft->getSignature() << "'" << separator;
 
   switch(source->getNumFormals()) {
     case 0:
-      *em << "without parameters";
+      em << "without parameters";
       break;
     case 1:
-      *em << "with parameter '" << *source << "'";
+      em << "with parameter '" << *source << "'";
       break;
     default:
-      *em << "with parameters\n'" << *source << "'";
+      em << "with parameters\n'" << *source << "'";
   }
 }
 
@@ -584,15 +700,15 @@ application *callExp::getApplication(coenv &e)
     } 
     case ty_overloaded:
       //cerr << "resolving overloaded\n";
-      return resolve(e, (overloaded *)ft, source);
+      return resolveWithCache(e, (overloaded *)ft, source, false);
     default:
       //cerr << "not a function\n";
-      em->error(getPos());
+      em.error(getPos());
       symbol *s = callee->getName();
       if (s)
-        *em << "\'" << *s << "\' is not a function";
+        em << "\'" << *s << "\' is not a function";
       else
-        *em << "called expression is not a function";
+        em << "called expression is not a function";
       return 0;
   }
 }
@@ -610,8 +726,8 @@ types::ty *callExp::trans(coenv &e)
 
   application *a= ca ? ca : getApplication(e);
   
-  // The cached application is no longer needed after translation, so let be
-  // garbage collected.
+  // The cached application is no longer needed after translation, so
+  // let it be garbage collected.
   ca=0;
 
   if (!a)
@@ -648,11 +764,10 @@ types::ty *callExp::getType(coenv &e)
     case ty_function:
       return ((function *)ft)->result;
     case ty_overloaded: {
-      app_list l=multimatch(e.e, (overloaded *)ft, source, *args);
-
-      if (l.size()==1) {
+      application *a=resolveWithCache(e, (overloaded *)ft, source, true);
+      if (a) {
         // Cache the application to avoid calling multimatch again later.
-        ca=l.front();
+        ca=a;
         return ca->getType()->result;
       }
       else
@@ -663,7 +778,7 @@ types::ty *callExp::getType(coenv &e)
   }
 }
     
-void pairExp::prettyprint(ostream &out, int indent)
+void pairExp::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "pairExp",indent);
 
@@ -681,7 +796,7 @@ types::ty *pairExp::trans(coenv &e)
   return types::primPair();
 }
 
-void tripleExp::prettyprint(ostream &out, int indent)
+void tripleExp::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "tripleExp",indent);
 
@@ -701,7 +816,7 @@ types::ty *tripleExp::trans(coenv &e)
   return types::primTriple();
 }
 
-void transformExp::prettyprint(ostream &out, int indent)
+void transformExp::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "transformExp",indent);
 
@@ -727,7 +842,7 @@ types::ty *transformExp::trans(coenv &e)
   return types::primTransform();
 }
 
-void castExp::prettyprint(ostream &out, int indent)
+void castExp::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "castExp",indent);
 
@@ -743,8 +858,8 @@ types::ty *castExp::tryCast(coenv &e, types::ty *t, types::ty *s,
     return 0;
   }
   if (ss->kind == ty_overloaded) {
-    em->error(getPos());
-    *em << "cast is ambiguous";
+    em.error(getPos());
+    em << "cast is ambiguous";
     return primError();
   }
   else {
@@ -759,14 +874,15 @@ types::ty *castExp::tryCast(coenv &e, types::ty *t, types::ty *s,
 
 types::ty *castExp::trans(coenv &e)
 {
+  target->addOps(e, (record *)0);
   types::ty *t=target->trans(e);
 
   types::ty *s=castee->cgetType(e);
 
   if (!tryCast(e, t, s, symbol::ecastsym))
     if (!tryCast(e, t, s, symbol::castsym)) {
-      em->error(getPos());
-      *em << "cannot cast '" << *s << "' to '" << *t << "'";
+      em.error(getPos());
+      em << "cannot cast '" << *s << "' to '" << *t << "'";
     }
 
   return t;
@@ -778,7 +894,7 @@ types::ty *castExp::getType(coenv &e)
 }
 
 
-void conditionalExp::prettyprint(ostream &out, int indent)
+void conditionalExp::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "conditionalExp",indent);
 
@@ -790,12 +906,12 @@ void conditionalExp::prettyprint(ostream &out, int indent)
 void conditionalExp::baseTransToType(coenv &e, types::ty *target) {
   test->transToType(e, types::primBoolean());
 
-  int tlabel = e.c.fwdLabel();
+  Int tlabel = e.c.fwdLabel();
   e.c.useLabel(inst::cjmp,tlabel);
 
   onFalse->transToType(e, target);
 
-  int end = e.c.fwdLabel();
+  Int end = e.c.fwdLabel();
   e.c.useLabel(inst::jmp,end);
 
   e.c.defLabel(tlabel);
@@ -808,8 +924,8 @@ void conditionalExp::transToType(coenv &e, types::ty *target)
 {
   if (isAnArray(e, test)) {
     if (target->kind != ty_array) {
-      em->error(getPos());
-      *em << "cannot cast vectorized conditional to '" << *target << "'";
+      em.error(getPos());
+      em << "cannot cast vectorized conditional to '" << *target << "'";
     }
     test->transToType(e, types::boolArray());
     onTrue->transToType(e, target);
@@ -866,13 +982,13 @@ types::ty *conditionalExp::trans(coenv &e)
 
   types::ty *t=promote(e, tt, ft);
   if (!t) {
-    em->error(getPos());
-    *em << "types in conditional expression do not match";
+    em.error(getPos());
+    em << "types in conditional expression do not match";
     return primError();
   }
   else if (t->kind == ty_overloaded) {
-    em->error(getPos());
-    *em << "type of conditional expression is ambiguous";
+    em.error(getPos());
+    em << "type of conditional expression is ambiguous";
     return primError();
   }
 
@@ -892,7 +1008,7 @@ types::ty *conditionalExp::getType(coenv &e)
 }
  
 
-void orExp::prettyprint(ostream &out, int indent)
+void orExp::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "orExp", indent);
 
@@ -913,7 +1029,7 @@ types::ty *orExp::trans(coenv &e)
 }
 
 
-void andExp::prettyprint(ostream &out, int indent)
+void andExp::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "andExp", indent);
 
@@ -934,7 +1050,7 @@ types::ty *andExp::trans(coenv &e)
 }
 
 
-void joinExp::prettyprint(ostream &out, int indent)
+void joinExp::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "joinExp",indent);
 
@@ -943,7 +1059,7 @@ void joinExp::prettyprint(ostream &out, int indent)
 }
 
 
-void specExp::prettyprint(ostream &out, int indent)
+void specExp::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out,indent);
   out << "specExp '" << *op << "' " 
@@ -956,19 +1072,19 @@ void specExp::prettyprint(ostream &out, int indent)
 
 types::ty *specExp::trans(coenv &e)
 {
-  intExp ie(getPos(), (int)s);
+  intExp ie(getPos(), (Int)s);
   binaryExp be(getPos(), arg, op, &ie);
   return be.trans(e);
 }
 
 types::ty *specExp::getType(coenv &e)
 {
-  intExp ie(getPos(), (int)s);
+  intExp ie(getPos(), (Int)s);
   binaryExp be(getPos(), arg, op, &ie);
   return be.cgetType(e);
 }
 
-void assignExp::prettyprint(ostream &out, int indent)
+void assignExp::prettyprint(ostream &out, Int indent)
 {
   prettyname(out, "assignExp",indent);
 
@@ -997,13 +1113,13 @@ types::ty *assignExp::trans(coenv &e)
 
   types::ty *t = e.e.castTarget(lt, rt, symbol::castsym);
   if (!t) {
-    em->error(getPos());
-    *em << "cannot convert '" << *rt << "' to '" << *lt << "' in assignment";
+    em.error(getPos());
+    em << "cannot convert '" << *rt << "' to '" << *lt << "' in assignment";
     return primError();
   }
   else if (t->kind == ty_overloaded) {
-    em->error(getPos());
-    *em << "assignment is ambiguous";
+    em.error(getPos());
+    em << "assignment is ambiguous";
     return primError();
   }
   else {
@@ -1023,7 +1139,7 @@ types::ty *assignExp::getType(coenv &e)
 }
 
 
-void selfExp::prettyprint(ostream &out, int indent)
+void selfExp::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out, indent);
   out << "selfExp '" << *op << "'\n";
@@ -1033,7 +1149,7 @@ void selfExp::prettyprint(ostream &out, int indent)
 }
 
 
-void prefixExp::prettyprint(ostream &out, int indent)
+void prefixExp::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out, indent);
   out << "prefixExp '" << *op << "'\n";
@@ -1060,7 +1176,7 @@ types::ty *prefixExp::getType(coenv &e)
   return se.getType(e);
 }
 
-void postfixExp::prettyprint(ostream &out, int indent)
+void postfixExp::prettyprint(ostream &out, Int indent)
 {
   prettyindent(out, indent);
   out << "postfixExp <illegal>";
@@ -1071,8 +1187,8 @@ void postfixExp::prettyprint(ostream &out, int indent)
 
 types::ty *postfixExp::trans(coenv &)
 {
-  em->error(getPos());
-  *em << "postfix expressions are not allowed";
+  em.error(getPos());
+  em << "postfix expressions are not allowed";
   return primError();
 }
 
