@@ -39,7 +39,7 @@ public:
   exp(position pos)
     : varinit(pos), ct(0) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   // When reporting errors with function calls, it is nice to say "no
   // function f(int)" instead of "no function matching signature
@@ -50,8 +50,20 @@ public:
     return 0;
   }
 
-  // Checks if expression can be used as the right side of a scale
+  // If two expressions have the same marker, then they are guaranteed to
+  // return the same (possibly overloaded) types.  This is exploited by callExp
+  // when caching the results of resolving overloaded functions. If this returns
+  // 0 then there are no guarantees on the type of the expression.
+  // Marker are currently only implemented for simple variable names, by the
+  // markers provided by venv::getMarker().
+  virtual void *getMarker(coenv &)
+  {
+    return 0;
+  }
+
+  // Checks if the expression can be used as the right side of a scale
   // expression.  ie. 3sin(x)
+  // If a "non-scalable" expression is scaled a warning is issued.
   virtual bool scalable() { return true; }
   
   // Specifies if the value of the expression should be written to interactive
@@ -59,7 +71,7 @@ public:
   // > 2+3;
   // should write 5, but
   // > x=2+3;
-  // shouldn't.  (This choices are largely aesthetic)
+  // shouldn't.  (These choices are largely aesthetic)
   virtual bool writtenToPrompt() { return true; }
 
   // Translates the expression to the given target type.  This should only be
@@ -112,8 +124,8 @@ public:
   // so that the value on top of stack is put into the address (but not popped
   // off the stack).
   virtual void transWrite(coenv &, types::ty *) {
-    em->error(getPos());
-    *em << "expression cannot be used as an address";
+    em.error(getPos());
+    em << "expression cannot be used as an address";
   }
 
   // Translates code for calling a function.  The arguments, in the order they
@@ -185,13 +197,18 @@ public:
   nameExp(position pos, symbol *id)
     : exp(pos), value(new simpleName(pos, id)) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   symbol *getName()
   {
     return value->getName();
   }
 
+  void *getMarker(coenv &e)
+  {
+    return value->getMarker(e);
+  }
+
   void transAsType(coenv &e, types::ty *target) {
     value->varTrans(trans::READ, e, target);
     
@@ -205,13 +222,13 @@ public:
   types::ty *trans(coenv &e) {
     types::ty *t=cgetType(e);
     if (t->kind == types::ty_error) {
-      em->error(getPos());
-      *em << "no matching variable \'" << *value << "\'";
+      em.error(getPos());
+      em << "no matching variable \'" << *value << "\'";
       return types::primError();
     }
     if (t->kind == types::ty_overloaded) {
-      em->error(getPos());
-      *em << "use of variable \'" << *value << "\' is ambiguous";
+      em.error(getPos());
+      em << "use of variable \'" << *value << "\' is ambiguous";
       return types::primError();
     }
     else {
@@ -271,21 +288,21 @@ class fieldExp : public nameExp {
     // As a type:
     types::ty *typeTrans(coenv &, bool tacit = false) {
       if (!tacit) {
-        em->error(getPos());
-        *em << "expression is not a type";
+        em.error(getPos());
+        em << "expression is not a type";
       }
       return types::primError();
     }
 
     trans::varEntry *getVarEntry(coenv &) {
-      em->compiler(getPos());
-      *em << "expression cannot be used as part of a type";
+      em.compiler(getPos());
+      em << "expression cannot be used as part of a type";
       return 0;
     }
 
     trans::tyEntry *tyEntryTrans(coenv &) {
-      em->compiler(getPos());
-      *em << "expression cannot be used as part of a type";
+      em.compiler(getPos());
+      em << "expression cannot be used as part of a type";
       return 0;
     }
 
@@ -293,7 +310,7 @@ class fieldExp : public nameExp {
       return 0;
     }
 
-    void prettyprint(ostream &out, int indent);
+    void prettyprint(ostream &out, Int indent);
     void print(ostream& out) const {
       out << "<exp>";
     }
@@ -312,7 +329,7 @@ public:
                                      field)),
       object(object), field(field) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   symbol *getName()
   {
@@ -338,7 +355,7 @@ public:
   subscriptExp(position pos, exp *set, exp *index)
     : exp(pos), set(set), index(index) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &e);
@@ -357,7 +374,7 @@ public:
   thisExp(position pos)
     : exp(pos) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &e);
@@ -372,13 +389,13 @@ public:
 };
 
 class intExp : public literalExp {
-  int value;
+  Int value;
 
 public:
-  intExp(position pos, int value)
+  intExp(position pos, Int value)
     : literalExp(pos), value(value) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &) { return types::primInt(); }
@@ -392,7 +409,7 @@ public:
   realExp(position pos, double value)
     : literalExp(pos), value(value) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &) { return types::primReal(); }
@@ -406,7 +423,7 @@ public:
   stringExp(position pos, string str)
     : literalExp(pos), str(str) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &) { return types::primString(); }
@@ -419,7 +436,7 @@ public:
   booleanExp(position pos, bool value)
     : literalExp(pos), value(value) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &) { return types::primBoolean(); }
@@ -431,7 +448,7 @@ public:
   cycleExp(position pos)
     : literalExp(pos) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &) { return types::primCycleToken(); }
@@ -443,7 +460,7 @@ public:
   newPictureExp(position pos)
     : literalExp(pos) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &) { return types::primPicture(); }
@@ -455,7 +472,7 @@ public:
   nullPathExp(position pos)
     : literalExp(pos) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &) { return types::primPath(); }
@@ -467,7 +484,7 @@ public:
   nullExp(position pos)
     : literalExp(pos) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &) { return types::primNull(); }
@@ -480,7 +497,7 @@ public:
   quoteExp(position pos, runnable *value)
     : exp(pos), value(value) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &) { return types::primCode(); }
@@ -501,7 +518,7 @@ public:
     exps.push_back(e);
   }
 
-  virtual void prettyprint(ostream &out, int indent);
+  virtual void prettyprint(ostream &out, Int indent);
 
   virtual size_t size() {
     return exps.size();
@@ -521,7 +538,7 @@ struct argument {
     : val(val), name(name) {}
 #endif
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   // Tests if a named argument could be mistaken for an assignment, and
   // prints a warning if so.
@@ -557,7 +574,7 @@ public:
     add(a);
   }
 
-  virtual void prettyprint(ostream &out, int indent);
+  virtual void prettyprint(ostream &out, Int indent);
 
   virtual size_t size() {
     return args.size();
@@ -572,6 +589,9 @@ public:
   }
 };
 
+// callExp has a global cache of resolved overloaded functions.  This clears
+// this cache so the associated data can be garbage collected.
+void clearCachedCalls();
 
 class callExp : public exp {
 protected:
@@ -579,7 +599,7 @@ protected:
   arglist *args;
 
 private:
-  // Cache the application when it's determined.
+  // Per object caching - Cache the application when it's determined.
   application *ca;
 
   // Warns of ambiguity with assign expression in named arguments.
@@ -588,7 +608,12 @@ private:
   types::signature *argTypes(coenv& e);
   application *resolve(coenv &e,
                        types::overloaded *o,
-                       types::signature *source);
+                       types::signature *source,
+                       bool tacit);
+  application *resolveWithCache(coenv &e,
+                                types::overloaded *o,
+                                types::signature *source,
+                                bool tacit);
   void reportMismatch(symbol *s,
                       types::function *ft,
                       types::signature *source);
@@ -619,7 +644,7 @@ public:
       args->add(arg3);
     }
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &e);
@@ -633,7 +658,7 @@ public:
   pairExp(position pos, exp *x, exp *y)
     : exp(pos), x(x), y(y) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &) { return types::primPair(); }
@@ -648,7 +673,7 @@ public:
   tripleExp(position pos, exp *x, exp *y, exp *z)
     : exp(pos), x(x), y(y), z(z) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &) { return types::primTriple(); }
@@ -664,7 +689,7 @@ public:
 	       exp *yy)
     : exp(pos), x(x), y(y), xx(xx), xy(xy), yx(yx), yy(yy) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &) { return types::primTransform(); }
@@ -680,7 +705,7 @@ public:
   castExp(position pos, ty *target, exp *castee)
     : exp(pos), target(target), castee(castee) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &e);
@@ -716,7 +741,7 @@ public:
   scaleExp(position pos, exp *left, exp *right)
     : binaryExp(pos, left, symbol::trans("*"), right) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
   //types::ty *getType(coenv &e);
@@ -742,7 +767,7 @@ public:
   conditionalExp(position pos, exp *test, exp *onTrue, exp *onFalse)
     : exp(pos), test(test), onTrue(onTrue), onFalse(onFalse) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   void baseTransToType(coenv &e, types::ty *target);
 
@@ -773,7 +798,7 @@ public:
   orExp(position pos, exp *left, symbol *op, exp *right)
     : andOrExp(pos, left, op, right) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
 };
@@ -783,7 +808,7 @@ public:
   andExp(position pos, exp *left, symbol *op, exp *right)
     : andOrExp(pos, left, op, right) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
 };
@@ -800,7 +825,7 @@ public:
     args->add(e);
   }
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 };
 
 class specExp : public exp {
@@ -816,7 +841,7 @@ public:
     s=ss;
   }
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &e);
@@ -838,7 +863,7 @@ public:
   assignExp(position pos, exp *dest, exp *value)
     : exp(pos), dest(dest), value(value) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   // Don't write the result of an assignment to the prompt.
   bool writtenToPrompt() { return false; }
@@ -859,7 +884,7 @@ public:
   selfExp(position pos, exp *dest, symbol *op, exp *value)
     : assignExp(pos, dest, value), op(op) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 };
 
 class prefixExp : public exp {
@@ -870,7 +895,7 @@ public:
   prefixExp(position pos, exp *dest, symbol *op)
     : exp(pos), dest(dest), op(op) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   bool scalable() { return false; }
 
@@ -892,7 +917,7 @@ public:
   postfixExp(position pos, exp *dest, symbol *op)
     : exp(pos), dest(dest), op(op) {}
 
-  void prettyprint(ostream &out, int indent);
+  void prettyprint(ostream &out, Int indent);
 
   types::ty *trans(coenv &e);
   types::ty *getType(coenv &) { return types::primError(); }
