@@ -13,18 +13,17 @@ namespace camp {
 string tab="\t";
 string newline="\n";
 
-ofile nullfile("");
 ofile Stdout("");
+file nullfile("",false,false,true);
 
-void ifile::ignoreComment(bool readstring)
+void ifile::ignoreComment()
 {
   if(comment == 0) return;
   int c;
   bool eol=(stream->peek() == '\n');
-  if(eol && (readstring || (csvmode && nullfield))) return;
+  if(eol && csvmode && nullfield) return;
   for(;;) {
     while(isspace(c=stream->peek())) {
-      if(c == '\n' && readstring) return;
       stream->ignore();
       whitespace += (char) c;
     }
@@ -32,10 +31,8 @@ void ifile::ignoreComment(bool readstring)
       whitespace="";
       while((c=stream->peek()) != '\n' && c != EOF)
 	stream->ignore();
-      if(c == '\n') {
-	if(readstring) return;
+      if(c == '\n')
 	stream->ignore();
-      }
     } else {if(eol) stream->unget(); return;}
   }
 }
@@ -90,29 +87,42 @@ void ifile::csv()
   std::ios::iostate rdstate=stream->rdstate();
   if(stream->fail()) stream->clear();
   int c=stream->peek();
-  if(c == ',' || (c == '\n' && !linemode)) stream->ignore();
-  else stream->clear(rdstate);
+  if(c == ',') stream->ignore();
+  else if(c == '\n') {
+    stream->ignore();
+    if(linemode && stream->peek() != EOF) stream->unget();
+  } else stream->clear(rdstate);
   if(c == ',') comma=true;
 }
   
 void ifile::Read(string& val)
 {
   string s;
-  if(csvmode) {
+  if(wordmode) {
+    whitespace="";
+    while(isspace(stream->peek())) stream->ignore();
+  }
+  if(csvmode || wordmode) {
     bool quote=false;
     while(stream->good()) {
       int c=stream->peek();
       if(c == '"') {quote=!quote; stream->ignore(); continue;}
-      if(!quote && (c == ',' || c == '\n')) {
-	if(c == '\n') ignoreComment(true);
-	break;
+      if(!quote) {
+	if(comment && c == comment) {
+	  while((c=stream->peek()) != '\n' && c != EOF)
+	    stream->ignore();
+	  if(s.empty() && c == '\n') {
+	    stream->ignore();
+	    continue;
+	  }
+       }
+	if(csvmode && (c == ',' || c == '\n'))
+	  break;
+	if(wordmode && isspace(c))
+	  break;
       }
       s += (char) stream->get();
     }
-  } else if(wordmode) {
-    s=string(); 
-    *stream >> s;
-    whitespace="";
   } else
     getline(*stream,s);
   
@@ -127,15 +137,15 @@ void ifile::Read(string& val)
 	break;
       }
     }
-    val=whitespace+s;
   }
+  val=whitespace+s;
 }
   
 void ofile::writeline() 
 {
-  if(standard && interact::interactive && !vm::indebugger) {
-    int scroll=settings::getScroll();
-    if(scroll && lines > 0 && lines % scroll == 0) {
+  if(standard && interact::query && !vm::indebugger) {
+    Int scroll=settings::getScroll();
+    if(scroll && interact::lines > 0 && interact::lines % scroll == 0) {
       for(;;) {
 	if(!cin.good()) {
 	  *stream << newline;
@@ -146,12 +156,13 @@ void ofile::writeline()
 	if(c == '\n') break;
 	// Discard any additional characters
 	while(cin.good() && cin.get() != '\n');
-	if(c == 'q') throw quit();
+	if(c == 's') {interact::query=false; break;}
+	if(c == 'q') {interact::query=false; interact::lines=0; throw quit();}
       }
     } else *stream << newline;
-    ++lines;
+    ++interact::lines;
   } else *stream << newline;
-  if(errorstream::interrupt) throw interrupted();
+  if(errorstream::interrupt) {interact::lines=0; throw interrupted();}
 }
   
 } // namespace camp
