@@ -12,15 +12,12 @@
 #include <iostream>
 #include <sstream>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "common.h"
 
 #ifdef HAVE_RPC_RPC_H
 #include "xstream.h"
 #endif
 
-#include "common.h"
 #include "pair.h"
 #include "triple.h"
 #include "guide.h"
@@ -30,6 +27,7 @@
 #include "interact.h"
 #include "errormsg.h"
 #include "util.h"
+#include "process.h"
 
 namespace vm {
   extern bool indebugger;  
@@ -40,26 +38,25 @@ namespace camp {
 extern string tab;
 extern string newline;
   
-class file : public gc_cleanup {
+class file : public gc {
 protected:  
   string name;
-  int nx,ny,nz;    // Array dimensions
+  Int nx,ny,nz;    // Array dimensions
   bool linemode;   // Array reads will stop at eol instead of eof.
   bool csvmode;    // Read comma-separated values.
   bool wordmode;   // Delimit strings by white space instead of eol.
-  bool singlemode; // Read/write single-precision XDR/binary values.
+  bool singlereal; // Read/write single-precision XDR/binary reals.
+  bool singleint;  // Read/write single-precision XDR/binary ints.
   bool closed;     // File has been closed.
-  bool checkerase; // Check input for errors/erase output.
+  bool check;      // Check whether input file exists.
   bool standard;   // Standard input/output
   bool binary;     // Read in binary mode.
-  int lines;       // Number of scrolled lines
   
   bool nullfield;  // Used to detect a final null field in csv+line mode.
   string whitespace;
+  size_t index;	   // Terminator index.
 public: 
 
-  void resetlines() {lines=0;}
-  
   bool Standard() {return standard;}
   
   void standardEOF() {
@@ -89,12 +86,14 @@ public:
       standardEOF();
   }
   
-  void dimension(int Nx=-1, int Ny=-1, int Nz=-1) {nx=Nx; ny=Ny; nz=Nz;}
+  void dimension(Int Nx=-1, Int Ny=-1, Int Nz=-1) {nx=Nx; ny=Ny; nz=Nz;}
   
-  file(const string& name, bool checkerase=true, bool binary=false) :
-    name(name), linemode(false), csvmode(false), singlemode(false),
-    closed(false), checkerase(checkerase), standard(name.empty()),
-    binary(binary), lines(0), nullfield(false), whitespace("") {dimension();}
+  file(const string& name, bool check=true, bool binary=false,
+       bool closed=false) : 
+    name(name), linemode(false), csvmode(false),
+    singlereal(false), singleint(true),
+    closed(closed), check(check), standard(name.empty()),
+    binary(binary), nullfield(false), whitespace("") {dimension();}
   
   virtual void open() {}
   
@@ -108,12 +107,14 @@ public:
   
   virtual ~file() {}
 
-  virtual const char* Mode()=0;
+  virtual const char* Mode() {return "";}
 
   bool isOpen() {
     if(closed) {
       ostringstream buf;
-      buf << "I/O operation attempted on closed file \'" << name << "\'.";
+      buf << "I/O operation attempted on ";
+      if(name != "") buf << "closed file \'" << name << "\'.";
+      else buf << "null file.";
       reportError(buf);
     }
     return true;
@@ -123,14 +124,14 @@ public:
   virtual bool eol() {return false;}
   virtual bool nexteol() {return false;}
   virtual bool text() {return false;}
-  virtual bool eof()=0;
-  virtual bool error()=0;
-  virtual void close()=0;
-  virtual void clear()=0;
-  virtual void precision(int) {}
+  virtual bool eof() {return true;}
+  virtual bool error() {return true;}
+  virtual void close() {}
+  virtual void clear() {}
+  virtual void precision(Int) {}
   virtual void flush() {}
   virtual size_t tell() {return 0;}
-  virtual void seek(int, bool=true) {}
+  virtual void seek(Int, bool=true) {}
   
   void unsupported(const char *rw, const char *type) {
     ostringstream buf;
@@ -143,7 +144,7 @@ public:
   void nowrite(const char *type) {unsupported("Write",type);}
   
   virtual void Read(bool&) {noread("bool");}
-  virtual void Read(int&) {noread("int");}
+  virtual void Read(Int&) {noread("Int");}
   virtual void Read(double&) {noread("real");}
   virtual void Read(float&) {noread("real");}
   virtual void Read(pair&) {noread("pair");}
@@ -153,7 +154,7 @@ public:
   virtual void readwhite(string&) {noread("string");}
   
   virtual void write(bool) {nowrite("bool");}
-  virtual void write(int) {nowrite("int");}
+  virtual void write(Int) {nowrite("Int");}
   virtual void write(double) {nowrite("real");}
   virtual void write(const pair&) {nowrite("pair");}
   virtual void write(const triple&) {nowrite("triple");}
@@ -163,7 +164,7 @@ public:
   virtual void write(const transform&) {nowrite("transform");}
   virtual void writeline() {nowrite("string");}
   
-  virtual void ignoreComment(bool=false) {};
+  virtual void ignoreComment() {};
   virtual void csv() {};
   
   template<class T>
@@ -191,9 +192,13 @@ public:
     }
   }
   
-  int Nx() {return nx;}
-  int Ny() {return ny;}
-  int Nz() {return nz;}
+  Int Nx() {return nx;}
+  Int Ny() {return ny;}
+  Int Nz() {return nz;}
+  
+  void Nx(Int n) {nx=n;}
+  void Ny(Int n) {ny=n;}
+  void Nz(Int n) {nz=n;}
   
   void LineMode(bool b) {linemode=b;}
   bool LineMode() {return linemode;}
@@ -204,33 +209,54 @@ public:
   void WordMode(bool b) {wordmode=b; if(b) csvmode=false;}
   bool WordMode() {return wordmode;}
   
-  void SingleMode(bool b) {singlemode=b;}
-  bool SingleMode() {return singlemode;}
+  void SingleReal(bool b) {singlereal=b;}
+  bool SingleReal() {return singlereal;}
+  
+  void SingleInt(bool b) {singleint=b;}
+  bool SingleInt() {return singleint;}
 };
 
 class ifile : public file {
 protected:  
   istream *stream;
-  std::fstream fstream;
+  std::fstream *fstream;
   char comment;
+  std::ios::openmode mode;
   bool comma;
   
 public:
-  ifile(const string& name, char comment, bool check=true)
-    : file(name,check), comment(comment), comma(false) {stream=&cin;}
+  ifile(const string& name, char comment, bool check=true,
+	std::ios::openmode mode=std::ios::in) :
+    file(name,check), stream(NULL), fstream(NULL), comment(comment), mode(mode),
+    comma(false) {
+    stream=&cin;
+  }
   
   // Binary file
-  ifile(const string& name, bool check=true) : file(name,check,true) {}
+  ifile(const string& name, bool check=true,
+	std::ios::openmode mode=std::ios::in) :
+    file(name,check,true), mode(mode) {}
   
   ~ifile() {close();}
   
   void open() {
     if(standard) {
+      if(mode & std::ios::binary) 
+	reportError("Cannot open standard input in binary mode");
       stream=&cin;
     } else {
-      fstream.open(name.c_str());
-      stream=&fstream;
-      if(checkerase) Check();
+      stream=fstream=new std::fstream(name.c_str(),mode);
+      if(mode & std::ios::out) {
+	checkLocal(name);
+	if(error()) {
+	  delete fstream;
+	  std::ofstream f(name.c_str());
+	  f.close();
+	}
+	stream=fstream=new std::fstream(name.c_str(),mode);
+      }
+      index=processData().ifile.add(fstream);
+      if(check) Check();
     }
   }
   
@@ -242,27 +268,42 @@ public:
   bool text() {return true;}
   bool eof() {return stream->eof();}
   bool error() {return stream->fail();}
-  void close() {if(!standard && !closed) {fstream.close(); closed=true;}}
-  void clear() {stream->clear();}
   
-  void seek(int pos, bool begin=true) {
-    if(!standard && !closed) {
-      clear();
-      fstream.seekg(pos,begin ? std::ios::beg : std::ios::end);
+  void close() {
+    if(!standard && fstream) {
+      fstream->close();
+      closed=true;
+      delete fstream;
+      fstream=NULL;
+      processData().ifile.remove(index);
     }
   }
   
-  size_t tell() {return fstream.tellg();}
+  void clear() {stream->clear();}
+  
+  void seek(Int pos, bool begin=true) {
+    if(!standard && fstream) {
+      clear();
+      fstream->seekg(pos,begin ? std::ios::beg : std::ios::end);
+    }
+  }
+  
+  size_t tell() {
+    if(fstream) 
+      return fstream->tellg();
+    else
+      return 0;
+  }
   
   void csv();
   
-  virtual void ignoreComment(bool readstring=false);
+  virtual void ignoreComment();
   
   // Skip over white space
   void readwhite(string& val) {val=string(); *stream >> val;}
   
   void Read(bool &val) {string t; readwhite(t); val=(t == "true");}
-  void Read(int& val) {*stream >> val;}
+  void Read(Int& val) {*stream >> val;}
   void Read(double& val) {*stream >> val;}
   void Read(pair& val) {*stream >> val;}
   void Read(triple& val) {*stream >> val;}
@@ -272,44 +313,50 @@ public:
   
 class iofile : public ifile {
 public:
-  iofile(const string& name, char comment=0) : ifile(name,true,comment) {}
+  iofile(const string& name, char comment=0) : 
+    ifile(name,true,comment,std::ios::in | std::ios::out) {}
 
-  void precision(int p) {stream->precision(p);}
-  void flush() {fstream.flush();}
+  void precision(Int p) {stream->precision(p);}
+  void flush() {if(fstream) fstream->flush();}
   
-  void write(bool val) {fstream << (val ? "true " : "false ");}
-  void write(int val) {fstream << val;}
-  void write(double val) {fstream << val;}
-  void write(const pair& val) {fstream << val;}
-  void write(const triple& val) {fstream << val;}
-  void write(const string& val) {fstream << val;}
-  void write(const pen& val) {fstream << val;}
-  void write(guide *val) {fstream << *val;}
-  void write(const transform& val) {fstream << val;}
+  void write(bool val) {*fstream << (val ? "true " : "false ");}
+  void write(Int val) {*fstream << val;}
+  void write(double val) {*fstream << val;}
+  void write(const pair& val) {*fstream << val;}
+  void write(const triple& val) {*fstream << val;}
+  void write(const string& val) {*fstream << val;}
+  void write(const pen& val) {*fstream << val;}
+  void write(guide *val) {*fstream << *val;}
+  void write(const transform& val) {*fstream << val;}
   
   void writeline() {
-    fstream << newline;
+    *fstream << newline;
     if(errorstream::interrupt) throw interrupted();
   }
 };
   
 class ofile : public file {
 protected:
-  std::ostream *stream;
-  std::ofstream fstream;
+  ostream *stream;
+  std::ofstream *fstream;
+  std::ios::openmode mode;
 public:
-  ofile(const string& name) 
-    : file(name) {stream=&cout;}
+  ofile(const string& name, std::ios::openmode mode=std::ios::trunc) :
+    file(name), stream(NULL), fstream(NULL), mode(mode) {
+    stream=&cout;
+  }
   
   ~ofile() {close();}
   
   void open() {
-    checkLocal(name);
     if(standard) {
+      if(mode & std::ios::binary) 
+	reportError("Cannot open standard output in binary mode");
       stream=&cout;
     } else {
-      fstream.open(name.c_str(),std::ios::trunc);
-      stream=&fstream;
+      checkLocal(name);
+      stream=fstream=new std::ofstream(name.c_str(),mode | std::ios::trunc);
+      index=processData().ofile.add(fstream);
       Check();
     }
   }
@@ -319,22 +366,36 @@ public:
   bool text() {return true;}
   bool eof() {return stream->eof();}
   bool error() {return stream->fail();}
-  void close() {if(!standard && !closed) {fstream.close(); closed=true;}}
+  
+  void close() {
+    if(!standard && fstream) {
+      fstream->close();
+      closed=true;
+      delete fstream;
+      fstream=NULL;
+      processData().ofile.remove(index);
+    }
+  }
   void clear() {stream->clear();}
-  void precision(int p) {stream->precision(p);}
+  void precision(Int p) {stream->precision(p);}
   void flush() {stream->flush();}
   
-  void seek(int pos, bool begin=true) {
-    if(!standard && !closed) {
+  void seek(Int pos, bool begin=true) {
+    if(!standard && fstream) {
       clear();
-      fstream.seekp(pos,begin ? std::ios::beg : std::ios::end);
+      fstream->seekp(pos,begin ? std::ios::beg : std::ios::end);
     }
   }
   
-  size_t tell() {return fstream.tellp();}
+  size_t tell() {
+    if(fstream) 
+      return fstream->tellp();
+    else
+      return 0;
+  }
   
   void write(bool val) {*stream << (val ? "true " : "false ");}
-  void write(int val) {*stream << val;}
+  void write(Int val) {*stream << val;}
   void write(double val) {*stream << val;}
   void write(const pair& val) {*stream << val;}
   void write(const triple& val) {*stream << val;}
@@ -347,58 +408,53 @@ public:
 };
 
 class ibfile : public ifile {
-protected:  
-  std::fstream fstream;
 public:
-  ibfile(const string& name, bool check=true) : ifile(name,check) {}
-
-  void open() {
-    if(standard) {
-      reportError("Cannot open standard input in binary mode");
-    } else {
-      fstream.open(name.c_str(),std::ios::binary |
-		   std::ios::in | std::ios::out);
-      stream=&fstream;
-      if(checkerase) Check();
-    }
-  }
-  
+  ibfile(const string& name, bool check=true,
+	 std::ios::openmode mode=std::ios::in) : 
+    ifile(name,check,mode | std::ios::binary) {}
   template<class T>
   void iread(T& val) {
     val=T();
-    fstream.read((char *) &val,sizeof(T));
+    fstream->read((char *) &val,sizeof(T));
   }
   
   void Read(bool& val) {iread(val);}
-  void Read(int& val) {iread(val);}
+  void Read(Int& val) {
+    if(singleint) {int ival; iread(ival); val=ival;}
+    else iread(val);
+  }
   void Read(char& val) {iread(val);}
   void Read(string& val) {iread(val);}
   
   void Read(double& val) {
-    if(singlemode) {float fval=0.0; iread(fval); val=fval;}
+    if(singlereal) {float fval; iread(fval); val=fval;}
     else iread(val);
   }
 };
   
 class iobfile : public ibfile {
 public:
-  iobfile(const string& name) : ibfile(name,true) {}
+  iobfile(const string& name) : ibfile(name,true,std::ios::in | std::ios::out)
+  {}
 
-  void flush() {fstream.flush();}
+  void flush() {if(fstream) fstream->flush();}
   
   template<class T>
   void iwrite(T val) {
-    fstream.write((char *) &val,sizeof(T));
+    fstream->write((char *) &val,sizeof(T));
   }
   
   void write(bool val) {iwrite(val);}
-  void write(int val) {iwrite(val);}
+  void write(Int val) {
+    if(singleint) iwrite(intcast(val));
+    else iwrite(val);
+  }
   void write(const string& val) {iwrite(val);}
   void write(const pen& val) {iwrite(val);}
   void write(guide *val) {iwrite(val);}
   void write(const transform& val) {iwrite(val);}
   void write(double val) {
-    if(singlemode) {float fval=val; iwrite(fval);}
+    if(singlereal) iwrite((float) val);
     else iwrite(val);
   }
   void write(const pair& val) {
@@ -415,32 +471,24 @@ public:
   
 class obfile : public ofile {
 public:
-  obfile(const string& name) : ofile(name) {}
+  obfile(const string& name) : ofile(name,std::ios::binary) {}
 
-  void open() {
-    checkLocal(name);
-    if(standard) {
-      reportError("Cannot open standard output in binary mode");
-    } else {
-      fstream.open(name.c_str(),std::ios::binary | std::ios::trunc);
-      stream=&fstream;
-      Check();
-    }
-  }
-  
   template<class T>
   void iwrite(T val) {
-    fstream.write((char *) &val,sizeof(T));
+    fstream->write((char *) &val,sizeof(T));
   }
   
   void write(bool val) {iwrite(val);}
-  void write(int val) {iwrite(val);}
+  void write(Int val) {
+    if(singleint) iwrite(intcast(val));
+    else iwrite(val);
+  }
   void write(const string& val) {iwrite(val);}
   void write(const pen& val) {iwrite(val);}
   void write(guide *val) {iwrite(val);}
   void write(const transform& val) {iwrite(val);}
   void write(double val) {
-    if(singlemode) {float fval=val; iwrite(fval);}
+    if(singlereal) iwrite((float) val);
     else iwrite(val);
   }
   void write(const pair& val) {
@@ -460,38 +508,64 @@ public:
 
 class ixfile : public file {
 protected:  
-  xdr::ioxstream stream;
+  xdr::ioxstream *fstream;
+  xdr::xios::open_mode mode;
 public:
   ixfile(const string& name, bool check=true,
 	 xdr::xios::open_mode mode=xdr::xios::in) :
-    file(name,check,true), stream(name.c_str(), mode) {
+    file(name,check,true), fstream(NULL), mode(mode) {}
+
+  void open() {
+    fstream=new xdr::ioxstream(name.c_str(),mode);
+    index=processData().ixfile.add(fstream);
     if(check) Check();
   }
-
+    
+  void close() {
+    if(fstream) {
+      fstream->close();
+      closed=true;
+      delete fstream;
+      fstream=NULL;
+      processData().ixfile.remove(index);
+    }
+  }
+  
   ~ixfile() {close();}
   
   const char* Mode() {return "xinput";}
   
-  bool eof() {return stream.eof();}
-  bool error() {return stream.fail();}
-  void close() {if(!closed) {stream.close(); closed=true;}}
-  void clear() {stream.clear();}
+  bool eof() {return fstream ? fstream->eof() : true;}
+  bool error() {return fstream ? fstream->fail() : true;}
+
+  void clear() {if(fstream) fstream->clear();}
   
-  void seek(int pos, bool begin=true) {
-    if(!standard && !closed) {
+  void seek(Int pos, bool begin=true) {
+    if(!standard && fstream) {
       clear();
-      stream.seek(pos,begin ? xdr::xios::beg : xdr::xios::end);
+      fstream->seek(pos,begin ? xdr::xios::beg : xdr::xios::end);
     }
   }
   
-  size_t tell() {return stream.tell();}
+  size_t tell() {
+    if(fstream) 
+      return fstream->tell();
+    else
+      return 0;
+  }
   
-  void Read(int& val) {val=0; stream >> val;}
+  void Read(Int& val) {
+    if(singleint) {int ival=0; *fstream >> ival; val=ival;}
+    else {
+      val=0;
+      *fstream >> val;
+    }
+  }
   void Read(double& val) {
-    if(singlemode) {float fval=0.0; stream >> fval; val=fval;}
+    if(singlereal) {float fval=0.0; *fstream >> fval; val=fval;}
     else {
       val=0.0;
-      stream >> val;
+      *fstream >> val;
     }
   }
   void Read(pair& val) {
@@ -513,12 +587,15 @@ class ioxfile : public ixfile {
 public:
   ioxfile(const string& name) : ixfile(name,true,xdr::xios::out) {}
 
-  void flush() {stream.flush();}
+  void flush() {if(fstream) fstream->flush();}
   
-  void write(int val) {stream << val;}
+  void write(Int val) {
+    if(singleint) *fstream << intcast(val);
+    else *fstream << val;
+  }
   void write(double val) {
-    if(singlemode) {float fval=val; stream << fval;}
-    else stream << val;
+    if(singlereal) *fstream << (float) val;
+    else *fstream << val;
   }
   void write(const pair& val) {
     write(val.getx());
@@ -532,36 +609,56 @@ public:
 };
   
 class oxfile : public file {
-  xdr::oxstream stream;
+  xdr::oxstream *fstream;
 public:
-  oxfile(const string& name) : 
-    file(name), stream((checkLocal(name),name.c_str()),xdr::xios::trunc) {
+  oxfile(const string& name) : file(name), fstream(NULL) {}
+
+  void open() {
+    fstream=new xdr::oxstream((checkLocal(name),name.c_str()),xdr::xios::trunc);
+    index=processData().oxfile.add(fstream);
     Check();
   }
-
+  
+  void close() {
+    if(fstream) {
+      fstream->close();
+      closed=true;
+      delete fstream;
+      fstream=NULL;
+      processData().oxfile.remove(index);
+    }
+  }
+  
   ~oxfile() {close();}
   
   const char* Mode() {return "xoutput";}
   
-  bool eof() {return stream.eof();}
-  bool error() {return stream.fail();}
-  void close() {if(!closed) {stream.close(); closed=true;}}
-  void clear() {stream.clear();}
-  void flush() {stream.flush();}
+  bool eof() {return fstream ? fstream->eof() : true;}
+  bool error() {return fstream ? fstream->fail() : true;}
+  void clear() {if(fstream) fstream->clear();}
+  void flush() {if(fstream) fstream->flush();}
   
-  void seek(int pos, bool begin=true) {
-    if(!standard && !closed) {
+  void seek(Int pos, bool begin=true) {
+    if(!standard && fstream) {
       clear();
-      stream.seek(pos,begin ? xdr::xios::beg : xdr::xios::end);
+      fstream->seek(pos,begin ? xdr::xios::beg : xdr::xios::end);
     }
   }
   
-  size_t tell() {return stream.tell();}
+  size_t tell() {
+    if(fstream) 
+      return fstream->tell();
+    else
+      return 0;
+  }
   
-  void write(int val) {stream << val;}
+  void write(Int val) {
+    if(singleint) *fstream << intcast(val);
+    else *fstream << val;
+  }
   void write(double val) {
-    if(singlemode) {float fval=val; stream << fval;}
-    else stream << val;
+    if(singlereal) *fstream << (float) val;
+    else *fstream << val;
   }
   void write(const pair& val) {
     write(val.getx());
@@ -577,7 +674,7 @@ public:
 #endif
 
 extern ofile Stdout;
-extern ofile nullfile;
+extern file nullfile;
 
 } // namespace camp
 
