@@ -16,10 +16,6 @@
 #include <sstream>
 #include <unistd.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "genv.h"
 #include "env.h"
 #include "dec.h"
@@ -66,7 +62,7 @@ record *genv::loadModule(symbol *id, string filename) {
   
   inTranslation.push_front(filename);
 
-  em->sync();
+  em.sync();
 
   record *r=ast->transAsFile(*this, id);
   
@@ -78,9 +74,9 @@ record *genv::loadModule(symbol *id, string filename) {
 void genv::checkRecursion(string filename) {
   if (find(inTranslation.begin(), inTranslation.end(), filename) !=
          inTranslation.end()) {
-    em->sync();
-    *em << "error: recursive loading of module '" << filename << "'\n";
-    em->sync();
+    em.sync();
+    em << "error: recursive loading of module '" << filename << "'\n";
+    em.sync();
     throw handled_error();
   }
 }
@@ -95,7 +91,7 @@ record *genv::getModule(symbol *id, string filename) {
     record *r=loadModule(id, filename);
     // Don't add an erroneous module to the dictionary in interactive mode, as
     // the user may try to load it again.
-    if (!interact::interactive || !em->errors())
+    if (!interact::interactive || !em.errors())
       imap[filename]=r;
 
     return r;
@@ -107,7 +103,7 @@ typedef vm::stack::importInitMap importInitMap;
 
 importInitMap *genv::getInitMap()
 {
-  struct initMap : public importInitMap {
+  struct initMap : public importInitMap, public gc {
     genv &ge;
     initMap(genv &ge)
       : ge(ge) {}
