@@ -8,16 +8,21 @@
 
 namespace camp {
 
-void multiguide::flatten(flatguide& g)
+bool multiguide::flatten(flatguide& g, bool allowsolve)
 {
-  for (size_t i=0; i<v.size(); ++i)
-    v[i]->flatten(g);
+  size_t n=v.size();
+  if(n == 0) return false;
+  bool precycle=false;
+  for(size_t i=0; i+1 < n; ++i)
+    if(v[i]->flatten(g)) precycle=true;
+  v[n-1]->flatten(g,allowsolve);
+  return precycle;
 }
 
 void multiguide::print(ostream& out) const
 {
   side lastLoc=JOIN;
-  for (size_t i=0; i<v.size(); ++i) {
+  for(size_t i=0; i < v.size(); ++i) {
     side loc=v[i]->printLocation();
     adjustLocation(out,lastLoc,loc);
     v[i]->print(out);
