@@ -9,13 +9,15 @@
 #include <iostream>
 #include <sstream>
 #include <sys/wait.h>
+#include <sys/types.h>
 #include <unistd.h>
+#include <csignal>
+
 #include "interact.h"
 
 #if defined(HAVE_LIBREADLINE) && defined(HAVE_LIBCURSES)
 #include <readline/readline.h>
 #include <readline/history.h>
-#include <csignal>
 #endif
 
 #include "util.h"
@@ -29,9 +31,12 @@ namespace run {
 
 namespace interact {
 
-int interactive=false;
+bool interactive=false;
 bool uptodate=true;
+int lines=0;  
+bool query=false;
 
+bool tty=isatty(STDIN_FILENO);  
 completer *currentCompleter=0;
 
 void setCompleter(completer *c) {
@@ -52,7 +57,7 @@ void init_completion() {
   // completed.  All characters that can't form part of an identifier are
   // treated as break characters.
   static char break_characters[128];
-  int j=0;
+  Int j=0;
   for (unsigned char c=9; c<128; ++c)
     if (!isalnum(c) && c != '_') {
       break_characters[j]=c;
@@ -63,103 +68,116 @@ void init_completion() {
 }
 #endif  
 
+char *(*Readline)(const char *prompt);
+
+char *verbatimreadline(const char *prompt)
+{
+  if(!cin.good()) {cin.clear(); return NULL;}
+  cout << prompt;
+  string s;
+  getline(cin,s);
+  return StrdupMalloc(s);
+}
+  
 void pre_readline()
 {
 #if defined(HAVE_LIBREADLINE) && defined(HAVE_LIBCURSES)
-  run::init_readline(getSetting<bool>("tabcompletion"));
-#endif  
+  if(tty) {
+    run::init_readline(getSetting<bool>("tabcompletion"));
+    Readline=readline;
+  } else
+#endif
+  Readline=verbatimreadline;
 }
 
 void init_interactive()
 {
 #if defined(HAVE_LIBREADLINE) && defined(HAVE_LIBCURSES)
-  init_completion();
-  read_history(historyname.c_str());
-#endif  
+  if(tty) {
+    init_completion();
+    read_history(historyname.c_str());
+  }
+#endif
 }
   
-#if !defined(HAVE_LIBREADLINE) || !defined(HAVE_LIBCURSES)
-char *readline(const char *prompt) {
-  if(!cin.good()) {cin.clear(); return NULL;}
-  cout << prompt;
-  string s;
-  getline(cin,s);
-  char *p=(char *) malloc(s.size()+1);
-  return strcpy(p,s.c_str());
-}
-#endif  
-  
 string simpleline(string prompt) {
   // Rebind tab key, as the setting tabcompletion may be changed at runtime.
   pre_readline();
+  
+  // Get a line from the user.
+  char *line=Readline(prompt.c_str());
 
-  /* Get a line from the user. */
-  char *line = readline(prompt.c_str());
-
-  /* Ignore keyboard interrupts while taking input. */
+  // Reset scroll count.
+  interact::lines=0;
+  interact::query=tty;
+  
+  // Ignore keyboard interrupts while taking input.
   errorstream::interrupt=false;
 
-  if (line) {
+  if(line) {
     string s=line;
     free(line);
     return s;
   } else {
     cout << endl;
+    if(!tty)
+      throw eof();
     return "\n";
   }
 }
 
 void addToHistory(string line) {
 #if defined(HAVE_LIBREADLINE) && defined(HAVE_LIBCURSES)
-    // Only add it if it has something other than newlines.
-    if (line.find_first_not_of('\n') != string::npos) {
-      add_history(line.c_str());
-    }
+  // Only add it if it has something other than newlines.
+  if(tty && line.find_first_not_of('\n') != string::npos) {
+    add_history(line.c_str());
+  }
 #endif    
 }
 
 string getLastHistoryLine() {
 #if defined(HAVE_LIBREADLINE) && defined(HAVE_LIBCURSES)
-  HIST_ENTRY *entry=history_list()[history_length-1];
-  if (!entry) {
-    em->compiler(position());
-    *em << "can't access last history line";
-    return "";
-  }
-  else {
-    return entry->line;
-  }
-#else
-  return "";
+  if(tty) {
+    HIST_ENTRY *entry=history_list()[history_length-1];
+    if(!entry) {
+      em.compiler();
+      em << "cannot access last history line";
+      return "";
+    } else 
+      return entry->line;
+  } else
 #endif
+  return "";
 }
 
 void setLastHistoryLine(string line) {
 #if defined(HAVE_LIBREADLINE) && defined(HAVE_LIBCURSES)
-  HIST_ENTRY *entry=remove_history(history_length-1);
-  if (!entry) {
-    em->compiler(position());
-    *em << "can't modify last history line";
-  }
-  else {
-    addToHistory(line);
+  if(tty) {
+    HIST_ENTRY *entry=remove_history(history_length-1);
+    if(!entry) {
+      em.compiler();
+      em << "cannot modify last history line";
+    } else {
+      addToHistory(line);
 
-    free(entry->line);
-    free(entry);
+      free(entry->line);
+      free(entry);
+    }
   }
 #endif
 }
 
 void deleteLastLine() {
 #if defined(HAVE_LIBREADLINE) && defined(HAVE_LIBCURSES)
-  HIST_ENTRY *entry=remove_history(history_length-1);
-  if (!entry) {
-    em->compiler(position());
-    *em << "can't delete last history line";
-  }
-  else {
-    free(entry->line);
-    free(entry);
+  if(tty) {
+    HIST_ENTRY *entry=remove_history(history_length-1);
+    if(!entry) {
+      em.compiler();
+      em << "cannot delete last history line";
+    } else {
+      free(entry->line);
+      free(entry);
+    }
   }
 #endif
 }
@@ -167,8 +185,10 @@ void deleteLastLine() {
 void cleanup_interactive() {
 #if defined(HAVE_LIBREADLINE) && defined(HAVE_LIBCURSES)
   // Write the history file.
-  stifle_history(getSetting<int>("historylines"));
-  write_history(historyname.c_str());
+  if(tty) {
+    stifle_history(intcast(getSetting<Int>("historylines")));
+    write_history(historyname.c_str());
+  }
 #endif
 }
 
