@@ -1,5 +1,5 @@
 /****
- * common.h
+ * memory.h
  *
  * Interface to the Boehm Garbage Collector.
  *****/
@@ -9,7 +9,6 @@
 
 #include <list>
 #include <vector>
-#include <deque>
 #include <stack>
 #include <map>
 #include <string>
@@ -19,7 +18,7 @@
 #include <ext/hash_map>
 #endif
 
-#if defined(__DECCXX_LIBCXX_RH70)
+#ifdef __DECCXX_LIBCXX_RH70
 #define CONST
 #else
 #define CONST const  
@@ -29,25 +28,39 @@
 
 #include <gc.h>
 
+#ifdef GC_DEBUG
 extern "C" {
 #include <gc_backptr.h>
 }
+#endif
+
+inline void *asy_malloc(size_t n)
+{
+#ifdef GC_DEBUG
+  if(void *mem=GC_debug_malloc_ignore_off_page(n, GC_EXTRAS))
+#else
+  if(void *mem=GC_malloc_ignore_off_page(n))
+#endif
+    return mem;
+  throw std::bad_alloc();
+}
+
+inline void *asy_malloc_atomic(size_t n)
+{
+#ifdef GC_DEBUG
+  if(void *mem=GC_debug_malloc_atomic_ignore_off_page(n, GC_EXTRAS))
+#else
+  if(void *mem=GC_malloc_atomic_ignore_off_page(n))
+#endif
+    return mem;
+  throw std::bad_alloc();
+}
 
 #undef GC_MALLOC
-# ifdef GC_DEBUG
-inline void *GC_MALLOC(size_t n) { \
-  if (void *mem=GC_debug_malloc(n, GC_EXTRAS))	\
-    return mem;                    \
-  GC_generate_random_backtrace();  \
-  throw std::bad_alloc();          \
-}
-#else  
-inline void *GC_MALLOC(size_t n) { \
-  if (void *mem=GC_malloc(n))  \
-    return mem;                    \
-  throw std::bad_alloc();          \
-}
-#endif
+#undef GC_MALLOC_ATOMIC
+
+#define GC_MALLOC(sz) asy_malloc(sz)
+#define GC_MALLOC_ATOMIC(sz) asy_malloc_atomic(sz)
 
 #include <gc_allocator.h>
 #include <gc_cpp.h>
@@ -66,8 +79,15 @@ inline void* operator new(size_t size, GCPlacement) {
   return operator new(size);
 }
 
-#define GC_MALLOC(size) ::operator new(size)
-#define GC_FREE(ptr) ::operator delete(ptr)
+inline void* operator new[](size_t size, GCPlacement) {
+  return operator new(size);
+}
+
+template<class T>
+struct GC_type_traits {};
+
+#define GC_DECLARE_PTRFREE(T) \
+template<> struct GC_type_traits<T> {}
 
 #endif // USEGC
 
@@ -75,7 +95,7 @@ namespace mem {
 
 #define GC_CONTAINER(KIND)                                               \
   template <typename T>                                                  \
-  struct KIND : public std::KIND<T, gc_allocator<T> > {                  \
+  struct KIND : public std::KIND<T, gc_allocator<T> >, public gc {       \
     KIND() : std::KIND<T, gc_allocator<T> >() {}                         \
     KIND(size_t n) : std::KIND<T, gc_allocator<T> >(n) {}                \
     KIND(size_t n, const T& t) : std::KIND<T, gc_allocator<T> >(n,t) {}  \
@@ -83,10 +103,9 @@ namespace mem {
 
 GC_CONTAINER(list);
 GC_CONTAINER(vector);
-GC_CONTAINER(deque);
 
-template <typename T, typename Container = deque<T> >
-struct stack : public std::stack<T, Container> {
+template <typename T, typename Container = vector<T> >
+struct stack : public std::stack<T, Container>, public gc {
 };
 
 #undef GC_CONTAINER
@@ -94,7 +113,7 @@ struct stack : public std::stack<T, Container> {
 #define GC_CONTAINER(KIND)                                                    \
   template <typename Key, typename T, typename Compare = std::less<Key> >     \
   struct KIND : public                                                        \
-  std::KIND<Key,T,Compare,gc_allocator<std::pair<Key,T> > > {                 \
+  std::KIND<Key,T,Compare,gc_allocator<std::pair<Key,T> > >, public gc {      \
     KIND() : std::KIND<Key,T,Compare,gc_allocator<std::pair<Key,T> > > () {}  \
   }
 
@@ -110,8 +129,10 @@ GC_CONTAINER(multimap);
             typename Hash = EXT::hash<Key>,                                   \
             typename Eq = std::equal_to<Key> >                                \
   struct KIND : public                                                        \
-  EXT::KIND<Key,T,Hash,Eq,gc_allocator<std::pair<Key, T> > > {                \
+  EXT::KIND<Key,T,Hash,Eq,gc_allocator<std::pair<Key, T> > >, public gc {     \
     KIND() : EXT::KIND<Key,T,Hash,Eq,gc_allocator<std::pair<Key, T> > > () {} \
+    KIND(size_t n)                                                            \
+      : EXT::KIND<Key,T,Hash,Eq,gc_allocator<std::pair<Key, T> > > (n) {}     \
   }
 
 GC_CONTAINER(hash_map);
@@ -122,10 +143,14 @@ GC_CONTAINER(hash_multimap);
 #endif
 
 #ifdef USEGC
-typedef std::basic_string<char,std::char_traits<char>,gc_allocator<char> > string;
-typedef std::basic_istringstream<char,std::char_traits<char>,gc_allocator<char> > istringstream;
-typedef std::basic_ostringstream<char,std::char_traits<char>,gc_allocator<char> > ostringstream;
-typedef std::basic_stringbuf<char,std::char_traits<char>,gc_allocator<char> > stringbuf;
+typedef std::basic_string<char,std::char_traits<char>,
+                          gc_allocator<char> > string;
+typedef std::basic_istringstream<char,std::char_traits<char>,
+                                 gc_allocator<char> > istringstream;
+typedef std::basic_ostringstream<char,std::char_traits<char>,
+                                 gc_allocator<char> > ostringstream;
+typedef std::basic_stringbuf<char,std::char_traits<char>,
+                             gc_allocator<char> > stringbuf;
 #else
 typedef std::string string;
 typedef std::istringstream istringstream;
@@ -133,7 +158,6 @@ typedef std::ostringstream ostringstream;
 typedef std::stringbuf stringbuf;
 #endif // USEGC
 
-
 } // namespace mem
 
-#endif 
+#endif
