@@ -12,6 +12,7 @@
 #include "util.h"
 #include "angle.h"
 #include "camperror.h"
+#include "mathop.h"
 
 namespace camp {
 
@@ -19,11 +20,17 @@ const double Fuzz=10.0*DBL_EPSILON;
 const double Fuzz2=Fuzz*Fuzz;
 const double sqrtFuzz=sqrt(Fuzz);
 
+path nullpath;
+  
 // Accurate computation of sqrt(1+x)-1.
 inline double sqrt1pxm1(double x)
 {
   return x/(sqrt(1.0+x)+1.0);
 }
+inline pair sqrt1pxm1(pair x)
+{
+  return x/(Sqrt(1.0+x)+1.0);
+}
   
 // Solve for the real roots of the quadratic equation ax^2+bx+c=0.
 quadraticroots::quadraticroots(double a, double b, double c)
@@ -56,7 +63,7 @@ quadraticroots::quadraticroots(double a, double b, double c)
     double factor=0.5*b/a;
     double x=-2.0*c/(b*factor);
     if(x > -1.0) {
-      distinct = quadraticroots::TWO;
+      distinct=quadraticroots::TWO;
       roots=2;
       double sqrtm1=sqrt1pxm1(x);
       double r2=factor*sqrtm1;
@@ -79,6 +86,33 @@ quadraticroots::quadraticroots(double a, double b, double c)
   }
 }
 
+// Solve for the complex roots of the quadratic equation ax^2+bx+c=0.
+Quadraticroots::Quadraticroots(pair a, pair b, pair c)
+{
+  if(a == 0.0) {
+    if(b != 0.0) {
+      roots=1;
+      z1=-c/b;
+    } else if(c == 0.0) {
+      roots=1;
+      z1=0.0;
+    } else
+      roots=0;
+  } else {
+    roots=2;
+    if(b == 0.0) {
+      z1=Sqrt(-c/a);
+      z2=-z1;
+    } else {
+      pair factor=0.5*b/a;
+      pair x=-2.0*c/(b*factor);
+      pair sqrtm1=sqrt1pxm1(x);
+      z1=factor*sqrtm1;
+      z2=-z1-2.0*factor;
+    }
+  }
+}
+
 inline bool goodroot(double t)
 {
   return 0.0 <= t && t <= 1.0;
@@ -125,13 +159,32 @@ cubicroots::cubicroots(double a, double b, double c, double d)
     return;
   }
   
+  // Detect roots at numerical zero.
+  if(fabs(d) <= Fuzz*(fabs(c)+fabs(b)*Fuzz+fabs(a)*Fuzz*Fuzz)) {
+    quadraticroots q(a,b,c);
+    roots=q.roots+1;
+    t1=0;
+    if(q.roots >= 1) t2=q.t1;
+    if(q.roots == 2) t3=q.t2;
+    return;
+  }
+  
   double ainv=1.0/a;
   b *= ainv; c *= ainv; d *= ainv;
   
   double b2=b*b;
-  double Q=(3.0*c-b2)*ninth;
+  double Q=3.0*c-b2;
+  if(fabs(Q) < Fuzz*(3.0*fabs(c)+fabs(b2)))
+    Q=0.0;
+  
+  double R=(3.0*Q+b2)*b-27.0*d;
+  if(fabs(R) < Fuzz*((3.0*fabs(Q)+fabs(b2))*fabs(b)+27.0*fabs(d)))
+    R=0.0;
+  
+  Q *= ninth;
+  R *= fiftyfourth;
+  
   double Q3=Q*Q*Q;
-  double R=(9.0*b*c-27.0*d-2.0*b2*b)*fiftyfourth;
   double R2=R*R;
   double D=Q3+R2;
   double mthirdb=-b*third;
@@ -159,11 +212,10 @@ cubicroots::cubicroots(double a, double b, double c, double d)
   
 pair path::point(double t) const
 {
-  emptyError();
+  checkEmpty(n);
     
-  // NOTE: there may be better methods, but let's not split hairs, yet.
-  int i = Floor(t);
-  int iplus;
+  Int i = Floor(t);
+  Int iplus;
   t = fmod(t,1);
   if (t < 0) t += 1;
 
@@ -196,11 +248,10 @@ pair path::point(double t) const
 
 pair path::precontrol(double t) const
 {
-  emptyError();
+  checkEmpty(n);
 		     
-  // NOTE: may be better methods, but let's not split hairs, yet.
-  int i = Floor(t);
-  int iplus;
+  Int i = Floor(t);
+  Int iplus;
   t = fmod(t,1);
   if (t < 0) t += 1;
 
@@ -230,11 +281,10 @@ pair path::precontrol(double t) const
  
 pair path::postcontrol(double t) const
 {
-  emptyError();
+  checkEmpty(n);
   
-  // NOTE: may be better methods, but let's not split hairs, yet.
-  int i = Floor(t);
-  int iplus;
+  Int i = Floor(t);
+  Int iplus;
   t = fmod(t,1);
   if (t < 0) t += 1;
 
@@ -263,9 +313,9 @@ pair path::postcontrol(double t) const
 
 path path::reverse() const
 {
-  solvedKnot *nodes = new solvedKnot[n];
-  int len=length();
-  for (int i = 0, j = len; i < n; i++, j--) {
+  mem::vector<solvedKnot> nodes(n);
+  Int len=length();
+  for (Int i = 0, j = len; i < n; i++, j--) {
     nodes[i].pre = postcontrol(j);
     nodes[i].point = point(j);
     nodes[i].post = precontrol(j);
@@ -274,13 +324,13 @@ path path::reverse() const
   return path(nodes, n, cycles);
 }
 
-path path::subpath(int a, int b) const
+path path::subpath(Int a, Int b) const
 {
   if(empty()) return path();
 
   if (a > b) {
     const path &rp = reverse();
-    int len=length();
+    Int len=length();
     path result = rp.subpath(len-a, len-b);
     return result;
   }
@@ -292,10 +342,10 @@ path path::subpath(int a, int b) const
       b = n-1;
   }
 
-  int sn = b-a+1;
-  solvedKnot *nodes = new solvedKnot[sn];
+  Int sn = b-a+1;
+  mem::vector<solvedKnot> nodes(sn);
 
-  for (int i = 0, j = a; j <= b; i++, j++) {
+  for (Int i = 0, j = a; j <= b; i++, j++) {
     nodes[i].pre = precontrol(j);
     nodes[i].point = point(j);
     nodes[i].post = postcontrol(j);
@@ -307,10 +357,10 @@ path path::subpath(int a, int b) const
   return path(nodes, sn);
 }
 
-inline pair split(double t, pair x, pair y) { return x+(y-x)*t; }
+inline pair split(double t, const pair& x, const pair& y) { return x+(y-x)*t; }
 
-inline void splitCubic(solvedKnot sn[], double t, solvedKnot left_,
-		       solvedKnot right_)
+inline void splitCubic(solvedKnot sn[], double t, const solvedKnot& left_,
+		       const solvedKnot& right_)
 {
   solvedKnot &left=(sn[0]=left_), &mid=sn[1], &right=(sn[2]=right_);
   pair x=split(t,left.post,right.pre);
@@ -327,7 +377,7 @@ path path::subpath(double a, double b) const
   
   if (a > b) {
     const path &rp = reverse();
-    int len=length();
+    Int len=length();
     return rp.subpath(len-a, len-b);
   }
 
@@ -343,17 +393,17 @@ path path::subpath(double a, double b) const
       if (a > n-1)
 	a = n-1;
     }
-    aL = nodes[(int)floor(a)];
-    aR = nodes[(int)ceil(a)];
-    bL = nodes[(int)floor(b)];
-    bR = nodes[(int)ceil(b)];
+    aL = nodes[(Int)floor(a)];
+    aR = nodes[(Int)ceil(a)];
+    bL = nodes[(Int)floor(b)];
+    bR = nodes[(Int)ceil(b)];
   } else {
-    if(fabs(a) > INT_MAX || fabs(b) > INT_MAX)
-      reportError("invalid path index");
-    aL = nodes[imod((int) floor(a),n)];
-    aR = nodes[imod((int) ceil(a),n)];
-    bL = nodes[imod((int) floor(b),n)];
-    bR = nodes[imod((int) ceil(b),n)];
+    if(run::validInt(a) && run::validInt(b)) {
+      aL = nodes[imod((Int) floor(a),n)];
+      aR = nodes[imod((Int) ceil(a),n)];
+      bL = nodes[imod((Int) floor(b),n)];
+      bR = nodes[imod((Int) ceil(b),n)];
+    } else reportError("invalid path index");
   }
 
   if (a == b) return path(point(a));
@@ -376,9 +426,19 @@ path path::subpath(double a, double b) const
   return p;
 }
 
+// Special case of subpath used by intersect.
+void path::halve(path &first, path &second) const
+{
+  solvedKnot sn[3];
+  splitCubic(sn,0.5,nodes[0],nodes[1]);
+  first=path(sn[0],sn[1]);
+  second=path(sn[1],sn[2]);
+}
+  
 // Calculate coefficients of Bezier derivative.
 static inline void derivative(pair& a, pair& b, pair& c,
-			      pair z0, pair z0p, pair z1m, pair z1)
+			      const pair& z0, const pair& z0p,
+			      const pair& z1m, const pair& z1)
 {
   a=z1-z0+3.0*(z0p-z1m);
   b=2.0*(z0+z1m)-4.0*z0p;
@@ -394,8 +454,8 @@ bbox path::bounds() const
     return bbox();
   }
 
-  int len=length();
-  for (int i = 0; i < len; i++) {
+  Int len=length();
+  for (Int i = 0; i < len; i++) {
     box += point(i);
     if(straight(i)) continue;
     
@@ -426,8 +486,8 @@ bbox path::bounds(double min, double max) const
   
   static pair I(0,1);
   
-  int len=length();
-  for (int i = 0; i < len; i++) {
+  Int len=length();
+  for (Int i = 0; i < len; i++) {
     pair v=I*dir(i);
     box += point(i)+min*v;
     box += point(i)+max*v;
@@ -477,8 +537,8 @@ bbox path::internalbounds(const bbox& padding) const
   bbox box;
   
   // Check interior nodes.
-  int len=length();
-  for (int i = 1; i < len; i++) {
+  Int len=length();
+  for (Int i = 1; i < len; i++) {
     
     pair pre=point(i)-precontrol(i);
     pair post=postcontrol(i)-point(i);
@@ -499,7 +559,7 @@ bbox path::internalbounds(const bbox& padding) const
   }
 			      
   // Check interior segments.
-  for (int i = 0; i < len; i++) {
+  for (Int i = 0; i < len; i++) {
     if(straight(i)) continue;
     
     pair a,b,c;
@@ -542,11 +602,13 @@ static double ds(double t)
 {
   double dx=quadratic(a.getx(),b.getx(),c.getx(),t);
   double dy=quadratic(a.gety(),b.gety(),c.gety(),t);
+//  cout << t << " " << sqrt(dx*dx+dy*dy) << endl;
   return sqrt(dx*dx+dy*dy);
 }
 
 // Calculates arclength of a cubic using adaptive simpson integration.
-double cubiclength(pair z0, pair z0p, pair z1m, pair z1, double goal=-1)
+double cubiclength(const pair& z0, const pair& z0p,
+		   const pair& z1m, const pair& z1, double goal=-1)
 {
   double L,integral;
   derivative(a,b,c,z0,z0p,z1m,z1);
@@ -557,9 +619,9 @@ double cubiclength(pair z0, pair z0p, pair z1m, pair z1, double goal=-1)
   if(goal < 0 || goal >= L) return L;
   
   static const double third=1.0/3.0;
+  double t=goal/L;
   goal *= third;
-  double t=0.5;
-  if(!unsimpson(goal,ds,0.0,t,10.0*DBL_EPSILON,integral,1.0))
+  if(!unsimpson(goal,ds,0.0,t,10.0*DBL_EPSILON,integral,1.0,sqrt(DBL_EPSILON)))
     reportError("nesting capacity exceeded in computing arctime");
   return -t;
 }
@@ -568,7 +630,7 @@ double path::arclength() const {
   if (cached_length != -1) return cached_length;
 
   double L=0.0;
-  for (int i = 0; i < n-1; i++) {
+  for (Int i = 0; i < n-1; i++) {
     L += cubiclength(point(i),postcontrol(i),precontrol(i+1),point(i+1));
   }
   if(cycles) L += cubiclength(point(n-1),postcontrol(n-1),precontrol(n),
@@ -587,7 +649,7 @@ double path::arctime(double goal) const {
       return result;
     }
     if (cached_length > 0 && goal >= cached_length) {
-      int loops = (int)(goal / cached_length);
+      Int loops = (Int)(goal / cached_length);
       goal -= loops*cached_length;
       return loops*n+arctime(goal);
     }      
@@ -599,7 +661,7 @@ double path::arctime(double goal) const {
   }
     
   double l,L=0;
-  for (int i = 0; i < n-1; i++) {
+  for (Int i = 0; i < n-1; i++) {
     l = cubiclength(point(i),postcontrol(i),precontrol(i+1),point(i+1),goal);
     if (l < 0)
       return (-l+i);
@@ -663,12 +725,12 @@ inline double cubicDir(const solvedKnot& left, const solvedKnot& right,
 
 // TODO: Check that we handle corner cases.
 // Velocity(t) == (0,0)
-double path::directiontime(pair dir) const {
+double path::directiontime(const pair& dir) const {
   if (dir == pair(0,0)) return 0;
   pair rot = pair(1,0)/unit(dir);
     
   double t; double pre,post;
-  for (int i = 0; i < n-1+cycles; ) {
+  for (Int i = 0; i < n-1+cycles; ) {
     t = cubicDir(this->nodes[i],(cycles && i==n-1) ? nodes[0]:nodes[i+1],rot);
     if (t >= 0) return i+t;
     i++;
@@ -692,104 +754,210 @@ double path::directiontime(pair dir) const {
 
 // {{{ Path Intersection Calculation
 
-static unsigned count;  
-unsigned maxIntersectCount=100000;
+const unsigned maxdepth=DBL_MANT_DIG;
+const unsigned mindepth=maxdepth-16;
 
-// Algorithm derived from Knuth's MetaFont
-bool intersectcubics(pair &t, solvedKnot left1, solvedKnot right1,
-                     solvedKnot left2, solvedKnot right2,
-		     double fuzz, unsigned depth=DBL_MANT_DIG)
+bool intersect(double& S, double& T, path& p, path& q, double fuzz,
+	       unsigned depth)
 {
-  bbox box1, box2;
-  box1 += left1.point; box1 += left1.post;
-  box1 += right1.pre;  box1 += right1.point;
-  box2 += left2.point; box2 += left2.post;
-  box2 += right2.pre;  box2 += right2.point;
+  if(errorstream::interrupt) throw interrupted();
   
-  double lambda=box1.diameter()+box2.diameter();
+  pair maxp=p.max();
+  pair minp=p.min();
+  pair maxq=q.max();
+  pair minq=q.min();
   
-  if(box1.Max().getx()+fuzz >= box2.Min().getx() &&
-     box1.Max().gety()+fuzz >= box2.Min().gety() &&
-     box2.Max().getx()+fuzz >= box1.Min().getx() &&
-     box2.Max().gety()+fuzz >= box1.Min().gety()) {
-    if(lambda <= fuzz || depth == 0 || count == 0) {
-      t=pair(0,0);
-      return true;
-    }
+  if(maxp.getx()+fuzz >= minq.getx() &&
+     maxp.gety()+fuzz >= minq.gety() && 
+     maxq.getx()+fuzz >= minp.getx() &&
+     maxq.gety()+fuzz >= minp.gety()) {
+    // Overlapping bounding boxes
+
     --depth;
-    --count;
-    solvedKnot sn1[3], sn2[3];
-    splitCubic(sn1,0.5,left1,right1);
-    splitCubic(sn2,0.5,left2,right2);
-    pair T;
-    if(intersectcubics(T,sn1[0],sn1[1],sn2[0],sn2[1],fuzz,depth)) {
-      t=T*0.5;
+    if((maxp-minp).length()+(maxq-minq).length() <= fuzz || depth == 0) {
+      S=0;
+      T=0;
       return true;
     }
-    if(intersectcubics(T,sn1[0],sn1[1],sn2[1],sn2[2],fuzz,depth)) {
-      t=T*0.5+pair(0,1);
+    
+    Int lp=p.length();
+    path p1,p2;
+    double pscale,poffset;
+    
+    if(lp == 1) {
+      p.halve(p1,p2);
+      pscale=poffset=0.5;
+    } else {
+      Int tp=lp/2;
+      p1=p.subpath(0,tp);
+      p2=p.subpath(tp,lp);
+      poffset=tp;
+      pscale=1.0;
+    }
+      
+    Int lq=q.length();
+    path q1,q2;
+    double qscale,qoffset;
+    
+    if(lq == 1) {
+      q.halve(q1,q2);
+      qscale=qoffset=0.5;
+    } else {
+      Int tq=lq/2;
+      q1=q.subpath(0,tq);
+      q2=q.subpath(tq,lq);
+      qoffset=tq;
+      qscale=1.0;
+    }
+      
+    if(intersect(S,T,p1,q1,fuzz,depth)) {
+      S=S*pscale;
+      T=T*qscale;
       return true;
     }
-    if(intersectcubics(T,sn1[1],sn1[2],sn2[0],sn2[1],fuzz,depth)) {
-      t=T*0.5+pair(1,0);
+    if(intersect(S,T,p1,q2,fuzz,depth)) {
+      S=S*pscale;
+      T=T*qscale+qoffset;
       return true;
     }
-    if(intersectcubics(T,sn1[1],sn1[2],sn2[1],sn2[2],fuzz,depth)) {
-      t=T*0.5+pair(1,1);
+    if(intersect(S,T,p2,q1,fuzz,depth)) {
+      S=S*pscale+poffset;
+      T=T*qscale;
+      return true;
+    }
+    if(intersect(S,T,p2,q2,fuzz,depth)) {
+      S=S*pscale+poffset;
+      T=T*qscale+qoffset;
       return true;
     }
   }
   return false;
 }
 
-// TODO: Handle corner cases. (Done I think)
-bool intersect(pair &t, path p1, path p2, double fuzz=0.0)
+void add(std::vector<double>& S, std::vector<double>& T, double s, double t,
+	 const path& p, const path& q, double fuzz)
 {
-  fuzz=max(fuzz,Fuzz*max(max(length(p1.max()),length(p1.min())),
-			 max(length(p2.max()),length(p2.min()))));
-  solvedKnot *n1=p1.Nodes();
-  solvedKnot *n2=p2.Nodes();
-  int L1=p1.length();
-  int L2=p2.length();
-  int icycle=p1.cyclic() ? p1.size()-1 : -1;
-  int jcycle=p2.cyclic() ? p2.size()-1 : -1;
-  if(p1.size() == 1) {L1=1; icycle=0;}
-  if(p2.size() == 1) {L2=1; jcycle=0;}
-  for(int i=0; i < L1; ++i) {
-    solvedKnot& left1=n1[i];
-    solvedKnot& right1=(i == icycle) ? n1[0] : n1[i+1];
-    for(int j=0; j < L2; ++j) {
-      count=maxIntersectCount;
-      pair T;
-      if(intersectcubics(T,left1,right1,
-			 n2[j],(j == jcycle) ? n2[0] : n2[j+1],fuzz)) {
-	t=T*0.5+pair(i,j);
-	return true;
-      }
-    }
-  }
-  return false;  
+  for(unsigned i=0; i < S.size(); ++i)
+    if((p.point(S[i])-p.point(s)).length() <= fuzz &&
+       (q.point(T[i])-q.point(t)).length() <= fuzz) return;
+  S.push_back(s);
+  T.push_back(t);
 }
+  
+void add(std::vector<double>& S, std::vector<double>& T,
+	 std::vector<double>& S1, std::vector<double>& T1,
+	 double pscale, double qscale, double poffset, double qoffset,
+	 const path& p, const path& q, double fuzz)
+{
+  fuzz *= 2.0;
+  for(unsigned i=0; i < S1.size(); ++i)
+    add(S,T,pscale*S1[i]+poffset,qscale*T1[i]+qoffset,p,q,fuzz);
+}
+
+void intersections(std::vector<double>& S, std::vector<double>& T,
+		   path& p, path& q, double fuzz, unsigned depth)
+{
+  if(errorstream::interrupt) throw interrupted();
+  
+  pair maxp=p.max();
+  pair minp=p.min();
+  pair maxq=q.max();
+  pair minq=q.min();
+  
+  if(maxp.getx()+fuzz >= minq.getx() &&
+     maxp.gety()+fuzz >= minq.gety() && 
+     maxq.getx()+fuzz >= minp.getx() &&
+     maxq.gety()+fuzz >= minp.gety()) {
+    // Overlapping bounding boxes
+
+    --depth;
+    if((maxp-minp).length()+(maxq-minq).length() <= fuzz || depth == 0) {
+      S.push_back(0.0);
+      T.push_back(0.0);
+      return;
+    }
+    
+    Int lp=p.length();
+    path p1,p2;
+    double pscale,poffset;
+    
+    if(lp <= 1) {
+      p.halve(p1,p2);
+      pscale=poffset=0.5;
+    } else {
+      Int tp=lp/2;
+      p1=p.subpath(0,tp);
+      p2=p.subpath(tp,lp);
+      poffset=tp;
+      pscale=1.0;
+    }
+      
+    Int lq=q.length();
+    path q1,q2;
+    double qscale,qoffset;
+    
+    if(lq <= 1) {
+      q.halve(q1,q2);
+      qscale=qoffset=0.5;
+    } else {
+      Int tq=lq/2;
+      q1=q.subpath(0,tq);
+      q2=q.subpath(tq,lq);
+      qoffset=tq;
+      qscale=1.0;
+    }
+      
+    std::vector<double> S1,T1;
+    intersections(S1,T1,p1,q1,fuzz,depth);
+    add(S,T,S1,T1,pscale,qscale,0.0,0.0,p,q,fuzz);
+
+    if(depth <= mindepth && S1.size() > 0)
+      return;
+    
+    S1.clear();
+    T1.clear();
+    intersections(S1,T1,p1,q2,fuzz,depth);
+    add(S,T,S1,T1,pscale,qscale,0.0,qoffset,p,q,fuzz);
+    
+    if(depth <= mindepth && S1.size() > 0)
+      return;
+    
+    S1.clear();
+    T1.clear();
+    intersections(S1,T1,p2,q1,fuzz,depth);
+    add(S,T,S1,T1,pscale,qscale,poffset,0.0,p,q,fuzz);
+    
+    if(depth <= mindepth && S1.size() > 0)
+      return;
+    
+    S1.clear();
+    T1.clear();
+    intersections(S1,T1,p2,q2,fuzz,depth);
+    add(S,T,S1,T1,pscale,qscale,poffset,qoffset,p,q,fuzz);
+  }
+}
+
 // }}}
 
-ostream& operator<< (ostream& out, const path p)
+ostream& operator<< (ostream& out, const path& p)
 {
   size_t oldPrec = out.precision(6);
   
-  int n = p.n;
+  Int n = p.n;
   switch(n) {
   case 0:
     out << "<nullpath>";
     break;
     
   case 1:
-    out << p.point(0);
+    out << p.point((Int) 0);
     break;
 
   default:
-    out << p.point(0) << ".. controls " << p.postcontrol(0) << " and ";
+    out << p.point((Int) 0) << ".. controls " << p.postcontrol((Int) 0) 
+	<< " and ";
 
-    for (int i = 1; i < n-1; i++) {
+    for (Int i = 1; i < n-1; i++) {
       out << p.precontrol(i) << newl;
 
       out << " .." << p.point(i);
@@ -802,7 +970,7 @@ ostream& operator<< (ostream& out, const path p)
 
     if (p.cycles) 
       out << ".. controls " << p.postcontrol(n-1) << " and "
-	  << p.precontrol(0) << newl
+	  << p.precontrol((Int) 0) << newl
 	  << " ..cycle";
     break;
   }
@@ -812,29 +980,29 @@ ostream& operator<< (ostream& out, const path p)
   return out;
 }
 
-path concat(path p1, path p2)
+path concat(const path& p1, const path& p2)
 {
-  int n1 = p1.length(), n2 = p2.length();
+  Int n1 = p1.length(), n2 = p2.length();
 
   if (n1 == -1) return p2;
   if (n2 == -1) return p1;
   pair a=p1.point(n1);
-  pair b=p2.point(0);
-  if ((a-b).abs2() > Fuzz*max(a.abs2(),b.abs2()))
+  pair b=p2.point((Int) 0);
+  if ((a-b).abs2() > Fuzz2*max(a.abs2(),b.abs2()))
     reportError("paths in concatenation do not meet");
 
-  solvedKnot *nodes = new solvedKnot[n1+n2+1];
+  mem::vector<solvedKnot> nodes(n1+n2+1);
 
-  int i = 0;
-  nodes[0].pre = p1.point(0);
-  for (int j = 0; j < n1; j++) {
+  Int i = 0;
+  nodes[0].pre = p1.point((Int) 0);
+  for (Int j = 0; j < n1; j++) {
     nodes[i].point = p1.point(j);
     nodes[i].straight = p1.straight(j);
     nodes[i].post = p1.postcontrol(j);
     nodes[i+1].pre = p1.precontrol(j+1);
     i++;
   }
-  for (int j = 0; j < n2; j++) {
+  for (Int j = 0; j < n2; j++) {
     nodes[i].point = p2.point(j);
     nodes[i].straight = p2.straight(j);
     nodes[i].post = p2.postcontrol(j);
@@ -847,7 +1015,7 @@ path concat(path p1, path p2)
 }
 
 // Increment count if the path has a vertical component at t.
-bool path::Count(int& count, double t) const
+bool path::Count(Int& count, double t) const
 {
   pair z=point(t);
   pair Pre=z-precontrol(t);
@@ -856,13 +1024,13 @@ bool path::Count(int& count, double t) const
   double post=unit(Post).gety();
   if(pre == 0.0 && Pre != pair(0.0,0.0)) pre=post;
   if(post == 0.0 && Post != pair(0.0,0.0)) post=pre;
-  int incr=(pre*post > Fuzz) ? sgn1(pre) : 0;
+  Int incr=(pre*post > Fuzz) ? sgn1(pre) : 0;
   count += incr;
   return incr != 0.0;
 }
   
 // Count if t is in (begin,end] and z lies to the left of point(i+t).
-void path::countleft(int& count, double x, int i, double t, double begin,
+void path::countleft(Int& count, double x, Int i, double t, double begin,
 		     double end, double& mint, double& maxt) const 
 {
   if(t > -Fuzz && t < Fuzz) t=0;
@@ -874,11 +1042,11 @@ void path::countleft(int& count, double x, int i, double t, double begin,
 
 // Return the winding number of the region bounded by the (cyclic) path
 // relative to the point z.
-int path::windingnumber(const pair& z) const
+Int path::windingnumber(const pair& z) const
 {
   if(!cycles)
     reportError("path is not cyclic");
-  int count=0;
+  Int count=0;
   
   double x=z.getx();
   double y=z.gety();
@@ -891,7 +1059,7 @@ int path::windingnumber(const pair& z) const
   if(z.getx() < b.left || z.getx() > b.right ||
      z.gety() < b.bottom || z.gety() > b.top) return 0;
   
-  for(int i=0; i < n; ++i) {
+  for(Int i=0; i < n; ++i) {
     pair a=point(i);
     pair d=point(i+1);
       
@@ -932,9 +1100,9 @@ int path::windingnumber(const pair& z) const
 
 path path::transformed(const transform& t) const
 {
-  solvedKnot *nodes = new solvedKnot[n];
+  mem::vector<solvedKnot> nodes(n);
 
-  for (int i = 0; i < n; ++i) {
+  for (Int i = 0; i < n; ++i) {
     nodes[i].pre = t * this->nodes[i].pre;
     nodes[i].point = t * this->nodes[i].point;
     nodes[i].post = t * this->nodes[i].post;
@@ -945,12 +1113,12 @@ path path::transformed(const transform& t) const
   return p;
 }
 
-path transformed(const transform& t, path p)
+path transformed(const transform& t, const path& p)
 {
-  int n = p.size();
-  solvedKnot *nodes = new solvedKnot[n];
+  Int n = p.size();
+  mem::vector<solvedKnot> nodes(n);
 
-  for (int i = 0; i < n; ++i) {
+  for (Int i = 0; i < n; ++i) {
     nodes[i].pre = t * p.precontrol(i);
     nodes[i].point = t * p.point(i);
     nodes[i].post = t * p.postcontrol(i);
@@ -960,4 +1128,45 @@ path transformed(const transform& t, path p)
   return path(nodes, n, p.cyclic());
 }
 
+path nurb(pair z0, pair z1, pair z2, pair z3,
+	  double w0, double w1, double w2, double w3, Int m)
+{
+  mem::vector<solvedKnot> nodes(m+1);
+
+  if(m < 1) reportError("invalid sampling interval");
+
+  double step=1.0/m;
+  for(Int i=0; i <= m; ++i) { 
+    double t=i*step;
+    double t2=t*t;
+    double onemt=1.0-t;
+    double onemt2=onemt*onemt;
+    double W0=w0*onemt2*onemt;
+    double W1=w1*3.0*t*onemt2;
+    double W2=w2*3.0*t2*onemt;
+    double W3=w3*t2*t;
+    nodes[i].point=(W0*z0+W1*z1+W2*z2+W3*z3)/(W0+W1+W2+W3);
+  }
+  
+  static const double onethird=1.0/3.0;
+  static const double twothirds=2.0/3.0;
+  pair z=nodes[0].point;
+  nodes[0].pre=z;
+  nodes[0].post=twothirds*z+onethird*nodes[1].point;
+  for(int i=1; i < m; ++i) {
+    pair z0=nodes[i].point;
+    pair zm=nodes[i-1].point;
+    pair zp=nodes[i+1].point;
+    pair pre=twothirds*z0+onethird*zm;
+    pair pos=twothirds*z0+onethird*zp;
+    pair dir=unit(pos-pre);
+    nodes[i].pre=z0-length(z0-pre)*dir;
+    nodes[i].post=z0+length(pos-z0)*dir;
+  }
+  z=nodes[m].point;
+  nodes[m].pre=twothirds*z+onethird*nodes[m-1].point;
+  nodes[m].post=z;
+  return path(nodes,m+1);
+}
+
 } //namespace camp
