@@ -16,20 +16,13 @@
 
 namespace camp {
 
-class texstream : public iopipestream {
-public:
-  void pipeclose();
-};
-extern texstream tex; // Bi-directional pipe to latex (to find label bbox)
-
-typedef mem::list<drawElement*> nodelist;
-  
 class picture : public gc {
 private:
   bool labels;
   size_t lastnumber;
-  transform *T; // Keep track of accumlative picture transform
+  transform T; // Keep track of accumulative picture transform
   bbox b;
+  bbox b_cached;   // Cached bounding box
   boxvector labelbounds;
   bboxlist bboxstack;
   
@@ -37,9 +30,10 @@ private:
   static double paperWidth,paperHeight;
 
 public:
+  typedef mem::list<drawElement*> nodelist;
   nodelist nodes;
   
-  picture() : labels(false), lastnumber(0), T(0) {}
+  picture() : labels(false), lastnumber(0), T(identity) {}
   
   // Destroy all of the owned picture objects.
   ~picture();
@@ -68,12 +62,13 @@ public:
 		  const string& prefix, const pair& bboxshift); 
     
   bool postprocess(const string& prename, const string& outname, 
-		   const string& outputformat, bool wait, bool view);
+		   const string& outputformat, double magnification,
+		   bool wait, bool view);
     
   // Ship the picture out to PostScript & TeX files.
   bool shipout(picture* preamble, const string& prefix,
-	       const string& format, bool wait, bool view=true,
-	       bool Delete=false);
+	       const string& format, double magnification=0.0,
+	       bool wait=false, bool view=true);
  
   picture *transformed(const transform& t);
   
