@@ -4,7 +4,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -20,6 +20,7 @@
 #define PIPESTREAM_H
 
 #include <iostream>
+#include <cstring>
 #include <sys/wait.h>
 #include <cerrno>
 #include <sstream>
@@ -47,7 +48,6 @@ protected:
 public:
   
   void clear() {
-    if(last) delete last;
     last=new string(buffer);
     *buffer=0;
   }
@@ -98,6 +98,8 @@ public:
 	char **argv=args(command);
 	if(argv) execvp(argv[0],argv);
 	execError(command,hint,application);
+	kill(wrapperpid,SIGTERM);
+	exit(-1);
       }
       exit(0);
     } else {
