@@ -25,12 +25,17 @@
 
 #include "process.h"
 
-int signalCount=0;
+namespace camp {
+  pen& defaultpen() {
+    return processData().defaultpen;
+  }
+}
 
 namespace run {
   void cleanup();
   void exitFunction(vm::stack *Stack);
   void updateFunction(vm::stack *Stack);
+  void purge();
 }
 
 namespace vm {
@@ -49,27 +54,24 @@ using types::record;
 using interact::interactive;
 using interact::uptodate;
 
+using absyntax::runnable;
+using absyntax::block;
+
+mem::stack<processDataStruct *> processDataStack;
+
+processDataStruct &processData() {
+  assert(!processDataStack.empty());
+  return *processDataStack.top();
+}
+
 void init(bool resetpath=true)
 {
   vm::indebugger=false;
   if(resetpath)
-    setPath("");  /* On second and subsequent calls to init, sets the
-		     path to what it was when the program started. */
-  ShipoutNumber=0;
-  if(!em)
-    em = new errorstream();
+    setPath("");  /* On second and subsequent calls, sets the path
+		     to what it was when the program started. */
 }
 
-void purge()
-{
-#ifdef USEGC
-  GC_gcollect();
-#endif
-}
-
-using absyntax::runnable;
-using absyntax::block;
-
 // This helper class does nothing but call the interactiveTrans method of the
 // base object in place of trans, so that the runnable can exhibit special
 // behaviour when run at the interactive prompt.
@@ -80,7 +82,7 @@ public:
   interactiveRunnable(runnable *base)
     : runnable(base->getPos()), base(base) {}
 
-  void prettyprint(ostream &out, int indent) {
+  void prettyprint(ostream &out, Int indent) {
     absyntax::prettyname(out, "interactiveRunnable", indent);
     base->prettyprint(out, indent+1);
   }
@@ -109,8 +111,8 @@ bool runRunnable(runnable *r, coenv &e, istack &s, transMode tm=TRANS_NORMAL) {
   lambda *codelet= tm==TRANS_INTERACTIVE ?
                        interactiveRunnable(r).transAsCodelet(e) :
                        r->transAsCodelet(e);
-  em->sync();
-  if(!em->errors()) {
+  em.sync();
+  if(!em.errors()) {
     if(getSetting<bool>("translate")) print(cout,codelet->code);
     s.run(codelet);
 
@@ -120,7 +122,7 @@ bool runRunnable(runnable *r, coenv &e, istack &s, transMode tm=TRANS_NORMAL) {
     e.e.endScope(); // Remove any changes to the environment.
 
     // Should an interactive error hurt the status?
-    em->statusError();
+    em.statusError();
 
     return false;   
   }
@@ -132,7 +134,6 @@ void runAutoplain(coenv &e, istack &s) {
   runRunnable(r,e,s);
 }
 
-
 // Abstract base class for the core object being run in line-at-a-time mode, it
 // may be a block of code, file, or interactive prompt.
 struct icore {
@@ -141,16 +142,12 @@ struct icore {
   virtual void doParse() = 0;
   virtual void doList() = 0;
 
-protected:
-  // NOTE: Get this out of here!
-  mem::list<string> TeXpipepreamble_save;
-  mem::list<string> TeXpreamble_save;
 public:
 
+  // preRun and postRun are the optional activities that take place before and
+  // after running the code specified.  They can be overridden by derived class
+  // that wish different behaviour.
   virtual void preRun(coenv &e, istack &s) {
-    TeXpipepreamble_save = mem::list<string>(camp::TeXpipepreamble);
-    TeXpreamble_save = mem::list<string>(camp::TeXpreamble);
-
     if(getSetting<bool>("autoplain"))
       runAutoplain(e,s);
   }
@@ -159,19 +156,16 @@ public:
 
   virtual void postRun(coenv &, istack &s) {
     run::exitFunction(&s);
-    camp::TeXpipepreamble=TeXpipepreamble_save;
-    camp::TeXpreamble=TeXpreamble_save;
   }
 
-  virtual void doRun() {
-    assert(em);
-    em->sync();
-    if(em->errors())
+  virtual void doRun(bool purge=false, transMode tm=TRANS_NORMAL) {
+    em.sync();
+    if(em.errors())
       return;
 
     try {
-      purge();
-
+      if(purge) run::purge();
+      
       genv ge;
       env base_env(ge);
       coder base_coder;
@@ -183,29 +177,40 @@ public:
 
       preRun(e,s);
 
+      if(purge) run::purge();
+
       // Now that everything is set up, run the core.
-      run(e,s);
+      run(e,s,tm);
 
       postRun(e,s);
 
     } catch(std::bad_alloc&) {
       cerr << "error: out of memory" << endl;
-      em->statusError();
+      em.statusError();
+    } catch(quit) {
     } catch(handled_error) {
-      em->statusError();
-      run::cleanup();
+      em.statusError();
     }
-
-    em->clear();
+    
+    run::cleanup();
+    
+    em.clear();
   }
 
-  virtual void process() {
+  virtual void process(bool purge=false) {
     if (!interactive && getSetting<bool>("parseonly"))
       doParse();
     else if (getSetting<bool>("listvariables"))
       doList();
-    else
-      doRun();
+    else {
+      // This is not done in preRun as it is not an optional step.
+      processDataStruct data;
+      processDataStack.push(&data);
+
+      doRun(purge);
+
+      processDataStack.pop();
+    }
   }
 };
 
@@ -226,7 +231,7 @@ public:
       try {
         cachedTree=buildTree();
       } catch(handled_error) {
-        em->statusError();
+        em.statusError();
         return 0;
       }
     }
@@ -239,8 +244,8 @@ public:
 
   void doParse() {
     block *tree=getTree();
-    em->sync();
-    if(tree && !em->errors())
+    em.sync();
+    if(tree && !em.errors())
       tree->prettyprint(cout, 0);
   }
 
@@ -258,15 +263,21 @@ public:
     if (tree) {
       for(mem::list<runnable *>::iterator r=tree->stms.begin();
           r != tree->stms.end(); ++r)
-        if(!em->errors() || getSetting<bool>("debug"))
+        if(!em.errors() || getSetting<bool>("debug"))
           runRunnable(*r,e,s,tm);
     }
   }
 
-  void doRun() {
+  void doRun(transMode tm=TRANS_NORMAL) {
     // Don't prepare an environment to run the code if there isn't any code.
     if (getTree()) {
-      icore::doRun();
+      // This is not done in preRun as it is not an optional step.
+      processDataStruct data;
+      processDataStack.push(&data);
+
+      icore::doRun(false,tm);
+
+      processDataStack.pop();
     }
   }
 };
@@ -308,13 +319,13 @@ public:
 			stripDir(stripExt(string(filename), suffix)))) {}
   
   block *buildTree() {
-    return filename!="" ? parser::parseFile(filename) : 0;
+    return !filename.empty() ? parser::parseFile(filename,"Loading") : 0;
   }
 
   void preRun(coenv& e, istack& s) {
     outname_save=getSetting<string>("outname");
-    if(outname_save.empty())
-      Setting("outname")=outname;
+    if(stripDir(outname_save).empty())
+      Setting("outname")=outname_save+outname;
 
     itree::preRun(e, s);
   }
@@ -325,7 +336,7 @@ public:
     Setting("outname")=outname_save;
   }
 
-  void process() {
+  void process(bool purge=false) {
     try {
       init();
     } catch(handled_error) {
@@ -335,16 +346,15 @@ public:
       cout << "Processing " << outname << endl;
     
     try {
-      itree::process();
+      icore::process(purge);
     }
     catch(handled_error) {
-      em->statusError();
+      em.statusError();
     }
   }
 };
 
 void printGreeting() {
-  if(!getSetting<bool>("quiet"))
     cout << "Welcome to " << PROGRAM << " version " << VERSION
 	 << " (to view the manual, type help)" << endl;
 }
@@ -526,7 +536,6 @@ string cleanLine(const string line) {
   return endCommentOrString(deslash(line));
 }
 
-
 class iprompt : public icore {
   // Flag that is set to false to signal the prompt to exit.
   bool running;
@@ -541,9 +550,6 @@ class iprompt : public icore {
   protoenv *einteractive;
   
   void postRun(coenv &, istack &) {
-    run::cleanup();
-    camp::TeXpipepreamble=TeXpipepreamble_save;
-    camp::TeXpreamble=TeXpreamble_save;
   }
   
   // Commands are chopped into the starting word and the rest of the line.
@@ -601,9 +607,9 @@ class iprompt : public icore {
   typedef mem::map<CONST string, command> commandMap;
   commandMap commands;
 
-  bool quit(commandLine cl) {
+  bool exit(commandLine cl) {
     if (cl.simple()) {
-      // Don't store quit commands in the history file.
+      // Don't store exit commands in the history file.
       interact::deleteLastLine();
 
       running=false;
@@ -614,7 +620,7 @@ class iprompt : public icore {
 
   bool q(commandLine cl) {
     if(einteractive->ve.getType(symbol::trans("q"))) return false;
-    return quit(cl);
+    return exit(cl);
   }
 
   bool reset(commandLine cl) {
@@ -624,7 +630,7 @@ class iprompt : public icore {
       startline="";
 
       uptodate=true;
-      purge();
+      run::purge();
 
       return true;
     }
@@ -655,9 +661,9 @@ class iprompt : public icore {
 
     // keywords.pl looks for ADDCOMMAND to identify special commands in the
     // auto-completion.
-    ADDCOMMAND(quit,quit);
+    ADDCOMMAND(quit,exit);
     ADDCOMMAND(q,q);
-    ADDCOMMAND(exit,quit);
+    ADDCOMMAND(exit,exit);
     ADDCOMMAND(reset,reset);
     ADDCOMMAND(help, help);
     ADDCOMMAND(input, input);
@@ -692,6 +698,11 @@ class iprompt : public icore {
     interact::setLastHistoryLine(cleanLine(last)+line);
   }
 
+  void terminateLastHistoryLine() {
+    string last=interact::getLastHistoryLine();
+    interact::setLastHistoryLine(terminateLine(last));
+  }
+
   // Get input from the interactive prompt.  Such input may be over several
   // user-typed lines if he/she ends a line a with backslash to continue input
   // on the next line.  If continuation is true, the input started on a previous
@@ -733,6 +744,14 @@ class iprompt : public icore {
         i.run(e,s,TRANS_INTERACTIVE);
       }
       else {
+        // Add a semi-colon to the end of the line if one is not there.  Do this
+        // to the history as well, so the transcript can be run as regular asy
+        // code.  This also makes the history work correctly if the multiline
+        // setting is changed within an interactive session.
+        // It is added to the history at the last possible moment to avoid
+        // tampering with other features, such as using a slash to extend a
+        // line.
+        terminateLastHistoryLine();
         istring i(terminateLine(line), "-");
         i.run(e,s,TRANS_INTERACTIVE);
       }
@@ -744,15 +763,13 @@ class iprompt : public icore {
       vm::indebugger=false;
     } catch(interrupted&) {
       // Turn off the interrupted flag.
-      if (em)
-        em->Interrupt(false);
+      em.Interrupt(false);
       cout << endl;
+    } catch(quit&) {
     }
 
     // Ignore errors from this line when trying to run subsequent lines.
-    em->clear();
-
-    purge(); // Close any files that have gone out of scope.
+    em.clear();
   }
 
   void runStartCode(coenv &e, istack &s) {
@@ -780,9 +797,6 @@ public:
       // Read a line from the prompt.
       string line=getline(false);
 
-      // Copied from the old interactive prompt.  Needs to be moved.
-      ShipoutNumber=0;
-
       // Check if it is a special command.
       if (handleCommand(line))
         continue;
@@ -805,7 +819,10 @@ public:
 	restart=false;
 	icore::process();
       } catch(interrupted&) {
-	if(em) em->Interrupt(false);
+	em.Interrupt(false);
+	restart=true;
+      } catch(eof&) {
+	restart=false;
       }
     } while(restart);
       
@@ -816,8 +833,8 @@ public:
 void processCode(absyntax::block *code) {
   icode(code).process();
 }
-void processFile(const string& filename) {
-  ifile(filename).process();
+void processFile(const string& filename, bool purge) {
+  ifile(filename).process(purge);
 }
 void processPrompt() {
   iprompt().process();
@@ -826,8 +843,8 @@ void processPrompt() {
 void runCode(absyntax::block *code) {
   icode(code).doRun();
 }
-void runString(const string& s) {
-  istring(s).doRun();
+void runString(const string& s, bool interactiveWrite) {
+  istring(s).doRun(interactiveWrite ? TRANS_INTERACTIVE : TRANS_NORMAL);
 }
 void runFile(const string& filename) {
   ifile(filename).doRun();
