@@ -68,9 +68,15 @@ psfile::~psfile()
   close();
 }
   
+void psfile::header()
+{
+  Int level=settings::getSetting<Int>("level");
+  *out << "%!PS-Adobe-" << level << ".0 EPSF-" << level << ".0" << newl;
+}
+  
 void psfile::prologue(const bbox& box)
 {
-  *out << "%!PS-Adobe-3.0 EPSF-3.0" << newl;
+  header();
   BoundingBox(box);
   *out << "%%Creator: " << settings::PROGRAM << " " << settings::VERSION
        <<  newl;
@@ -86,6 +92,9 @@ void psfile::prologue(const bbox& box)
 
   *out << "%%Pages: 1" << newl;
   *out << "%%Page: 1 1" << newl;
+  
+  if(!pdfformat)
+    *out << "/Setlinewidth {0 exch dtransform dup abs 1 lt {pop 0}{round} ifelse idtransform setlinewidth pop} bind def" << newl;
 }
 
 void psfile::epilogue()
@@ -119,7 +128,6 @@ void psfile::setcolor(const pen& p, const string& begin="",
 void psfile::setpen(pen p)
 {
   p.convert();
-  if(p == lastpen) return;
     
   if(pdfformat) {
     if(p.blend() != lastpen.blend()) 
@@ -133,9 +141,10 @@ void psfile::setpen(pen p)
     *out << p.fillpattern() << " setpattern" << newl;
   else setcolor(p);
   
+  // Defer dynamic linewidth until stroke time in case currentmatrix changes.
   if(p.width() != lastpen.width()) {
-    *out << " 0 " << p.width() << 
-      " dtransform truncate idtransform setlinewidth pop" << newl;
+    *out << p.width() << (pdfformat ? " setlinewidth" : " Setlinewidth") 
+	 << newl;
   }
     
   if(p.cap() != lastpen.cap()) {
@@ -147,8 +156,10 @@ void psfile::setpen(pen p)
   }
     
   if(p.stroke() != lastpen.stroke()) {
+    out->setf(std::ios::fixed);
     *out << "[" << p.stroke() << "] " << std::setprecision(6) 
 	 << p.linetype().offset << " setdash" << newl;
+    out->unsetf(std::ios::fixed);
   }
     
   lastpen=p;
@@ -167,41 +178,42 @@ void psfile::write(pen p)
   
 void psfile::write(path p, bool newPath)
 {
-  int n = p.size();
+  Int n = p.size();
   assert(n != 0);
 
   if(newPath) newpath();
 
   if (n == 1) {
-    moveto(p.point(0));
-    lineto(p.point(0));
+    moveto(p.point((Int) 0));
+    lineto(p.point((Int) 0));
     return;
   }
 
   // Draw points
-  moveto(p.point(0));
-  for (int i = 1; i < n; i++) {
+  moveto(p.point((Int) 0));
+  for (Int i = 1; i < n; i++) {
     if(p.straight(i-1)) lineto(p.point(i));
     else curveto(p.postcontrol(i-1), p.precontrol(i), p.point(i));
   }
 
   if (p.cyclic()) {
-    if(p.straight(n-1)) lineto(p.point(0));
-    else curveto(p.postcontrol(n-1), p.precontrol(0), p.point(0));
+    if(p.straight(n-1)) lineto(p.point((Int) 0));
+    else curveto(p.postcontrol(n-1), p.precontrol((Int) 0), p.point((Int) 0));
     closepath();
   }    
 }
 
 static const char *inconsistent="inconsistent colorspaces";
   
-void psfile::latticeshade(array *a, const bbox& b)
+void psfile::latticeshade(const vm::array& a, const bbox& b)
 {
-  size_t n=a->size();
+  checkLevel();
+  size_t n=a.size();
   if(n == 0) return;
   
   array *a0=read<array *>(a,0);
   size_t m=a0->size();
-  setfirstpen(a0);
+  setfirstpen(*a0);
   
   ColorSpace colorspace=maxcolorspace2(a);
   checkColorSpace(colorspace);
@@ -253,6 +265,7 @@ void psfile::gradientshade(bool axial, const ColorSpace &colorspace,
 			   const pen& pena, const pair& a, double ra,
 			   const pen& penb, const pair& b, double rb)
 {
+  checkLevel();
   setpen(pena);
   checkColorSpace(colorspace);
   
@@ -280,9 +293,11 @@ void psfile::gradientshade(bool axial, const ColorSpace &colorspace,
        << "shfill" << newl;
 }
   
-void psfile::gouraudshade(array *pens, array *vertices, array *edges)
+void psfile::gouraudshade(const array& pens, const array& vertices,
+			  const array& edges)
 {
-  size_t size=pens->size();
+  checkLevel();
+  size_t size=pens.size();
   if(size == 0) return;
   
   setfirstpen(pens);
@@ -292,7 +307,7 @@ void psfile::gouraudshade(array *pens, array *vertices, array *edges)
        << "/ColorSpace /Device" << ColorDeviceSuffix[colorspace] << newl
        << "/DataSource [" << newl;
   for(size_t i=0; i < size; i++) {
-    write(read<int>(edges,i));
+    write(read<Int>(edges,i));
     write(read<pair>(vertices,i));
     pen *p=read<pen *>(pens,i);
     p->convert();
@@ -308,16 +323,18 @@ void psfile::gouraudshade(array *pens, array *vertices, array *edges)
 }
  
 // Tensor-product patch shading
-void psfile::tensorshade(array *pens, array *boundaries, array *z)
+void psfile::tensorshade(const array& pens, const array& boundaries,
+			 const array& z)
 {
-  size_t size=pens->size();
+  checkLevel();
+  size_t size=pens.size();
   if(size == 0) return;
-  size_t nz=z->size();
+  size_t nz=z.size();
   
   array *p0=read<array *>(pens,0);
   if(checkArray(p0) != 4)
     reportError("4 pens required");
-  setfirstpen(p0);
+  setfirstpen(*p0);
   
   ColorSpace colorspace=maxcolorspace2(pens);
   checkColorSpace(colorspace);
@@ -334,14 +351,14 @@ void psfile::tensorshade(array *pens, array *boundaries, array *z)
     path g=read<path>(boundaries,i);
     if(!(g.cyclic() && g.size() == 4))
       reportError("specify cyclic path of length 4");
-    for(int j=0; j < 4; ++j) {
+    for(Int j=0; j < 4; ++j) {
       write(g.point(j));
       write(g.postcontrol(j));
       write(g.precontrol(j+1));
     }
     if(nz == 0) { // Coons patch
       static double nineth=1.0/9.0;
-      for(int j=0; j < 4; ++j) {
+      for(Int j=0; j < 4; ++j) {
 	write(nineth*(-4.0*g.point(j)+6.0*(g.precontrol(j)+g.postcontrol(j))
 		      -2.0*(g.point(j-1)+g.point(j+1))
 		      +3.0*(g.precontrol(j-1)+g.postcontrol(j+1))
@@ -351,14 +368,14 @@ void psfile::tensorshade(array *pens, array *boundaries, array *z)
       array *zi=read<array *>(z,i);
       if(checkArray(zi) != 4)
 	reportError("specify 4 internal control points for each path");
-      for(int j=0; j < 4; ++j)
+      for(Int j=0; j < 4; ++j)
 	write(read<pair>(zi,j));
     }
     
     array *pi=read<array *>(pens,i);
     if(checkArray(pi) != 4)
       reportError("specify 4 pens for each path");
-    for(int j=0; j < 4; ++j) {
+    for(Int j=0; j < 4; ++j) {
       pen *p=read<pen *>(pi,j);
       p->convert();
       if(!p->promote(colorspace))
@@ -374,7 +391,7 @@ void psfile::tensorshade(array *pens, array *boundaries, array *z)
        << "shfill" << newl;
 }
  
-inline unsigned int byte(double r) // Map [0,1] to [0,255]
+inline unsigned byte(double r) // Map [0,1] to [0,255]
 {
   if(r < 0.0) r=0.0;
   else if(r > 1.0) r=1.0;
@@ -383,7 +400,7 @@ inline unsigned int byte(double r) // Map [0,1] to [0,255]
   return a;
 }
 
-void psfile::writeHex(pen *p, int ncomponents) 
+void psfile::writeHex(pen *p, Int ncomponents) 
 {
   switch(ncomponents) {
   case 0:
@@ -431,10 +448,10 @@ void psfile::imageheader(double width, double height, ColorSpace colorspace)
        << "image" << newl;
 }
 
-void psfile::image(array *a, array *P)
+void psfile::image(const array& a, const array& P)
 {
-  size_t asize=a->size();
-  size_t Psize=P->size();
+  size_t asize=a.size();
+  size_t Psize=P.size();
   if(asize == 0 || Psize == 0) return;
   
   array *a0=read<array *>(a,0);
@@ -479,16 +496,16 @@ void psfile::image(array *a, array *P)
   *out << ">" << endl;
 }
 
-void psfile::image(array *a)
+void psfile::image(const array& a)
 {
-  size_t asize=a->size();
+  size_t asize=a.size();
   if(asize == 0) return;
   
   array *a0=read<array *>(a,0);
   size_t a0size=a0->size();
   if(a0size == 0) return;
   
-  setfirstpen(a0);
+  setfirstpen(*a0);
   
   ColorSpace colorspace=maxcolorspace2(a);
   checkColorSpace(colorspace);
