@@ -33,7 +33,7 @@ class psfile {
   string filename;
   bool pdfformat; // Is final output format PDF?
   bool pdf;       // Output direct PDF?
-  std::stack<pen> pens;
+  mem::stack<pen> pens;
 
   void write(transform t) {
     if(!pdf) *out << "[";
@@ -43,7 +43,7 @@ class psfile {
     if(!pdf) *out << "]";
   }
 
-  void writeHex(unsigned int n) {
+  void writeHex(unsigned n) {
     *out << std::hex << std::setw(2) << std::setfill('0') << n << std::dec;
   }
   
@@ -63,6 +63,7 @@ public:
   
   void prologue(const bbox& box);
   void epilogue();
+  void header();
 
   void close();
   
@@ -74,7 +75,7 @@ public:
     *out << " " << z.getx() << " " << z.gety();
   }
 
-  void writeHex(pen *p, int ncomponents);
+  void writeHex(pen *p, Int ncomponents);
   
   void resetpen() {
     lastpen=pen(initialpen);
@@ -141,19 +142,27 @@ public:
     }
   }
   
-  void latticeshade(vm::array *a, const bbox& b);
+  void checkLevel() {
+    int n=settings::getSetting<Int>("level");
+    if(n < 3)
+      reportError("PostScript shading requires -level 3");
+  }
+  
+  void latticeshade(const vm::array& a, const bbox& b);
   
   void gradientshade(bool axial, const ColorSpace& colorspace,
 		     const pen& pena, const pair& a, double ra,
 		     const pen& penb, const pair& b, double rb);
   
-  void gouraudshade(vm::array *pens, vm::array *vertices, vm::array *edges);
-  void tensorshade(vm::array *pens, vm::array *boundaries, vm::array *z);
+  void gouraudshade(const vm::array& pens, const vm::array& vertices,
+		    const vm::array& edges);
+  void tensorshade(const vm::array& pens, const vm::array& boundaries,
+		   const vm::array& z);
 
   void imageheader(double width, double height, ColorSpace colorspace);
   
-  void image(vm::array *a, vm::array *p);
-  void image(vm::array *a);
+  void image(const vm::array& a, const vm::array& p);
+  void image(const vm::array& a);
 
   void gsave(bool tex=false) {
     if(pdf) *out << "q";
@@ -165,7 +174,7 @@ public:
   void grestore(bool tex=false) {
     if(pens.size() < 1)
       reportError("grestore without matching gsave");
-    lastpen = pens.top();
+    lastpen=pens.top();
     pens.pop();
     if(pdf) *out << "Q";
     else *out << "grestore";
@@ -173,17 +182,18 @@ public:
   }
 
   void translate(pair z) {
-   if(pdf) *out << " 1 0 0 1 " << newl;
-   write(z);
-   if(pdf) *out << " cm" << newl;
+    if(pdf) *out << " 1 0 0 1 " << newl;
+    write(z);
+    if(pdf) *out << " cm" << newl;
     *out << " translate" << newl;
   }
 
   // Multiply on a transform to the transformation matrix.
   void concat(transform t) {
-   write(t);
-   if(pdf) *out << " cm" << newl;
-   else *out << " concat" << newl;
+    if(t.isIdentity()) return;
+    write(t);
+    if(pdf) *out << " cm" << newl;
+    else *out << " concat" << newl;
   }
   
   void verbatimline(const string& s) {
@@ -195,31 +205,31 @@ public:
   }
 
   // Determine shading and image transparency based on first pen.
-  void setfirstpen(vm::array *pens) {
-    if(pens->size() > 0) {
+  void setfirstpen(const vm::array& pens) {
+    if(pens.size() > 0) {
       pen *p=vm::read<pen *>(pens,0);
       setpen(*p);
     }
   }
   
-  ColorSpace maxcolorspace(vm::array *pens) {
-    int colorspace=0;
-    size_t size=pens->size();
+  ColorSpace maxcolorspace(const vm::array& pens) {
+    ColorSpace colorspace=DEFCOLOR;
+    size_t size=pens.size();
     for(size_t i=0; i < size; i++) {
       pen *p=vm::read<pen *>(pens,i);
       p->convert();
-      colorspace=max(colorspace,(int) p->colorspace());
+      colorspace=max(colorspace,p->colorspace());
     }
-    return (ColorSpace) colorspace;
+    return colorspace;
   }
   
-  ColorSpace maxcolorspace2(vm::array *penarray) {
-    int colorspace=0;
-    size_t size=penarray->size();
+  ColorSpace maxcolorspace2(const vm::array& penarray) {
+    ColorSpace colorspace=DEFCOLOR;
+    size_t size=penarray.size();
     for(size_t i=0; i < size; i++)
       colorspace=max(colorspace,
-		     (int) maxcolorspace(vm::read<vm::array*>(penarray,i)));
-    return (ColorSpace) colorspace;
+		     maxcolorspace(vm::read<vm::array>(penarray,i)));
+    return colorspace;
   }
 
 };
