@@ -10,6 +10,8 @@
 #include "runtime.h"
 #include "coder.h"
 
+
+
 namespace types {
 
 record::record(symbol *name, frame *level)
@@ -61,7 +63,7 @@ dummyRecord::dummyRecord(string s)
 }
 
 void dummyRecord::add(string name, ty *t, trans::access *a) {
-  e.addVar(symbol::trans(name), new trans::varEntry(t, a, this)); 
+  e.addVar(symbol::trans(name), new trans::varEntry(t, a, this, position())); 
 }
 
 void dummyRecord::add(string name, function *t, vm::bltin f) {
