@@ -12,10 +12,9 @@
 #include <sys/stat.h>
 #include <cfloat>
 #include <locale.h>
+#include <unistd.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "common.h"
 
 #if HAVE_GNU_GETOPT_H
 #include <getopt.h>
@@ -25,7 +24,6 @@
 
 #include "util.h"
 #include "settings.h"
-#include "pen.h"
 #include "interact.h"
 #include "locate.h"
 #include "lexical.h"
@@ -36,22 +34,25 @@
 
 #ifdef HAVE_LIBCURSES
 extern "C" {
+
+#ifdef HAVE_NCURSES_CURSES_H
+#include <ncurses/curses.h>
+#include <ncurses/term.h>
+#elif HAVE_NCURSES_H
+#include <ncurses.h>
+#include <term.h>
+#elif HAVE_CURSES_H
 #include <curses.h>
+#include <term.h>
+#endif
+}
+#endif
 
 // Workaround broken curses.h files:
 #ifdef clear
 #undef clear
 #endif
 
-// Workaround missing symbolic link under CYGWIN
-#if defined(__CYGWIN__) && defined(HAVE_LIBNCURSES)
-#include <ncurses/term.h>
-#else
-#include <term.h>
-#endif
-}
-#endif
-
 using vm::item;
 
 using trans::itemRefAccess;
@@ -150,15 +151,15 @@ string getEntry(const string& key)
   
 void queryRegistry()
 {
-  string gs=getEntry("AFPL Ghostscript/*/GS_DLL");
+  string gs=getEntry("GPL Ghostscript/*/GS_DLL");
   if(gs.empty())
-    gs=getEntry("GPL Ghostscript/*/GS_DLL");
+    gs=getEntry("AFPL Ghostscript/*/GS_DLL");
   defaultGhostscript=stripFile(gs)+defaultGhostscript;
   defaultPDFViewer=getEntry("Adobe/Acrobat Reader/*/InstallPath/@")+"\\"+
     defaultPDFViewer;
   defaultPSViewer=getEntry("Ghostgum/GSview/*")+"\\gsview\\"+defaultPSViewer;
   defaultPython=getEntry("Python/PythonCore/*/InstallPath/@")+defaultPython;
-  asyInstallDir=getEntry("Microsoft/Windows/CurrentVersion/Uninstall/Asymptote/InstallLocation");
+  asyInstallDir=getEntry("Microsoft/Windows/CurrentVersion/App Paths/Asymptote/Path");
   defaultXasy=asyInstallDir+"\\"+defaultXasy;
 }
   
@@ -172,7 +173,7 @@ const char BUGREPORT[]=PACKAGE_BUGREPORT;
 char *argv0;
 
 // The verbosity setting, a global variable.
-int verbose;
+Int verbose;
   
 // Colorspace conversion flags (stored in global variables for efficiency). 
 bool gray;
@@ -186,9 +187,7 @@ bool safe=true;
 bool globaloption=false;
 bool globaloutname=false;
   
-bool global() {return globaloption || !safe;}
-  
-int ShipoutNumber=0;
+bool globalwrite() {return globaloption || !safe;}
   
 const string suffix="asy";
 const string guisuffix="gui";
@@ -196,9 +195,6 @@ const string guisuffix="gui";
 string initdir;
 string historyname;
 
-camp::pen *initialdefaultpen=NULL;
-camp::pen defaultpen=camp::pen::startupdefaultpen();
-  
 // Local versions of the argument list.
 int argCount = 0;
 char **argList = 0;
@@ -215,6 +211,7 @@ types::record *getSettingsModule() {
 class option;
 typedef mem::map<CONST string, option *> optionsMap_t;
 optionsMap_t optionsMap;
+
 typedef mem::map<CONST char, option *> codeMap_t;
 codeMap_t codeMap;
   
@@ -287,7 +284,7 @@ struct option : public gc {
   virtual void describe() {
     // Don't show the option if it has no desciption.
     if (!desc.empty()) {
-      const unsigned int WIDTH=22;
+      const unsigned WIDTH=22;
       string start=describeStart();
       cerr << std::left << std::setw(WIDTH) << start;
       if (start.size() >= WIDTH) {
@@ -476,7 +473,7 @@ struct stringOutnameSetting : public argumentSetting {
 
   bool getOption() {
     value=(item)(string)
-      ((globaloutname || global()) ? optarg : stripDir(optarg));
+      ((globaloutname || globalwrite()) ? optarg : stripDir(optarg));
     return true;
   }
 };
@@ -527,10 +524,10 @@ struct dataSetting : public argumentSetting {
   }
 };
 
-struct intSetting : public dataSetting<int> {
-  intSetting(string name, char code,
-	     string argname, string desc, int defaultValue=0)
-    : dataSetting<int>("an int", name, code, argname, desc,
+struct IntSetting : public dataSetting<Int> {
+  IntSetting(string name, char code,
+	     string argname, string desc, Int defaultValue=0)
+    : dataSetting<Int>("an int", name, code, argname, desc,
 		       types::primInt(), defaultValue) {}
 };
   
@@ -552,22 +549,21 @@ struct pairSetting : public dataSetting<pair> {
 struct alignSetting : public argumentSetting {
   alignSetting(string name, char code,
 	       string argname, string desc,
-	       int defaultValue=CENTER)
+	       Int defaultValue=(Int) CENTER)
     : argumentSetting(name, code, argname, desc,
 		      types::primInt(), (item)defaultValue) {}
 
   bool getOption() {
     string str=optarg;
-    if (str=="C")
-      value=(int)CENTER;
-    else if (str=="T")
-      value=(int)TOP;
-    else if (str=="B")
-      value=(int)BOTTOM;
-    else if (str=="Z") {
-      value=(int)ZERO;
-    }
-    else {
+    if (str == "C")
+      value=(Int) CENTER;
+    else if (str == "T")
+      value=(Int) TOP;
+    else if (str == "B")
+      value=(Int) BOTTOM;
+    else if (str == "Z") {
+      value=(Int) ZERO;
+    } else {
       error("invalid argument for option");
       return false;
     }
@@ -575,6 +571,16 @@ struct alignSetting : public argumentSetting {
   }
 };
 
+template<class T>
+string stringCast(T x)
+{
+  ostringstream buf;
+  buf.precision(DBL_DIG);
+  buf.setf(std::ios::boolalpha);
+  buf << x;
+  return string(buf.str());
+}
+
 template <class T>
 struct refSetting : public setting {
   T *ref;
@@ -583,8 +589,8 @@ struct refSetting : public setting {
 
   refSetting(string name, char code, string argname,
              string desc, types::ty *t, T *ref, T defaultValue,
-	     string Default, const char *text="")
-    : setting(name, code, argname, desc, t, Default),
+	     const char *text="")
+    : setting(name, code, argname, desc, t, stringCast(defaultValue)),
       ref(ref), defaultValue(defaultValue), text(text) {
     reset();
   }
@@ -609,11 +615,11 @@ struct refSetting : public setting {
 };
 
 struct boolrefSetting : public refSetting<bool> {
-  boolrefSetting(string name, char code, string desc, bool *ref)
+  boolrefSetting(string name, char code, string desc, bool *ref,
+		 bool Default=false)
     : refSetting<bool>(name, code, noarg, desc,
-		       types::primBoolean(), ref, false, "false") {}
+		       types::primBoolean(), ref, Default) {}
   bool getOption() {
-    // Increment the value.
     *ref=true;
     return true;
   }
@@ -643,10 +649,16 @@ struct boolrefSetting : public refSetting<bool> {
   }
 };
 
-struct incrementSetting : public refSetting<int> {
-  incrementSetting(string name, char code, string desc, int *ref)
-    : refSetting<int>(name, code, noarg, desc,
-		      types::primInt(), ref, 0, "0") {}
+struct boolintrefSetting : public boolrefSetting {
+  boolintrefSetting(string name, char code, string desc, int *ref,
+		    bool Default=false)
+    : boolrefSetting(name, code, desc, (bool *) ref, Default) {}
+};
+
+struct incrementSetting : public refSetting<Int> {
+  incrementSetting(string name, char code, string desc, Int *ref)
+    : refSetting<Int>(name, code, noarg, desc,
+		      types::primInt(), ref, 0) {}
 
   bool getOption() {
     // Increment the value.
@@ -680,11 +692,11 @@ struct incrementSetting : public refSetting<int> {
 };
 
 struct incrementOption : public option {
-  int *ref;
-  int level;
+  Int *ref;
+  Int level;
   
-  incrementOption(string name, char code, string desc, int *ref,
-		  int level=1)
+  incrementOption(string name, char code, string desc, Int *ref,
+		  Int level=1)
     : option(name, code, noarg, desc, true), ref(ref), level(level) {}
 
   bool getOption() {
@@ -800,13 +812,9 @@ string build_optstring() {
 c_option *build_longopts() {
   size_t n=optionsMap.size();
 
-#ifdef USEGC
-  c_option *longopts=new (UseGC) c_option[n];
-#else
-  c_option *longopts=new c_option[n];
-#endif
+  c_option *longopts=new(UseGC) c_option[n];
 
-  int i=0;
+  Int i=0;
   for (optionsMap_t::iterator p=optionsMap.begin();
        p !=optionsMap.end();
        ++p, ++i)
@@ -865,7 +873,9 @@ void getOptions(int argc, char *argv[])
 }
 
 #ifdef USEGC
-void no_GCwarn(char *, GC_word) {}
+void no_GCwarn(char *, GC_word)
+{
+}
 #endif
 
 void initSettings() {
@@ -882,20 +892,17 @@ void initSettings() {
   view->add(new boolSetting("interactiveView", 0,
 			    "View output in interactive mode", true));
   addOption(view);
-
-  addOption(new realSetting("deconstruct", 'x', "X",
-			    "Deconstruct into transparent objects magnified by X",
-			    0.0));
   addOption(new stringSetting("xformat", 0, "format", 
-			      "GUI deconstruction format [\"gif\"]","gif"));
-  addOption(new boolSetting("clearGUI", 'c', "Clear GUI operations"));
-  addOption(new boolSetting("ignoreGUI", 'i', "Ignore GUI operations"));
+			      "GUI deconstruction format [\"png\"]","png"));
   addOption(new stringSetting("outformat", 'f', "format",
 			      "Convert each output file to specified format",
 			      ""));
   addOption(new stringOutnameSetting("outname", 'o', "name",
 				     "Alternative output name for first file",
 				     ""));
+  addOption(new boolSetting("interactiveWrite", 0,
+                            "Write expressions entered at the prompt to stdout",
+                            true));
   addOption(new helpOption("help", 'h', "Show summary of options"));
   addOption(new versionOption("version", 0, "Show version"));
 
@@ -950,14 +957,24 @@ void initSettings() {
   addOption(new boolOption("unsafe", 0,
 			   "Enable system call (=> global)", &safe, false,
 			   false));
-  addOption(new boolOption("global", 0,
+  addOption(new boolOption("globalwrite", 0,
 			   "Allow write to other directory",
 			   &globaloption, true, false));
-  addOption(new boolOption("noglobal", 0,
+  addOption(new boolOption("noglobalwrite", 0,
 			   "", &globaloption, false, true));
   addOption(new stringOption("cd", 0, "directory", "Set current directory",
 			     &startpath));
   
+#ifdef USEGC  
+  addOption(new boolintrefSetting("compact", 0,
+				  "Conserve memory at the expense of speed",
+				  &GC_dont_expand));
+  addOption(new refSetting<GC_word>("divisor", 0, "n",
+			      "Free space divisor for garbage collection",
+				    types::primInt(),&GC_free_space_divisor,2,
+				    "an int"));
+#endif  
+  
   addOption(new stringSetting("prompt", 0,"string","Prompt [\"> \"]","> "));
   addOption(new stringSetting("prompt2", 0,"string",
                               "Continuation prompt for multiline input "
@@ -965,18 +982,18 @@ void initSettings() {
   addOption(new boolSetting("multiline", 0,
                             "Input code over multiple lines at the prompt"));
 
-  addOption(new boolSetting("quiet", 'q',"Suppress welcome message"));
   addOption(new boolSetting("wait", 0,
 			    "Wait for child processes to finish before exiting"));
-  // Signal parent process at completion of each shipout
-  addOption(new boolSetting("signal", 0, ""));
+  // Be interactive even in a pipe
+  addOption(new boolSetting("interactive", 0, ""));
 			    
   addOption(new boolSetting("localhistory", 0,
 			    "Use a local interactive history file"));
-  addOption(new intSetting("historylines", 0, "n",
+  addOption(new IntSetting("historylines", 0, "n",
 			   "Retain n lines of history [1000]",1000));
-  addOption(new intSetting("scroll", 0, "n",
+  addOption(new IntSetting("scroll", 0, "n",
 			   "Scroll standard output n lines at a time [0]",0));
+  addOption(new IntSetting("level", 0, "n", "Postscript level [3]",3));
   addOption(new boolSetting("autoplain", 0,
 			    "Enable automatic importing of plain",
 			    true));
@@ -985,6 +1002,8 @@ void initSettings() {
 			    false));
   addOption(new stringSetting("autoimport", 0, "string",
 			      "Module to automatically import [\"\"]", ""));
+  addOption(new userSetting("command", 'c', "string",
+			    "Command to autoexecute", ""));
   addOption(new userSetting("user", 'u', "string",
 			    "General purpose user string [\"\"]", ""));
   
@@ -992,6 +1011,7 @@ void initSettings() {
   addOption(new realSetting("paperheight", 0, "bp", ""));
   
   addOption(new stringSetting("dvipsOptions", 0, "string", "", ""));
+  addOption(new stringSetting("gsOptions", 0, "string", "", ""));
   
   addOption(new envSetting("config","config."+suffix));
   addOption(new envSetting("pdfviewer", defaultPDFViewer));
@@ -1014,9 +1034,10 @@ int numArgs() { return argCount; }
 char *getArg(int n) { return argList[n]; }
 
 void setInteractive() {
-  if(numArgs() == 0 && !getSetting<bool>("listvariables")) {
+  if(numArgs() == 0 && !getSetting<bool>("listvariables") && 
+     getSetting<string>("command").empty() &&
+     (isatty(STDIN_FILENO) || getSetting<bool>("interactive")))
     interact::interactive=true;
-  }
   
   historyname=getSetting<bool>("localhistory") ? "."+suffix+"_history" 
     : (initdir+"/history");
@@ -1050,7 +1071,6 @@ void initDir() {
 void setPath() {
   searchPath.clear();
   searchPath.push_back(".");
-  searchPath.push_back(initdir);
   string asydir=getSetting<string>("dir");
   if(asydir != "") {
     size_t p,i=0;
@@ -1060,6 +1080,7 @@ void setPath() {
     }
     if(i < asydir.length()) searchPath.push_back(asydir.substr(i));
   }
+  searchPath.push_back(initdir);
 #ifdef ASYMPTOTE_SYSDIR
   searchPath.push_back(ASYMPTOTE_SYSDIR);
 #endif
@@ -1110,19 +1131,19 @@ string defaultformat() {
 // TeX special command to set up currentmatrix for typesetting labels.
 const char *beginlabel(const string& texengine) {
   if(pdf(texengine))
-    return "\\special{pdf: q #5 0 0 cm}"
+    return "\\special{pdf:q #5 0 0 cm}"
       "\\wd\\ASYbox 0pt\\dp\\ASYbox 0pt\\ht\\ASYbox 0pt";
   else 
-    return "\\special{ps: gsave currentpoint currentpoint translate [#5 0 0] "
+    return "\\special{ps:gsave currentpoint currentpoint translate [#5 0 0] "
       "concat neg exch neg exch translate}";
 }
 
 // TeX special command to restore currentmatrix after typesetting labels.
 const char *endlabel(const string& texengine) {
   if(pdf(texengine))
-    return "\\special{pdf: Q}";
+    return "\\special{pdf:Q}";
   else
-    return "\\special{ps: currentpoint grestore moveto}";
+    return "\\special{ps:currentpoint grestore moveto}";
 }
 
 // TeX macro to typeset raw postscript code
@@ -1140,8 +1161,8 @@ const char *rawpostscript(const string& texengine) {
 // Begin TeX special command.
 const char *beginspecial(const string& texengine) {
   if(pdf(texengine))
-    return "\\special{pdf: ";
-  return "\\special{ps: ";
+    return "\\special{pdf:";
+  return "\\special{ps:";
 }
 
 // End TeX special command.
@@ -1171,9 +1192,9 @@ string texprogram()
   return (path == "") ? texengine() : (string) (path+"/"+texengine());
 }
 
-int getScroll() 
+Int getScroll() 
 {
-  int scroll=settings::getSetting<int>("scroll");
+  Int scroll=settings::getSetting<Int>("scroll");
 #ifdef HAVE_LIBCURSES  
   if(scroll < 0) {
     char *terminal=getenv("TERM");
@@ -1200,20 +1221,18 @@ void setOptions(int argc, char *argv[])
   
   // Read command-line options initially to obtain CONFIG and DIR.
   getOptions(argc,argv);
+  
   resetOptions();
   
   // Read user configuration file.
   setPath();
   doConfig(getSetting<string>("config"));
   
-  // Remember any changes to the defaultpen.
-  initialdefaultpen=new camp::pen(defaultpen);
-
   // Read command-line options again to override configuration file defaults.
   getOptions(argc,argv);
   
 #ifdef USEGC
-  if(!getSetting<bool>("debug")) GC_set_warn_proc(no_GCwarn);
+  if(verbose == 0 && !getSetting<bool>("debug")) GC_set_warn_proc(no_GCwarn);
 #endif  
 
   if(setlocale (LC_ALL, "") == NULL && getSetting<bool>("debug"))
