@@ -27,12 +27,9 @@ extern const string docdir;
   
 extern bool safe;
   
-extern bool global();
+extern bool globalwrite();
 
 enum origin {CENTER,BOTTOM,TOP,ZERO};
-//extern int origin;
-  
-extern int ShipoutNumber;
   
 extern const string suffix;
 extern const string guisuffix;
@@ -51,9 +48,9 @@ inline T getSetting(string name)
   return vm::get<T>(Setting(name));
 }
 
-extern int verbose;
+extern Int verbose;
 extern bool gray;
-extern bool bw;  
+extern bool bw;
 extern bool rgb;
 extern bool cmyk;
   
@@ -67,7 +64,7 @@ void setOptions(int argc, char *argv[]);
 int numArgs();
 char *getArg(int n);
  
-int getScroll();
+Int getScroll();
   
 bool pdf(const string& texengine);
 bool latex(const string& texengine);
