@@ -17,8 +17,6 @@ using settings::getSetting;
   
 namespace camp {
 
-mem::list<string> TeXpipepreamble, TeXpreamble;
-  
 texfile::texfile(const string& texname, const bbox& box) : box(box)
 {
   texengine=getSetting<string>("tex");
@@ -49,11 +47,11 @@ void texfile::prologue()
     std::ofstream *outpreamble=
       new std::ofstream(auxname(getSetting<string>("outname"),"pre").c_str(),
 			std::ios::app);
-    texpreamble(*outpreamble,TeXpreamble);
+    texpreamble(*outpreamble,processData().TeXpreamble);
     outpreamble->close();
   }
   
-  texdefines(*out,TeXpreamble,false);
+  texdefines(*out,processData().TeXpreamble,false);
   double width=box.right-box.left;
   double height=box.top-box.bottom;
   if(settings::pdf(texengine) && !inlinetex) {
@@ -70,9 +68,8 @@ void texfile::prologue()
       *out << "\\pdfpageheight=" << height << "bp" << newl;
   }
   if(settings::latex(texengine)) {
-    if(inlinetex)
-      *out << "\\setlength{\\unitlength}{1pt}" << newl;
-    else {
+    *out << "\\setlength{\\unitlength}{1pt}" << newl;
+    if(!inlinetex) {
       *out << "\\pagestyle{empty}" << newl
 	   << "\\textheight=" << height+18.0 << "bp" << newl
 	   << "\\textwidth=" << width+18.0 << "bp" << newl;
@@ -210,7 +207,8 @@ void texfile::put(const string& label, const transform& T, const pair& z,
        << "," << (z.gety()-box.bottom)*ps2tex
        << ")(" << align.getx()
        << "," << align.gety() 
-       << "){" << T.getxx() << " " << sign*T.getyx()
+       << "){";
+  *out << T.getxx() << " " << sign*T.getyx()
        << " " << sign*T.getxy() << " " << T.getyy()
        << "}{" << label << "}" << newl;
 }
