@@ -18,7 +18,7 @@
 
 namespace camp {
 
-class transform : virtual public gc {
+class transform : public gc {
   double x;
   double y;
   double xx;
@@ -30,6 +30,8 @@ public:
   transform()
     : x(0.0), y(0.0), xx(1.0), xy(0.0), yx(0.0), yy(1.0) {}
 
+  virtual ~transform() {}
+
   transform(double x, double y,
             double xx, double xy,
 	    double yx, double yy)
@@ -94,12 +96,18 @@ public:
     return !(t1 == t2);
   }
 
-  bool isIdentity()
+  bool isIdentity() const
   {
     return x == 0.0 && y == 0.0 &&
       xx == 1.0 && xy == 0.0 && yx == 0.0 && yy == 1.0;
   }
 
+  bool isNull() const
+  {
+    return x == 0.0 && y == 0.0 &&
+      xx == 0.0 && xy == 0.0 && yx == 0.0 && yy == 0.0;
+  }
+
   // Calculates the determinant, as if it were a matrix.
   friend double det(const transform& t)
   {
@@ -107,7 +115,7 @@ public:
   }
 
   // Tells if the transformation is invertible (bijective).
-  bool invertible()
+  bool invertible() const
   {
     return det(*this) != 0.0;
   }
@@ -136,10 +144,7 @@ public:
 };
 
 // The common transforms
-inline transform identity()
-{
-  return transform();
-}
+static const transform identity;
 
 inline transform shift(pair z)
 {
@@ -179,11 +184,11 @@ inline transform rotate(double theta)
   return transform (0.0, 0.0, c, -s, s, c);
 }
 
-// return rotate(angle(v)) if z != (0,0); otherwise return identity().
+// return rotate(angle(v)) if z != (0,0); otherwise return identity.
 inline transform rotate(pair z)
 {
   double d=z.length();
-  if(d == 0.0) return identity();
+  if(d == 0.0) return identity;
   d=1.0/d;
   return transform (0.0, 0.0, d*z.getx(), -d*z.gety(), d*z.gety(), d*z.getx());
 }
@@ -219,6 +224,12 @@ inline transform shift(transform t)
   return transform(t.getx(), t.gety(), 1.0, 0, 0, 1.0);
 } 
 
+// Return the translational component of t.
+inline pair shiftpair(transform t)
+{
+  return (t.getx(), t.gety());
+} 
+
 inline transform matrix(pair lb, pair rt)
 {
   pair size=rt-lb;
@@ -227,4 +238,6 @@ inline transform matrix(pair lb, pair rt)
 
 } //namespace camp
 
+GC_DECLARE_PTRFREE(camp::transform);
+
 #endif
