@@ -30,8 +30,8 @@ ty *primNull() { return &pNull; }
 
 array boolArray_(primBoolean());
 ty *boolArray() { return &boolArray_; }
-array intArray_(primInt());
-ty *intArray() { return &intArray_; }
+array IntArray_(primInt());
+ty *IntArray() { return &IntArray_; }
 array realArray_(primReal());
 ty *realArray() { return &realArray_; }
 array pairArray_(primPair());
@@ -51,8 +51,8 @@ ty *guideArray() { return &guideArray_; }
   
 array boolArray2_(boolArray());
 ty *boolArray2() { return &boolArray2_; }
-array intArray2_(intArray());
-ty *intArray2() { return &intArray2_; }
+array IntArray2_(IntArray());
+ty *IntArray2() { return &IntArray2_; }
 array realArray2_(realArray());
 ty *realArray2() { return &realArray2_; }
 array pairArray2_(pairArray());
@@ -66,8 +66,8 @@ ty *penArray2() { return &penArray2_; }
   
 array boolArray3_(boolArray2());
 ty *boolArray3() { return &boolArray3_; }
-array intArray3_(intArray2());
-ty *intArray3() { return &intArray3_; }
+array IntArray3_(IntArray2());
+ty *IntArray3() { return &IntArray3_; }
 array realArray3_(realArray2());
 ty *realArray3() { return &realArray3_; }
 array pairArray3_(pairArray2());
@@ -199,16 +199,20 @@ ty *array::appendType()
 
 ty *array::insertType()
 {
-  if (inserttype == 0)
-    inserttype = new function(celltype,formal(primInt(),"i"),
-			      formal(celltype,"x"));
+  if (inserttype == 0) {
+    function *f=new function(primVoid(),formal(primInt(),"i"));
+    f->addRest(this);
+    inserttype = f;
+  }
+  
   return inserttype;
 }
 
 ty *array::deleteType()
 {
   if (deletetype == 0)
-    deletetype = new function(celltype,formal(primInt(),"i"));
+    deletetype = new function(primVoid(),formal(primInt(),"i",true),
+			      formal(primInt(),"j",true));
 
   return deletetype;
 }
@@ -217,6 +221,10 @@ ty *cyclicType() {
   return new function(primVoid(),formal(primBoolean(),"b"));
 }
 
+ty *initializedType() {
+  return new function(primBoolean(),formal(primInt(),"i"));
+}
+
 ty *array::virtualFieldGetType(symbol *id)
 {
   return
@@ -226,6 +234,7 @@ ty *array::virtualFieldGetType(symbol *id)
     id == symbol::trans("append") ? appendType() : 
     id == symbol::trans("insert") ? insertType() : 
     id == symbol::trans("delete") ? deleteType() : 
+    id == symbol::trans("initialized") ? initializedType() : 
     ty::virtualFieldGetType(id);
 }
 
@@ -250,6 +259,13 @@ trans::varEntry *array::virtualField(symbol *id, signature *sig)
     static trans::varEntry v(cyclicType(), &a, 0, position());
     return &v;
   }
+  if (id == symbol::trans("initialized") &&
+      equivalent(sig, initializedType()->getSignature()))
+  {
+    static trans::bltinAccess a(run::arrayInitialized);
+    static trans::varEntry v(initializedType(), &a, 0, position());
+    return &v;
+  }
   if (id == symbol::trans("push") &&
       equivalent(sig, pushType()->getSignature()))
   {
@@ -299,7 +315,7 @@ trans::varEntry *array::virtualField(symbol *id, signature *sig)
     return ty::virtualField(id, sig);
 }
 
-std::ostream& operator<< (std::ostream& out, const formal& f)
+ostream& operator<< (ostream& out, const formal& f)
 {
   if (f.Explicit)
     out << "explicit ";
@@ -319,8 +335,22 @@ bool equivalent(formal& f1, formal& f2) {
   return equivalent(f1.t,f2.t);
 }
 
+bool argumentEquivalent(formal &f1, formal& f2) {
+  if (f1.name == f2.name) {
+    if (f1.t == 0)
+      return f2.t == 0;
+    else if (f2.t == 0)
+      return false;
 
-std::ostream& operator<< (std::ostream& out, const signature& s)
+    return f1.t->kind != ty_overloaded &&
+           f2.t->kind != ty_overloaded &&
+           equivalent(f1.t, f2.t);
+  }
+  else
+    return false;
+}
+
+ostream& operator<< (ostream& out, const signature& s)
 {
   out << "(";
 
@@ -361,6 +391,22 @@ bool equivalent(signature *s1, signature *s2)
          equivalent(s1->rest, s2->rest);
 }
 
+bool argumentEquivalent(signature *s1, signature *s2)
+{
+  // Handle null signature
+  if (s1 == 0)
+    return s2 == 0;
+  else if (s2 == 0)
+    return false;
+
+  if (s1->formals.size() != s2->formals.size())
+    return false;
+
+  return std::equal(s1->formals.begin(),s1->formals.end(),s2->formals.begin(),
+                    (bool (*)(formal&,formal&)) argumentEquivalent) &&
+         argumentEquivalent(s1->rest, s2->rest);
+}
+
 size_t signature::hash() {
   size_t x=2038;
   for (formal_vector::iterator i=formals.begin(); i!=formals.end(); ++i)
@@ -382,8 +428,8 @@ ty *function::stripDefaults()
 {
   function *f = new function(result);
 
-  int n = sig.getNumFormals();
-  for (int i = 0; i < n; ++i)
+  Int n = sig.getNumFormals();
+  for (Int i = 0; i < n; ++i)
     f->add(sig.getFormal(i), 0);
 
   return f;
