@@ -13,7 +13,6 @@
 #include <iostream>
 #include <cstdio>
 #include <cassert>
-#include <vector>
 
 #include "errormsg.h"
 #include "symbol.h"
@@ -228,7 +227,7 @@ struct array : public ty {
     return 1007 * celltype->hash();
   }
 
-  int depth() {
+  Int depth() {
     if (array *cell=dynamic_cast<array *>(celltype))
       return cell->depth() + 1;
     else
@@ -264,7 +263,7 @@ struct array : public ty {
 ty *primNull();
   
 ty *boolArray();
-ty *intArray();
+ty *IntArray();
 ty *realArray();
 ty *pairArray();
 ty *tripleArray();
@@ -275,7 +274,7 @@ ty *penArray();
 ty *guideArray();
 
 ty *boolArray2();
-ty *intArray2();
+ty *IntArray2();
 ty *realArray2();
 ty *pairArray2();
 ty *tripleArray2();
@@ -283,7 +282,7 @@ ty *stringArray2();
 ty *penArray2();
 
 ty *boolArray3();
-ty *intArray3();
+ty *IntArray3();
 ty *realArray3();
 ty *pairArray3();
 ty *tripleArray3();
@@ -313,6 +312,7 @@ struct formal {
 };
 
 bool equivalent(formal& f1, formal& f2);
+bool argumentEquivalent(formal &f1, formal& f2);
 
 typedef mem::vector<formal> formal_vector;
 
@@ -329,6 +329,9 @@ struct signature : public gc {
     : rest(0)
     {}
 
+  signature(signature &sig)
+    : formals(sig.formals), rest(sig.rest) {}
+
   virtual ~signature() {}
 
   void add(formal f) {
@@ -358,9 +361,14 @@ struct signature : public gc {
   friend ostream& operator<< (ostream& out, const signature& s);
 
   friend bool equivalent(signature *s1, signature *s2);
+
+  // Check if a signature of argument types (as opposed to formal parameters)
+  // are equivalent.  Here, the arguments, if named, must have the same names,
+  // and (for simplicity) no overloaded arguments are allowed.
+  friend bool argumentEquivalent(signature *s1, signature *s2);
 #if 0
   friend bool castable(signature *target, signature *source);
-  friend int numFormalsMatch(signature *s1, signature *s2);
+  friend Int numFormalsMatch(signature *s1, signature *s2);
 #endif
 
   size_t hash();
@@ -372,6 +380,8 @@ struct function : public ty {
 
   function(ty *result)
     : ty(ty_function), result(result) {}
+  function(ty *result, signature *sig)
+    : ty(ty_function), result(result), sig(*sig) {}
   function(ty *result, formal f1)
     : ty(ty_function), result(result) {
     add(f1);
@@ -578,7 +588,9 @@ public:
   }
 };
 
-
 } // namespace types
 
+GC_DECLARE_PTRFREE(types::primitiveTy);
+GC_DECLARE_PTRFREE(types::nullTy);
+
 #endif
