@@ -18,6 +18,7 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <signal.h>
+#include <cstring>
 
 #include "util.h"
 #include "settings.h"
@@ -29,6 +30,34 @@ using namespace settings;
 
 bool False=false;
 
+namespace vm {
+  void error(const char* message);
+}
+
+char *Strdup(string s)
+{
+  size_t size=s.size()+1;
+  char *dest=new(UseGC) char[size];
+  std::memcpy(dest,s.c_str(),size*sizeof(char));
+  return dest;
+}
+
+char *StrdupNoGC(string s)
+{
+  size_t size=s.size()+1;
+  char *dest=new char[size];
+  std::memcpy(dest,s.c_str(),size*sizeof(char));
+  return dest;
+}
+
+char *StrdupMalloc(string s)
+{
+  size_t size=s.size()+1;
+  char *dest=(char *) std::malloc(size);
+  std::memcpy(dest,s.c_str(),size*sizeof(char));
+  return dest;
+}
+
 string stripDir(string name)
 {
   size_t p;
@@ -88,12 +117,12 @@ string Getenv(const char *name, bool msdos)
 
 void writeDisabled()
 {
-  camp::reportError("Write/cd to other directories disabled; override with option -global");
+  camp::reportError("Write/cd to other directories disabled; override with option -globalwrite");
 }
 
 void checkLocal(string name)
 {
-  if(global()) return;
+  if(globalwrite()) return;
 #ifdef __CYGWIN__  
   if(name.rfind('\\') < string::npos) writeDisabled();
 #endif  
@@ -136,7 +165,7 @@ char **args(const char *command)
       if(!quote && c == ' ') {
 	if(!empty) {
 	  if(pass) {
-	    argv[n]=strcpy(new char[buf.str().size()+1],buf.str().c_str());
+	    argv[n]=StrdupNoGC(buf.str());
 	    buf.str("");
 	  }
 	  empty=true;
@@ -149,7 +178,7 @@ char **args(const char *command)
       }
     }
     if(!empty) {
-      if(pass) argv[n]=strcpy(new char[buf.str().size()+1],buf.str().c_str());
+      if(pass) argv[n]=StrdupNoGC(buf.str());
       n++;
     }
   }
@@ -181,7 +210,6 @@ void execError(const char *command, const char *hint, const char *application)
 	   << endl << "or use the command line option -" << hint 
 	   << "=\"PATH\"" << endl;
     }
-    exit(-1);
 }
 						    
 // quiet: 0=none; 1=suppress stdout; 2=suppress stdout+stderr.
@@ -215,6 +243,7 @@ int System(const char *command, int quiet, bool wait,
     if(argv) {
       execvp(argv[0],argv);
       execError(argv[0],hint,application);
+      exit(-1);
     }
   }
 
@@ -281,16 +310,15 @@ void noPath()
   camp::reportError("Cannot get current path");
 }
 
-char *getPath(char *p)
+char *getPath(char *p=currentpath)
 {
-  static int size=MAXPATHLEN;
-  if(!p) p=new char[size];
+  static size_t size=MAXPATHLEN;
+  if(!p) p=new(UseGC) char[size];
   if(!p) noPath();
   else while(getcwd(p,size) == NULL) {
     if(errno == ERANGE) {
       size *= 2;
-      delete [] p;
-      p=new char[size];
+      p=new(UseGC) char[size];
     } else {noPath(); p=NULL;}
   }
   return p;
@@ -307,12 +335,8 @@ const char *setPath(const char *s, bool quiet)
     camp::reportError(buf);
   }
   char *p=getPath();
-  if(p) {
-    if(interact::interactive && !quiet)
-      cout << p << endl;
-    else if(verbose > 1)
-      cout << "cd " << p << endl;
-  }
+  if(p && (!interact::interactive || quiet) && verbose > 1)
+    cout << "cd " << p << endl;
   return p;
 }
 
@@ -333,3 +357,16 @@ void popupHelp() {
   }
 }
 
+unsigned unsignedcast(Int n)
+{
+  if(n < 0 || n/2 > INT_MAX)
+    vm::error("Unsigned integer argument is outside valid range");
+  return (unsigned) n;
+}
+
+int intcast(Int n)
+{
+  if(Abs(n) > INT_MAX)
+  vm::error("Integer argument is outside valid range");
+  return (int) n;
+}
