@@ -15,6 +15,11 @@
 
 #include <strings.h>
 
+// Duplicate a string.
+char *Strdup(string s);
+char *StrdupNoGC(string s);
+char *StrdupMalloc(string s);
+  
 // Strip the directory from a filename.
 string stripDir(string name);
   
@@ -26,7 +31,7 @@ string stripExt(string name, const string& suffix="");
   
 void writeDisabled();
   
-// Check if global writes disabled and name contains a directory.
+// Check if global writes are disabled and name contains a directory.
 void checkLocal(string name);
   
 // Construct a filename from the original, adding aux at the end, and
@@ -48,8 +53,8 @@ int System(const ostringstream& command, int quiet=0, bool wait=true,
 	   int *pid=NULL); 
   
 #if defined(__DECCXX_LIBCXX_RH70)
-extern "C" int kill(pid_t pid, int sig) throw();
-extern "C" char *strsignal(int sig);
+extern "C" int kill(pid_t pid, Int sig) throw();
+extern "C" char *strsignal(Int sig);
 extern "C" double asinh(double x);
 extern "C" double acosh(double x);
 extern "C" double atanh(double x);
@@ -59,8 +64,8 @@ extern "C" double erfc(double x);
 extern "C" double tgamma(double x);
 extern "C" double remainder(double x, double y);
 extern "C" double hypot(double x, double y) throw();
-extern "C" double jn(int n, double x);
-extern "C" double yn(int n, double x);
+extern "C" double jn(Int n, double x);
+extern "C" double yn(Int n, double x);
 #endif
 
 #if defined(__mips)
@@ -73,6 +78,10 @@ extern "C" int fileno(FILE *);
 extern "C" char *strptime(const char *s, const char *format, struct tm *tm);
 #endif
 
+#if defined(__CYGWIN__)
+#define ARG_MAX _POSIX_ARG_MAX
+#endif
+
 extern bool False;
 
 // Strip blank lines (which would break the bidirectional TeX pipe)
@@ -81,7 +90,6 @@ string stripblanklines(const string& s);
 extern char *currentpath;
 
 const char *startPath();
-char *getPath(char *p=currentpath);
 const char* setPath(const char *s, bool quiet=false);
 const char *changeDirectory(const char *s);
 extern char *startpath;
@@ -95,4 +103,20 @@ void execError(const char *command, const char *hint, const char *application);
 // This invokes a viewer to display the manual.  Subsequent calls will only
 // pop-up a new viewer if the old one has been closed.
 void popupHelp();
+
+inline Int Abs(Int x) {
+#ifdef HAVE_LONG_LONG
+  return llabs(x);
+#else
+#ifdef HAVE_LONG
+  return labs(x);
+#else
+  return abs(x);
+#endif
+#endif
+}
+
+unsigned unsignedcast(Int n);
+int intcast(Int n);
+  
 #endif
